@@ -15,7 +15,7 @@ use serde_json::ser::PrettyFormatter;
 
 use crate::build_tools::py_schema_error_type;
 use crate::errors::LocItem;
-use crate::get_version;
+use crate::get_pydantic_version;
 use crate::serializers::{SerMode, SerializationState};
 use crate::tools::{safe_repr, SchemaDict};
 
@@ -111,9 +111,18 @@ fn include_url_env(py: Python) -> bool {
 
 static URL_PREFIX: GILOnceCell<String> = GILOnceCell::new();
 
+fn get_formated_url(py: Python) -> &'static str {
+    let pydantic_version = match get_pydantic_version(py) {
+        // include major and minor version only
+        Some(value) => value.split('.').collect::<Vec<&str>>()[..2].join("."),
+        None => "latest".to_string(),
+    };
+    URL_PREFIX.get_or_init(py, || format!("https://errors.pydantic.dev/{pydantic_version}/v/"))
+}
+
 fn get_url_prefix(py: Python, include_url: bool) -> Option<&str> {
     if include_url {
-        Some(URL_PREFIX.get_or_init(py, || format!("https://errors.pydantic.dev/{}/v/", get_version())))
+        Some(get_formated_url(py))
     } else {
         None
     }
@@ -230,14 +239,44 @@ impl ValidationError {
     }
 }
 
+// TODO: is_utf8_char_boundary, floor_char_boundary and ceil_char_boundary
+// with builtin methods once https://github.com/rust-lang/rust/issues/93743 is resolved
+// These are just copy pasted from the current implementation
+const fn is_utf8_char_boundary(value: u8) -> bool {
+    // This is bit magic equivalent to: b < 128 || b >= 192
+    (value as i8) >= -0x40
+}
+
+fn floor_char_boundary(value: &str, index: usize) -> usize {
+    if index >= value.len() {
+        value.len()
+    } else {
+        let lower_bound = index.saturating_sub(3);
+        let new_index = value.as_bytes()[lower_bound..=index]
+            .iter()
+            .rposition(|b| is_utf8_char_boundary(*b));
+
+        // SAFETY: we know that the character boundary will be within four bytes
+        unsafe { lower_bound + new_index.unwrap_unchecked() }
+    }
+}
+
+pub fn ceil_char_boundary(value: &str, index: usize) -> usize {
+    let upper_bound = Ord::min(index + 4, value.len());
+    value.as_bytes()[index..upper_bound]
+        .iter()
+        .position(|b| is_utf8_char_boundary(*b))
+        .map_or(upper_bound, |pos| pos + index)
+}
+
 macro_rules! truncate_input_value {
     ($out:expr, $value:expr) => {
         if $value.len() > 50 {
             write!(
                 $out,
                 ", input_value={}...{}",
-                &$value[0..25],
-                &$value[$value.len() - 24..]
+                &$value[0..floor_char_boundary($value, 25)],
+                &$value[ceil_char_boundary($value, $value.len() - 24)..]
             )?;
         } else {
             write!($out, ", input_value={}", $value)?;
@@ -305,7 +344,7 @@ impl TryFrom<&PyAny> for PyLineError {
             let context: Option<&PyDict> = dict.get_as(intern!(py, "ctx"))?;
             ErrorType::new(py, type_str.to_str()?, context)?
         } else if let Ok(custom_error) = type_raw.extract::<PydanticCustomError>() {
-            ErrorType::new_custom_error(custom_error)
+            ErrorType::new_custom_error(py, custom_error)
         } else {
             return Err(PyTypeError::new_err(
                 "`type` should be a `str` or `PydanticCustomError`",
@@ -351,7 +390,7 @@ impl PyLineError {
         }
         if let Some(url_prefix) = url_prefix {
             match self.error_type {
-                ErrorType::CustomError { custom_error: _ } => {
+                ErrorType::CustomError { .. } => {
                     // Don't add URLs for custom errors
                 }
                 _ => {
@@ -381,7 +420,7 @@ impl PyLineError {
         if !hide_input {
             let input_value = self.input_value.as_ref(py);
             let input_str = safe_repr(input_value);
-            truncate_input_value!(output, input_str);
+            truncate_input_value!(output, &input_str);
 
             if let Ok(type_) = input_value.get_type().name() {
                 write!(output, ", input_type={type_}")?;
@@ -389,7 +428,7 @@ impl PyLineError {
         }
         if let Some(url_prefix) = url_prefix {
             match self.error_type {
-                ErrorType::CustomError { custom_error: _ } => {
+                ErrorType::CustomError { .. } => {
                     // Don't display URLs for custom errors
                     output.push(']');
                 }
