@@ -1,13 +1,17 @@
 use pyo3::intern;
 use pyo3::prelude::*;
+
+use pyo3::exceptions::PyValueError;
 use pyo3::types::{PyDate, PyDateTime, PyDelta, PyDeltaAccess, PyDict, PyTime, PyTzInfo};
-use speedate::{Date, DateTime, Duration, ParseError, Time};
+use speedate::MicrosecondsPrecisionOverflowBehavior;
+use speedate::{Date, DateTime, Duration, ParseError, Time, TimeConfig};
 use std::borrow::Cow;
+
 use strum::EnumMessage;
 
-use crate::errors::{ErrorType, ValError, ValResult};
-
 use super::Input;
+use crate::errors::{ErrorType, ValError, ValResult};
+use crate::tools::py_err;
 
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub enum EitherDate<'a> {
@@ -74,7 +78,8 @@ impl<'a> From<&'a PyTime> for EitherTime<'a> {
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub enum EitherTimedelta<'a> {
     Raw(Duration),
-    Py(&'a PyDelta),
+    PyExact(&'a PyDelta),
+    PySubclass(&'a PyDelta),
 }
 
 impl<'a> From<Duration> for EitherTimedelta<'a> {
@@ -83,13 +88,38 @@ impl<'a> From<Duration> for EitherTimedelta<'a> {
     }
 }
 
-impl<'a> From<&'a PyDelta> for EitherTimedelta<'a> {
-    fn from(timedelta: &'a PyDelta) -> Self {
-        Self::Py(timedelta)
+impl<'a> EitherTimedelta<'a> {
+    pub fn to_duration(&self) -> PyResult<Duration> {
+        match self {
+            Self::Raw(timedelta) => Ok(timedelta.clone()),
+            Self::PyExact(py_timedelta) => Ok(pytimedelta_exact_as_duration(py_timedelta)),
+            Self::PySubclass(py_timedelta) => pytimedelta_subclass_as_duration(py_timedelta),
+        }
+    }
+
+    pub fn try_into_py(&self, py: Python<'a>) -> PyResult<&'a PyDelta> {
+        match self {
+            Self::PyExact(timedelta) => Ok(*timedelta),
+            Self::PySubclass(timedelta) => Ok(*timedelta),
+            Self::Raw(duration) => duration_as_pytimedelta(py, duration),
+        }
     }
 }
 
-pub fn pytimedelta_as_duration(py_timedelta: &PyDelta) -> Duration {
+impl<'a> TryFrom<&'a PyAny> for EitherTimedelta<'a> {
+    type Error = PyErr;
+
+    fn try_from(value: &'a PyAny) -> PyResult<Self> {
+        if let Ok(dt) = <PyDelta as PyTryFrom>::try_from_exact(value) {
+            Ok(EitherTimedelta::PyExact(dt))
+        } else {
+            let dt = value.downcast::<PyDelta>()?;
+            Ok(EitherTimedelta::PySubclass(dt))
+        }
+    }
+}
+
+pub fn pytimedelta_exact_as_duration(py_timedelta: &PyDelta) -> Duration {
     // see https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_DELTA_GET_DAYS
     // days can be negative, but seconds and microseconds are always positive.
     let mut days = py_timedelta.get_days(); // -999999999 to 999999999
@@ -112,30 +142,31 @@ pub fn pytimedelta_as_duration(py_timedelta: &PyDelta) -> Duration {
     Duration::new(positive, days as u32, seconds as u32, microseconds as u32).unwrap()
 }
 
-impl<'a> EitherTimedelta<'a> {
-    pub fn as_raw(&self) -> Duration {
-        match self {
-            Self::Raw(timedelta) => timedelta.clone(),
-            Self::Py(py_timedelta) => pytimedelta_as_duration(py_timedelta),
-        }
+pub fn pytimedelta_subclass_as_duration(py_timedelta: &PyDelta) -> PyResult<Duration> {
+    let total_seconds: f64 = py_timedelta
+        .call_method0(intern!(py_timedelta.py(), "total_seconds"))?
+        .extract()?;
+    if total_seconds.is_nan() {
+        return py_err!(PyValueError; "NaN values not permitted");
     }
+    let positive = total_seconds >= 0_f64;
+    let total_seconds = total_seconds.abs();
+    let microsecond = total_seconds.fract() * 1_000_000.0;
+    let days = (total_seconds / 86400f64) as u32;
+    let seconds = total_seconds as u64 % 86400;
+    Duration::new(positive, days, seconds as u32, microsecond.round() as u32)
+        .map_err(|err| PyValueError::new_err(err.to_string()))
+}
 
-    pub fn try_into_py(self, py: Python<'_>) -> PyResult<PyObject> {
-        let timedelta = match self {
-            Self::Py(timedelta) => Ok(timedelta),
-            Self::Raw(duration) => {
-                let sign = if duration.positive { 1 } else { -1 };
-                PyDelta::new(
-                    py,
-                    sign * duration.day as i32,
-                    sign * duration.second as i32,
-                    sign * duration.microsecond as i32,
-                    true,
-                )
-            }
-        }?;
-        Ok(timedelta.into_py(py))
-    }
+pub fn duration_as_pytimedelta<'py>(py: Python<'py>, duration: &Duration) -> PyResult<&'py PyDelta> {
+    let sign = if duration.positive { 1 } else { -1 };
+    PyDelta::new(
+        py,
+        sign * duration.day as i32,
+        sign * duration.second as i32,
+        sign * duration.microsecond as i32,
+        true,
+    )
 }
 
 pub fn pytime_as_time(py_time: &PyAny, py_dt: Option<&PyAny>) -> PyResult<Time> {
@@ -257,30 +288,53 @@ pub fn bytes_as_date<'a>(input: &'a impl Input<'a>, bytes: &[u8]) -> ValResult<'
         Err(err) => Err(ValError::new(
             ErrorType::DateParsing {
                 error: Cow::Borrowed(err.get_documentation().unwrap_or_default()),
+                context: None,
             },
             input,
         )),
     }
 }
 
-pub fn bytes_as_time<'a>(input: &'a impl Input<'a>, bytes: &[u8]) -> ValResult<'a, EitherTime<'a>> {
-    match Time::parse_bytes(bytes) {
+pub fn bytes_as_time<'a>(
+    input: &'a impl Input<'a>,
+    bytes: &[u8],
+    microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+) -> ValResult<'a, EitherTime<'a>> {
+    match Time::parse_bytes_with_config(
+        bytes,
+        &TimeConfig {
+            microseconds_precision_overflow_behavior: microseconds_overflow_behavior,
+            unix_timestamp_offset: Some(0),
+        },
+    ) {
         Ok(date) => Ok(date.into()),
         Err(err) => Err(ValError::new(
             ErrorType::TimeParsing {
                 error: Cow::Borrowed(err.get_documentation().unwrap_or_default()),
+                context: None,
             },
             input,
         )),
     }
 }
 
-pub fn bytes_as_datetime<'a, 'b>(input: &'a impl Input<'a>, bytes: &'b [u8]) -> ValResult<'a, EitherDateTime<'a>> {
-    match DateTime::parse_bytes(bytes) {
+pub fn bytes_as_datetime<'a, 'b>(
+    input: &'a impl Input<'a>,
+    bytes: &'b [u8],
+    microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+) -> ValResult<'a, EitherDateTime<'a>> {
+    match DateTime::parse_bytes_with_config(
+        bytes,
+        &TimeConfig {
+            microseconds_precision_overflow_behavior: microseconds_overflow_behavior,
+            unix_timestamp_offset: Some(0),
+        },
+    ) {
         Ok(dt) => Ok(dt.into()),
         Err(err) => Err(ValError::new(
             ErrorType::DatetimeParsing {
                 error: Cow::Borrowed(err.get_documentation().unwrap_or_default()),
+                context: None,
             },
             input,
         )),
@@ -292,11 +346,19 @@ pub fn int_as_datetime<'a>(
     timestamp: i64,
     timestamp_microseconds: u32,
 ) -> ValResult<EitherDateTime> {
-    match DateTime::from_timestamp(timestamp, timestamp_microseconds) {
+    match DateTime::from_timestamp_with_config(
+        timestamp,
+        timestamp_microseconds,
+        &TimeConfig {
+            unix_timestamp_offset: Some(0),
+            ..Default::default()
+        },
+    ) {
         Ok(dt) => Ok(dt.into()),
         Err(err) => Err(ValError::new(
             ErrorType::DatetimeParsing {
                 error: Cow::Borrowed(err.get_documentation().unwrap_or_default()),
+                context: None,
             },
             input,
         )),
@@ -309,6 +371,7 @@ macro_rules! nan_check {
             return Err(ValError::new(
                 ErrorType::$error_type {
                     error: Cow::Borrowed("NaN values not permitted"),
+                    context: None,
                 },
                 $input,
             ));
@@ -352,6 +415,7 @@ pub fn int_as_time<'a>(
             return Err(ValError::new(
                 ErrorType::TimeParsing {
                     error: Cow::Borrowed("time in seconds should be positive"),
+                    context: None,
                 },
                 input,
             ));
@@ -361,11 +425,19 @@ pub fn int_as_time<'a>(
         // ok
         t => t as u32,
     };
-    match Time::from_timestamp(time_timestamp, timestamp_microseconds) {
+    match Time::from_timestamp_with_config(
+        time_timestamp,
+        timestamp_microseconds,
+        &TimeConfig {
+            unix_timestamp_offset: Some(0),
+            ..Default::default()
+        },
+    ) {
         Ok(dt) => Ok(dt.into()),
         Err(err) => Err(ValError::new(
             ErrorType::TimeParsing {
                 error: Cow::Borrowed(err.get_documentation().unwrap_or_default()),
+                context: None,
             },
             input,
         )),
@@ -383,13 +455,24 @@ fn map_timedelta_err<'a>(input: &'a impl Input<'a>, err: ParseError) -> ValError
     ValError::new(
         ErrorType::TimeDeltaParsing {
             error: Cow::Borrowed(err.get_documentation().unwrap_or_default()),
+            context: None,
         },
         input,
     )
 }
 
-pub fn bytes_as_timedelta<'a, 'b>(input: &'a impl Input<'a>, bytes: &'b [u8]) -> ValResult<'a, EitherTimedelta<'a>> {
-    match Duration::parse_bytes(bytes) {
+pub fn bytes_as_timedelta<'a, 'b>(
+    input: &'a impl Input<'a>,
+    bytes: &'b [u8],
+    microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+) -> ValResult<'a, EitherTimedelta<'a>> {
+    match Duration::parse_bytes_with_config(
+        bytes,
+        &TimeConfig {
+            microseconds_precision_overflow_behavior: microseconds_overflow_behavior,
+            unix_timestamp_offset: Some(0),
+        },
+    ) {
         Ok(dt) => Ok(dt.into()),
         Err(err) => Err(map_timedelta_err(input, err)),
     }
@@ -418,7 +501,7 @@ pub fn float_as_duration<'a>(input: &'a impl Input<'a>, total_seconds: f64) -> V
 #[pyclass(module = "pydantic_core._pydantic_core", extends = PyTzInfo)]
 #[derive(Clone)]
 #[cfg_attr(debug_assertions, derive(Debug))]
-struct TzInfo {
+pub struct TzInfo {
     seconds: i32,
 }
 
@@ -457,4 +540,10 @@ impl TzInfo {
     fn __deepcopy__(&self, py: Python, _memo: &PyDict) -> PyResult<Py<Self>> {
         Py::new(py, self.clone())
     }
+
+    pub fn __reduce__(&self, py: Python) -> PyResult<PyObject> {
+        let args = (self.seconds,);
+        let cls = Py::new(py, self.clone())?.getattr(py, "__class__")?;
+        Ok((cls, args).into_py(py))
+    }
 }
