@@ -1,14 +1,19 @@
-use pyo3::prelude::*;
-use pyo3::types::PyDict;
+use std::borrow::Cow;
 
-use crate::errors::{ErrorType, InputValue, LocItem, ValError, ValResult};
+use pyo3::prelude::*;
+use pyo3::types::{PyDict, PyString, PyType};
+use speedate::MicrosecondsPrecisionOverflowBehavior;
+use strum::EnumMessage;
+
+use crate::errors::{ErrorType, ErrorTypeDefaults, InputValue, LocItem, ValError, ValResult};
+use crate::validators::decimal::create_decimal;
 
 use super::datetime::{
     bytes_as_date, bytes_as_datetime, bytes_as_time, bytes_as_timedelta, float_as_datetime, float_as_duration,
     float_as_time, int_as_datetime, int_as_duration, int_as_time, EitherDate, EitherDateTime, EitherTime,
 };
 use super::parse_json::JsonArray;
-use super::shared::{float_as_int, int_as_bool, map_json_err, str_as_bool, str_as_int};
+use super::shared::{float_as_int, int_as_bool, map_json_err, str_as_bool, str_as_float, str_as_int};
 use super::{
     EitherBytes, EitherFloat, EitherInt, EitherString, EitherTimedelta, GenericArguments, GenericIterable,
     GenericIterator, GenericMapping, Input, JsonArgs, JsonInput,
@@ -50,7 +55,7 @@ impl<'a> Input<'a> for JsonInput {
         match self {
             JsonInput::Object(object) => Ok(JsonArgs::new(None, Some(object)).into()),
             JsonInput::Array(array) => Ok(JsonArgs::new(Some(array), None).into()),
-            _ => Err(ValError::new(ErrorType::ArgumentsType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::ArgumentsType, self)),
         }
     }
 
@@ -59,7 +64,13 @@ impl<'a> Input<'a> for JsonInput {
             JsonInput::Object(object) => Ok(JsonArgs::new(None, Some(object)).into()),
             _ => {
                 let class_name = class_name.to_string();
-                Err(ValError::new(ErrorType::DataclassType { class_name }, self))
+                Err(ValError::new(
+                    ErrorType::DataclassType {
+                        class_name,
+                        context: None,
+                    },
+                    self,
+                ))
             }
         }
     }
@@ -67,27 +78,27 @@ impl<'a> Input<'a> for JsonInput {
     fn parse_json(&'a self) -> ValResult<'a, JsonInput> {
         match self {
             JsonInput::String(s) => serde_json::from_str(s.as_str()).map_err(|e| map_json_err(self, e)),
-            _ => Err(ValError::new(ErrorType::JsonType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::JsonType, self)),
         }
     }
 
     fn strict_str(&'a self) -> ValResult<EitherString<'a>> {
         match self {
             JsonInput::String(s) => Ok(s.as_str().into()),
-            _ => Err(ValError::new(ErrorType::StringType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::StringType, self)),
         }
     }
     fn lax_str(&'a self) -> ValResult<EitherString<'a>> {
         match self {
             JsonInput::String(s) => Ok(s.as_str().into()),
-            _ => Err(ValError::new(ErrorType::StringType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::StringType, self)),
         }
     }
 
     fn validate_bytes(&'a self, _strict: bool) -> ValResult<EitherBytes<'a>> {
         match self {
             JsonInput::String(s) => Ok(s.as_bytes().into()),
-            _ => Err(ValError::new(ErrorType::BytesType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::BytesType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
@@ -98,7 +109,7 @@ impl<'a> Input<'a> for JsonInput {
     fn strict_bool(&self) -> ValResult<bool> {
         match self {
             JsonInput::Bool(b) => Ok(*b),
-            _ => Err(ValError::new(ErrorType::BoolType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::BoolType, self)),
         }
     }
     fn lax_bool(&self) -> ValResult<bool> {
@@ -107,10 +118,12 @@ impl<'a> Input<'a> for JsonInput {
             JsonInput::String(s) => str_as_bool(self, s),
             JsonInput::Int(int) => int_as_bool(self, *int),
             JsonInput::Float(float) => match float_as_int(self, *float) {
-                Ok(int) => int.as_bool().ok_or_else(|| ValError::new(ErrorType::BoolParsing, self)),
-                _ => Err(ValError::new(ErrorType::BoolType, self)),
+                Ok(int) => int
+                    .as_bool()
+                    .ok_or_else(|| ValError::new(ErrorTypeDefaults::BoolParsing, self)),
+                _ => Err(ValError::new(ErrorTypeDefaults::BoolType, self)),
             },
-            _ => Err(ValError::new(ErrorType::BoolType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::BoolType, self)),
         }
     }
 
@@ -118,7 +131,8 @@ impl<'a> Input<'a> for JsonInput {
         match self {
             JsonInput::Int(i) => Ok(EitherInt::I64(*i)),
             JsonInput::Uint(u) => Ok(EitherInt::U64(*u)),
-            _ => Err(ValError::new(ErrorType::IntType, self)),
+            JsonInput::BigInt(b) => Ok(EitherInt::BigInt(b.clone())),
+            _ => Err(ValError::new(ErrorTypeDefaults::IntType, self)),
         }
     }
     fn lax_int(&'a self) -> ValResult<EitherInt<'a>> {
@@ -129,16 +143,17 @@ impl<'a> Input<'a> for JsonInput {
             },
             JsonInput::Int(i) => Ok(EitherInt::I64(*i)),
             JsonInput::Uint(u) => Ok(EitherInt::U64(*u)),
+            JsonInput::BigInt(b) => Ok(EitherInt::BigInt(b.clone())),
             JsonInput::Float(f) => float_as_int(self, *f),
             JsonInput::String(str) => str_as_int(self, str),
-            _ => Err(ValError::new(ErrorType::IntType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::IntType, self)),
         }
     }
 
     fn ultra_strict_float(&'a self) -> ValResult<EitherFloat<'a>> {
         match self {
             JsonInput::Float(f) => Ok(EitherFloat::F64(*f)),
-            _ => Err(ValError::new(ErrorType::FloatType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::FloatType, self)),
         }
     }
     fn strict_float(&'a self) -> ValResult<EitherFloat<'a>> {
@@ -146,7 +161,7 @@ impl<'a> Input<'a> for JsonInput {
             JsonInput::Float(f) => Ok(EitherFloat::F64(*f)),
             JsonInput::Int(i) => Ok(EitherFloat::F64(*i as f64)),
             JsonInput::Uint(u) => Ok(EitherFloat::F64(*u as f64)),
-            _ => Err(ValError::new(ErrorType::FloatType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::FloatType, self)),
         }
     }
     fn lax_float(&'a self) -> ValResult<EitherFloat<'a>> {
@@ -158,18 +173,27 @@ impl<'a> Input<'a> for JsonInput {
             JsonInput::Float(f) => Ok(EitherFloat::F64(*f)),
             JsonInput::Int(i) => Ok(EitherFloat::F64(*i as f64)),
             JsonInput::Uint(u) => Ok(EitherFloat::F64(*u as f64)),
-            JsonInput::String(str) => match str.parse::<f64>() {
-                Ok(i) => Ok(EitherFloat::F64(i)),
-                Err(_) => Err(ValError::new(ErrorType::FloatParsing, self)),
-            },
-            _ => Err(ValError::new(ErrorType::FloatType, self)),
+            JsonInput::String(str) => str_as_float(self, str),
+            _ => Err(ValError::new(ErrorTypeDefaults::FloatType, self)),
+        }
+    }
+
+    fn strict_decimal(&'a self, decimal_type: &'a PyType) -> ValResult<&'a PyAny> {
+        let py = decimal_type.py();
+        match self {
+            JsonInput::Float(f) => create_decimal(PyString::new(py, &f.to_string()), self, decimal_type),
+
+            JsonInput::String(..) | JsonInput::Int(..) | JsonInput::Uint(..) | JsonInput::BigInt(..) => {
+                create_decimal(self.to_object(py).into_ref(py), self, decimal_type)
+            }
+            _ => Err(ValError::new(ErrorTypeDefaults::DecimalType, self)),
         }
     }
 
     fn validate_dict(&'a self, _strict: bool) -> ValResult<GenericMapping<'a>> {
         match self {
             JsonInput::Object(dict) => Ok(dict.into()),
-            _ => Err(ValError::new(ErrorType::DictType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::DictType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
@@ -180,7 +204,7 @@ impl<'a> Input<'a> for JsonInput {
     fn validate_list(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
         match self {
             JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
-            _ => Err(ValError::new(ErrorType::ListType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::ListType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
@@ -192,7 +216,7 @@ impl<'a> Input<'a> for JsonInput {
         // just as in set's case, List has to be allowed
         match self {
             JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
-            _ => Err(ValError::new(ErrorType::TupleType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::TupleType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
@@ -204,7 +228,7 @@ impl<'a> Input<'a> for JsonInput {
         // we allow a list here since otherwise it would be impossible to create a set from JSON
         match self {
             JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
-            _ => Err(ValError::new(ErrorType::SetType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::SetType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
@@ -216,7 +240,7 @@ impl<'a> Input<'a> for JsonInput {
         // we allow a list here since otherwise it would be impossible to create a frozenset from JSON
         match self {
             JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
-            _ => Err(ValError::new(ErrorType::FrozenSetType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::FrozenSetType, self)),
         }
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
@@ -229,7 +253,7 @@ impl<'a> Input<'a> for JsonInput {
             JsonInput::Array(a) => Ok(GenericIterable::JsonArray(a)),
             JsonInput::String(s) => Ok(GenericIterable::JsonString(s)),
             JsonInput::Object(object) => Ok(GenericIterable::JsonObject(object)),
-            _ => Err(ValError::new(ErrorType::IterableType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::IterableType, self)),
         }
     }
 
@@ -242,14 +266,14 @@ impl<'a> Input<'a> for JsonInput {
                 let keys: Vec<JsonInput> = object.keys().map(|k| JsonInput::String(k.clone())).collect();
                 Ok(keys.into())
             }
-            _ => Err(ValError::new(ErrorType::IterableType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::IterableType, self)),
         }
     }
 
     fn validate_date(&self, _strict: bool) -> ValResult<EitherDate> {
         match self {
             JsonInput::String(v) => bytes_as_date(self, v.as_bytes()),
-            _ => Err(ValError::new(ErrorType::DateType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::DateType, self)),
         }
     }
     // NO custom `lax_date` implementation, if strict_date fails, the validator will fallback to lax_datetime
@@ -259,48 +283,74 @@ impl<'a> Input<'a> for JsonInput {
         self.validate_date(false)
     }
 
-    fn strict_time(&self) -> ValResult<EitherTime> {
+    fn strict_time(
+        &self,
+        microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTime> {
         match self {
-            JsonInput::String(v) => bytes_as_time(self, v.as_bytes()),
-            _ => Err(ValError::new(ErrorType::TimeType, self)),
+            JsonInput::String(v) => bytes_as_time(self, v.as_bytes(), microseconds_overflow_behavior),
+            _ => Err(ValError::new(ErrorTypeDefaults::TimeType, self)),
         }
     }
-    fn lax_time(&self) -> ValResult<EitherTime> {
+    fn lax_time(&self, microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior) -> ValResult<EitherTime> {
         match self {
-            JsonInput::String(v) => bytes_as_time(self, v.as_bytes()),
+            JsonInput::String(v) => bytes_as_time(self, v.as_bytes(), microseconds_overflow_behavior),
             JsonInput::Int(v) => int_as_time(self, *v, 0),
             JsonInput::Float(v) => float_as_time(self, *v),
-            _ => Err(ValError::new(ErrorType::TimeType, self)),
+            JsonInput::BigInt(_) => Err(ValError::new(
+                ErrorType::TimeParsing {
+                    error: Cow::Borrowed(
+                        speedate::ParseError::TimeTooLarge
+                            .get_documentation()
+                            .unwrap_or_default(),
+                    ),
+                    context: None,
+                },
+                self,
+            )),
+            _ => Err(ValError::new(ErrorTypeDefaults::TimeType, self)),
         }
     }
 
-    fn strict_datetime(&self) -> ValResult<EitherDateTime> {
+    fn strict_datetime(
+        &self,
+        microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherDateTime> {
         match self {
-            JsonInput::String(v) => bytes_as_datetime(self, v.as_bytes()),
-            _ => Err(ValError::new(ErrorType::DatetimeType, self)),
+            JsonInput::String(v) => bytes_as_datetime(self, v.as_bytes(), microseconds_overflow_behavior),
+            _ => Err(ValError::new(ErrorTypeDefaults::DatetimeType, self)),
         }
     }
-    fn lax_datetime(&self) -> ValResult<EitherDateTime> {
+    fn lax_datetime(
+        &self,
+        microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherDateTime> {
         match self {
-            JsonInput::String(v) => bytes_as_datetime(self, v.as_bytes()),
+            JsonInput::String(v) => bytes_as_datetime(self, v.as_bytes(), microseconds_overflow_behavior),
             JsonInput::Int(v) => int_as_datetime(self, *v, 0),
             JsonInput::Float(v) => float_as_datetime(self, *v),
-            _ => Err(ValError::new(ErrorType::DatetimeType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::DatetimeType, self)),
         }
     }
 
-    fn strict_timedelta(&self) -> ValResult<EitherTimedelta> {
+    fn strict_timedelta(
+        &self,
+        microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTimedelta> {
         match self {
-            JsonInput::String(v) => bytes_as_timedelta(self, v.as_bytes()),
-            _ => Err(ValError::new(ErrorType::TimeDeltaType, self)),
+            JsonInput::String(v) => bytes_as_timedelta(self, v.as_bytes(), microseconds_overflow_behavior),
+            _ => Err(ValError::new(ErrorTypeDefaults::TimeDeltaType, self)),
         }
     }
-    fn lax_timedelta(&self) -> ValResult<EitherTimedelta> {
+    fn lax_timedelta(
+        &self,
+        microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTimedelta> {
         match self {
-            JsonInput::String(v) => bytes_as_timedelta(self, v.as_bytes()),
+            JsonInput::String(v) => bytes_as_timedelta(self, v.as_bytes(), microseconds_overflow_behavior),
             JsonInput::Int(v) => Ok(int_as_duration(self, *v)?.into()),
             JsonInput::Float(v) => Ok(float_as_duration(self, *v)?.into()),
-            _ => Err(ValError::new(ErrorType::TimeDeltaType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::TimeDeltaType, self)),
         }
     }
 }
@@ -326,13 +376,19 @@ impl<'a> Input<'a> for String {
 
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn validate_args(&'a self) -> ValResult<'a, GenericArguments<'a>> {
-        Err(ValError::new(ErrorType::ArgumentsType, self))
+        Err(ValError::new(ErrorTypeDefaults::ArgumentsType, self))
     }
 
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn validate_dataclass_args(&'a self, class_name: &str) -> ValResult<'a, GenericArguments<'a>> {
         let class_name = class_name.to_string();
-        Err(ValError::new(ErrorType::DataclassType { class_name }, self))
+        Err(ValError::new(
+            ErrorType::DataclassType {
+                class_name,
+                context: None,
+            },
+            self,
+        ))
     }
 
     fn parse_json(&'a self) -> ValResult<'a, JsonInput> {
@@ -355,19 +411,19 @@ impl<'a> Input<'a> for String {
     }
 
     fn strict_bool(&self) -> ValResult<bool> {
-        Err(ValError::new(ErrorType::BoolType, self))
+        Err(ValError::new(ErrorTypeDefaults::BoolType, self))
     }
     fn lax_bool(&self) -> ValResult<bool> {
         str_as_bool(self, self)
     }
 
     fn strict_int(&'a self) -> ValResult<EitherInt<'a>> {
-        Err(ValError::new(ErrorType::IntType, self))
+        Err(ValError::new(ErrorTypeDefaults::IntType, self))
     }
     fn lax_int(&'a self) -> ValResult<EitherInt<'a>> {
         match self.parse() {
             Ok(i) => Ok(EitherInt::I64(i)),
-            Err(_) => Err(ValError::new(ErrorType::IntParsing, self)),
+            Err(_) => Err(ValError::new(ErrorTypeDefaults::IntParsing, self)),
         }
     }
 
@@ -377,18 +433,20 @@ impl<'a> Input<'a> for String {
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn strict_float(&'a self) -> ValResult<EitherFloat<'a>> {
-        Err(ValError::new(ErrorType::FloatType, self))
+        Err(ValError::new(ErrorTypeDefaults::FloatType, self))
     }
     fn lax_float(&'a self) -> ValResult<EitherFloat<'a>> {
-        match self.parse() {
-            Ok(f) => Ok(EitherFloat::F64(f)),
-            Err(_) => Err(ValError::new(ErrorType::FloatParsing, self)),
-        }
+        str_as_float(self, self)
+    }
+
+    fn strict_decimal(&'a self, decimal_type: &'a PyType) -> ValResult<&'a PyAny> {
+        let py = decimal_type.py();
+        create_decimal(self.to_object(py).into_ref(py), self, decimal_type)
     }
 
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn validate_dict(&'a self, _strict: bool) -> ValResult<GenericMapping<'a>> {
-        Err(ValError::new(ErrorType::DictType, self))
+        Err(ValError::new(ErrorTypeDefaults::DictType, self))
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn strict_dict(&'a self) -> ValResult<GenericMapping<'a>> {
@@ -397,7 +455,7 @@ impl<'a> Input<'a> for String {
 
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn validate_list(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
-        Err(ValError::new(ErrorType::ListType, self))
+        Err(ValError::new(ErrorTypeDefaults::ListType, self))
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn strict_list(&'a self) -> ValResult<GenericIterable<'a>> {
@@ -406,7 +464,7 @@ impl<'a> Input<'a> for String {
 
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn validate_tuple(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
-        Err(ValError::new(ErrorType::TupleType, self))
+        Err(ValError::new(ErrorTypeDefaults::TupleType, self))
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn strict_tuple(&'a self) -> ValResult<GenericIterable<'a>> {
@@ -415,7 +473,7 @@ impl<'a> Input<'a> for String {
 
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn validate_set(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
-        Err(ValError::new(ErrorType::SetType, self))
+        Err(ValError::new(ErrorTypeDefaults::SetType, self))
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn strict_set(&'a self) -> ValResult<GenericIterable<'a>> {
@@ -424,7 +482,7 @@ impl<'a> Input<'a> for String {
 
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn validate_frozenset(&'a self, _strict: bool) -> ValResult<GenericIterable<'a>> {
-        Err(ValError::new(ErrorType::FrozenSetType, self))
+        Err(ValError::new(ErrorTypeDefaults::FrozenSetType, self))
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
     fn strict_frozenset(&'a self) -> ValResult<GenericIterable<'a>> {
@@ -447,28 +505,49 @@ impl<'a> Input<'a> for String {
         self.validate_date(false)
     }
 
-    fn validate_time(&self, _strict: bool) -> ValResult<EitherTime> {
-        bytes_as_time(self, self.as_bytes())
+    fn validate_time(
+        &self,
+        _strict: bool,
+        microseconds_overflow_behavior: speedate::MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTime> {
+        bytes_as_time(self, self.as_bytes(), microseconds_overflow_behavior)
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_time(&self) -> ValResult<EitherTime> {
-        self.validate_time(false)
+    fn strict_time(
+        &self,
+        microseconds_overflow_behavior: speedate::MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTime> {
+        self.validate_time(false, microseconds_overflow_behavior)
     }
 
-    fn validate_datetime(&self, _strict: bool) -> ValResult<EitherDateTime> {
-        bytes_as_datetime(self, self.as_bytes())
+    fn validate_datetime(
+        &self,
+        _strict: bool,
+        microseconds_overflow_behavior: speedate::MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherDateTime> {
+        bytes_as_datetime(self, self.as_bytes(), microseconds_overflow_behavior)
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_datetime(&self) -> ValResult<EitherDateTime> {
-        self.validate_datetime(false)
+    fn strict_datetime(
+        &self,
+        microseconds_overflow_behavior: speedate::MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherDateTime> {
+        self.validate_datetime(false, microseconds_overflow_behavior)
     }
 
-    fn validate_timedelta(&self, _strict: bool) -> ValResult<EitherTimedelta> {
-        bytes_as_timedelta(self, self.as_bytes())
+    fn validate_timedelta(
+        &self,
+        _strict: bool,
+        microseconds_overflow_behavior: speedate::MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTimedelta> {
+        bytes_as_timedelta(self, self.as_bytes(), microseconds_overflow_behavior)
     }
     #[cfg_attr(has_no_coverage, no_coverage)]
-    fn strict_timedelta(&self) -> ValResult<EitherTimedelta> {
-        self.validate_timedelta(false)
+    fn strict_timedelta(
+        &self,
+        microseconds_overflow_behavior: speedate::MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTimedelta> {
+        self.validate_timedelta(false, microseconds_overflow_behavior)
     }
 }
 
