@@ -3,15 +3,17 @@ use std::str::from_utf8;
 
 use pyo3::prelude::*;
 use pyo3::types::{
-    PyBool, PyByteArray, PyBytes, PyDate, PyDateTime, PyDelta, PyDict, PyFloat, PyFrozenSet, PyInt, PyIterator, PyList,
+    PyBool, PyByteArray, PyBytes, PyDate, PyDateTime, PyDict, PyFloat, PyFrozenSet, PyInt, PyIterator, PyList,
     PyMapping, PySequence, PySet, PyString, PyTime, PyTuple, PyType,
 };
 #[cfg(not(PyPy))]
 use pyo3::types::{PyDictItems, PyDictKeys, PyDictValues};
 use pyo3::{intern, AsPyPointer, PyTypeInfo};
+use speedate::MicrosecondsPrecisionOverflowBehavior;
 
-use crate::errors::{ErrorType, InputValue, LocItem, ValError, ValResult};
+use crate::errors::{ErrorType, ErrorTypeDefaults, InputValue, LocItem, ValError, ValResult};
 use crate::tools::{extract_i64, safe_repr};
+use crate::validators::decimal::create_decimal;
 use crate::{ArgsKwargs, PyMultiHostUrl, PyUrl};
 
 use super::datetime::{
@@ -19,7 +21,7 @@ use super::datetime::{
     float_as_duration, float_as_time, int_as_datetime, int_as_duration, int_as_time, EitherDate, EitherDateTime,
     EitherTime,
 };
-use super::shared::{float_as_int, int_as_bool, map_json_err, str_as_bool, str_as_int};
+use super::shared::{float_as_int, int_as_bool, map_json_err, str_as_bool, str_as_float, str_as_int};
 use super::{
     py_string_str, EitherBytes, EitherFloat, EitherInt, EitherString, EitherTimedelta, GenericArguments,
     GenericIterable, GenericIterator, GenericMapping, Input, JsonInput, PyArgs,
@@ -155,7 +157,7 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(list) = self.downcast::<PyList>() {
             Ok(PyArgs::new(Some(list.to_tuple()), None).into())
         } else {
-            Err(ValError::new(ErrorType::ArgumentsType, self))
+            Err(ValError::new(ErrorTypeDefaults::ArgumentsType, self))
         }
     }
 
@@ -168,7 +170,13 @@ impl<'a> Input<'a> for PyAny {
             Ok(PyArgs::new(Some(args), kwargs).into())
         } else {
             let class_name = class_name.to_string();
-            Err(ValError::new(ErrorType::DataclassType { class_name }, self))
+            Err(ValError::new(
+                ErrorType::DataclassType {
+                    class_name,
+                    context: None,
+                },
+                self,
+            ))
         }
     }
 
@@ -181,15 +189,19 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(py_byte_array) = self.downcast::<PyByteArray>() {
             serde_json::from_slice(unsafe { py_byte_array.as_bytes() }).map_err(|e| map_json_err(self, e))
         } else {
-            Err(ValError::new(ErrorType::JsonType, self))
+            Err(ValError::new(ErrorTypeDefaults::JsonType, self))
         }
     }
 
     fn strict_str(&'a self) -> ValResult<EitherString<'a>> {
-        if let Ok(py_str) = <PyString as PyTryFrom>::try_from(self) {
+        if let Ok(py_str) = <PyString as PyTryFrom>::try_from_exact(self) {
             Ok(py_str.into())
+        } else if let Ok(py_str) = self.downcast::<PyString>() {
+            // force to a rust string to make sure behavior is consistent whether or not we go via a
+            // rust string in StrConstrainedValidator - e.g. to_lower
+            Ok(py_string_str(py_str)?.into())
         } else {
-            Err(ValError::new(ErrorType::StringType, self))
+            Err(ValError::new(ErrorTypeDefaults::StringType, self))
         }
     }
 
@@ -197,7 +209,7 @@ impl<'a> Input<'a> for PyAny {
         if let Ok(py_str) = <PyString as PyTryFrom>::try_from_exact(self) {
             Ok(EitherString::Py(py_str))
         } else {
-            Err(ValError::new(ErrorType::IntType, self))
+            Err(ValError::new(ErrorTypeDefaults::IntType, self))
         }
     }
 
@@ -205,7 +217,7 @@ impl<'a> Input<'a> for PyAny {
         if PyInt::is_exact_type_of(self) {
             Ok(EitherInt::Py(self))
         } else {
-            Err(ValError::new(ErrorType::IntType, self))
+            Err(ValError::new(ErrorTypeDefaults::IntType, self))
         }
     }
 
@@ -219,7 +231,7 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(bytes) = self.downcast::<PyBytes>() {
             let str = match from_utf8(bytes.as_bytes()) {
                 Ok(s) => s,
-                Err(_) => return Err(ValError::new(ErrorType::StringUnicode, self)),
+                Err(_) => return Err(ValError::new(ErrorTypeDefaults::StringUnicode, self)),
             };
             Ok(str.into())
         } else if let Ok(py_byte_array) = self.downcast::<PyByteArray>() {
@@ -227,11 +239,11 @@ impl<'a> Input<'a> for PyAny {
             // for why this is marked unsafe
             let str = match from_utf8(unsafe { py_byte_array.as_bytes() }) {
                 Ok(s) => s,
-                Err(_) => return Err(ValError::new(ErrorType::StringUnicode, self)),
+                Err(_) => return Err(ValError::new(ErrorTypeDefaults::StringUnicode, self)),
             };
             Ok(str.into())
         } else {
-            Err(ValError::new(ErrorType::StringType, self))
+            Err(ValError::new(ErrorTypeDefaults::StringType, self))
         }
     }
 
@@ -239,7 +251,7 @@ impl<'a> Input<'a> for PyAny {
         if let Ok(py_bytes) = self.downcast::<PyBytes>() {
             Ok(py_bytes.into())
         } else {
-            Err(ValError::new(ErrorType::BytesType, self))
+            Err(ValError::new(ErrorTypeDefaults::BytesType, self))
         }
     }
 
@@ -252,7 +264,7 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(py_byte_array) = self.downcast::<PyByteArray>() {
             Ok(py_byte_array.to_vec().into())
         } else {
-            Err(ValError::new(ErrorType::BytesType, self))
+            Err(ValError::new(ErrorTypeDefaults::BytesType, self))
         }
     }
 
@@ -260,24 +272,26 @@ impl<'a> Input<'a> for PyAny {
         if let Ok(bool) = self.downcast::<PyBool>() {
             Ok(bool.is_true())
         } else {
-            Err(ValError::new(ErrorType::BoolType, self))
+            Err(ValError::new(ErrorTypeDefaults::BoolType, self))
         }
     }
 
     fn lax_bool(&self) -> ValResult<bool> {
         if let Ok(bool) = self.downcast::<PyBool>() {
             Ok(bool.is_true())
-        } else if let Some(cow_str) = maybe_as_string(self, ErrorType::BoolParsing)? {
+        } else if let Some(cow_str) = maybe_as_string(self, ErrorTypeDefaults::BoolParsing)? {
             str_as_bool(self, &cow_str)
         } else if let Ok(int) = extract_i64(self) {
             int_as_bool(self, int)
         } else if let Ok(float) = self.extract::<f64>() {
             match float_as_int(self, float) {
-                Ok(int) => int.as_bool().ok_or_else(|| ValError::new(ErrorType::BoolParsing, self)),
-                _ => Err(ValError::new(ErrorType::BoolType, self)),
+                Ok(int) => int
+                    .as_bool()
+                    .ok_or_else(|| ValError::new(ErrorTypeDefaults::BoolParsing, self)),
+                _ => Err(ValError::new(ErrorTypeDefaults::BoolType, self)),
             }
         } else {
-            Err(ValError::new(ErrorType::BoolType, self))
+            Err(ValError::new(ErrorTypeDefaults::BoolType, self))
         }
     }
 
@@ -287,63 +301,101 @@ impl<'a> Input<'a> for PyAny {
         } else if PyInt::is_type_of(self) {
             // bools are a subclass of int, so check for bool type in this specific case
             if PyBool::is_exact_type_of(self) {
-                Err(ValError::new(ErrorType::IntType, self))
+                Err(ValError::new(ErrorTypeDefaults::IntType, self))
             } else {
                 Ok(EitherInt::Py(self))
             }
         } else {
-            Err(ValError::new(ErrorType::IntType, self))
+            Err(ValError::new(ErrorTypeDefaults::IntType, self))
         }
     }
 
     fn lax_int(&'a self) -> ValResult<EitherInt<'a>> {
         if PyInt::is_exact_type_of(self) {
             Ok(EitherInt::Py(self))
-        } else if let Some(cow_str) = maybe_as_string(self, ErrorType::IntParsing)? {
+        } else if let Some(cow_str) = maybe_as_string(self, ErrorTypeDefaults::IntParsing)? {
             str_as_int(self, &cow_str)
         } else if let Ok(float) = self.extract::<f64>() {
             float_as_int(self, float)
         } else {
-            Err(ValError::new(ErrorType::IntType, self))
+            Err(ValError::new(ErrorTypeDefaults::IntType, self))
         }
     }
 
     fn ultra_strict_float(&'a self) -> ValResult<EitherFloat<'a>> {
         if self.is_instance_of::<PyInt>() {
-            Err(ValError::new(ErrorType::FloatType, self))
-        } else if self.is_instance_of::<PyFloat>() {
-            Ok(EitherFloat::Py(self))
+            Err(ValError::new(ErrorTypeDefaults::FloatType, self))
+        } else if let Ok(float) = self.downcast::<PyFloat>() {
+            Ok(EitherFloat::Py(float))
         } else {
-            Err(ValError::new(ErrorType::FloatType, self))
+            Err(ValError::new(ErrorTypeDefaults::FloatType, self))
         }
     }
     fn strict_float(&'a self) -> ValResult<EitherFloat<'a>> {
         if PyFloat::is_exact_type_of(self) {
-            Ok(EitherFloat::Py(self))
+            // Safety: self is PyFloat
+            Ok(EitherFloat::Py(unsafe { self.downcast_unchecked::<PyFloat>() }))
         } else if let Ok(float) = self.extract::<f64>() {
             // bools are cast to floats as either 0.0 or 1.0, so check for bool type in this specific case
             if (float == 0.0 || float == 1.0) && PyBool::is_exact_type_of(self) {
-                Err(ValError::new(ErrorType::FloatType, self))
+                Err(ValError::new(ErrorTypeDefaults::FloatType, self))
             } else {
-                Ok(EitherFloat::Py(self))
+                Ok(EitherFloat::F64(float))
             }
         } else {
-            Err(ValError::new(ErrorType::FloatType, self))
+            Err(ValError::new(ErrorTypeDefaults::FloatType, self))
         }
     }
 
     fn lax_float(&'a self) -> ValResult<EitherFloat<'a>> {
         if PyFloat::is_exact_type_of(self) {
-            Ok(EitherFloat::Py(self))
-        } else if let Some(cow_str) = maybe_as_string(self, ErrorType::FloatParsing)? {
-            match cow_str.as_ref().parse::<f64>() {
-                Ok(i) => Ok(EitherFloat::F64(i)),
-                Err(_) => Err(ValError::new(ErrorType::FloatParsing, self)),
-            }
+            // Safety: self is PyFloat
+            Ok(EitherFloat::Py(unsafe { self.downcast_unchecked::<PyFloat>() }))
+        } else if let Some(cow_str) = maybe_as_string(self, ErrorTypeDefaults::FloatParsing)? {
+            str_as_float(self, &cow_str)
         } else if let Ok(float) = self.extract::<f64>() {
             Ok(EitherFloat::F64(float))
         } else {
-            Err(ValError::new(ErrorType::FloatType, self))
+            Err(ValError::new(ErrorTypeDefaults::FloatType, self))
+        }
+    }
+
+    fn strict_decimal(&'a self, decimal_type: &'a PyType) -> ValResult<&'a PyAny> {
+        // Fast path for existing decimal objects
+        if self.is_exact_instance(decimal_type) {
+            return Ok(self);
+        }
+
+        // Try subclasses of decimals, they will be upcast to Decimal
+        if self.is_instance(decimal_type)? {
+            return create_decimal(self, self, decimal_type);
+        }
+
+        Err(ValError::new(
+            ErrorType::IsInstanceOf {
+                class: decimal_type.name().unwrap_or("Decimal").to_string(),
+                context: None,
+            },
+            self,
+        ))
+    }
+
+    fn lax_decimal(&'a self, decimal_type: &'a PyType) -> ValResult<&'a PyAny> {
+        // Fast path for existing decimal objects
+        if self.is_exact_instance(decimal_type) {
+            return Ok(self);
+        }
+
+        if self.is_instance_of::<PyString>() || (self.is_instance_of::<PyInt>() && !self.is_instance_of::<PyBool>()) {
+            // checking isinstance for str / int / bool is fast compared to decimal / float
+            create_decimal(self, self, decimal_type)
+        } else if self.is_instance(decimal_type)? {
+            // upcast subclasses to decimal
+            return create_decimal(self, self, decimal_type);
+        } else if self.is_instance_of::<PyFloat>() {
+            create_decimal(self.str()?, self, decimal_type)
+        } else {
+            Err(ValError::new(ErrorTypeDefaults::DecimalType, self))
         }
     }
 
@@ -351,7 +403,7 @@ impl<'a> Input<'a> for PyAny {
         if let Ok(dict) = self.downcast::<PyDict>() {
             Ok(dict.into())
         } else {
-            Err(ValError::new(ErrorType::DictType, self))
+            Err(ValError::new(ErrorTypeDefaults::DictType, self))
         }
     }
 
@@ -361,7 +413,7 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(mapping) = self.downcast::<PyMapping>() {
             Ok(mapping.into())
         } else {
-            Err(ValError::new(ErrorType::DictType, self))
+            Err(ValError::new(ErrorTypeDefaults::DictType, self))
         }
     }
 
@@ -382,11 +434,11 @@ impl<'a> Input<'a> for PyAny {
                 if from_attributes_applicable(obj) {
                     Ok(GenericMapping::PyGetAttr(obj, Some(kwargs)))
                 } else {
-                    Err(ValError::new(ErrorType::ModelAttributesType, self))
+                    Err(ValError::new(ErrorTypeDefaults::ModelAttributesType, self))
                 }
             } else {
                 // note the error here gives a hint about from_attributes
-                Err(ValError::new(ErrorType::ModelAttributesType, self))
+                Err(ValError::new(ErrorTypeDefaults::ModelAttributesType, self))
             }
         } else {
             // otherwise we just call back to validate_dict if from_mapping is allowed, note that errors in this
@@ -398,19 +450,19 @@ impl<'a> Input<'a> for PyAny {
     fn strict_list(&'a self) -> ValResult<GenericIterable<'a>> {
         match self.lax_list()? {
             GenericIterable::List(iter) => Ok(GenericIterable::List(iter)),
-            _ => Err(ValError::new(ErrorType::ListType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::ListType, self)),
         }
     }
 
     fn lax_list(&'a self) -> ValResult<GenericIterable<'a>> {
         match self
             .extract_generic_iterable()
-            .map_err(|_| ValError::new(ErrorType::ListType, self))?
+            .map_err(|_| ValError::new(ErrorTypeDefaults::ListType, self))?
         {
             GenericIterable::PyString(_)
             | GenericIterable::Bytes(_)
             | GenericIterable::Dict(_)
-            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorType::ListType, self)),
+            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorTypeDefaults::ListType, self)),
             other => Ok(other),
         }
     }
@@ -418,19 +470,19 @@ impl<'a> Input<'a> for PyAny {
     fn strict_tuple(&'a self) -> ValResult<GenericIterable<'a>> {
         match self.lax_tuple()? {
             GenericIterable::Tuple(iter) => Ok(GenericIterable::Tuple(iter)),
-            _ => Err(ValError::new(ErrorType::TupleType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::TupleType, self)),
         }
     }
 
     fn lax_tuple(&'a self) -> ValResult<GenericIterable<'a>> {
         match self
             .extract_generic_iterable()
-            .map_err(|_| ValError::new(ErrorType::TupleType, self))?
+            .map_err(|_| ValError::new(ErrorTypeDefaults::TupleType, self))?
         {
             GenericIterable::PyString(_)
             | GenericIterable::Bytes(_)
             | GenericIterable::Dict(_)
-            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorType::TupleType, self)),
+            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorTypeDefaults::TupleType, self)),
             other => Ok(other),
         }
     }
@@ -438,19 +490,19 @@ impl<'a> Input<'a> for PyAny {
     fn strict_set(&'a self) -> ValResult<GenericIterable<'a>> {
         match self.lax_set()? {
             GenericIterable::Set(iter) => Ok(GenericIterable::Set(iter)),
-            _ => Err(ValError::new(ErrorType::SetType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::SetType, self)),
         }
     }
 
     fn lax_set(&'a self) -> ValResult<GenericIterable<'a>> {
         match self
             .extract_generic_iterable()
-            .map_err(|_| ValError::new(ErrorType::SetType, self))?
+            .map_err(|_| ValError::new(ErrorTypeDefaults::SetType, self))?
         {
             GenericIterable::PyString(_)
             | GenericIterable::Bytes(_)
             | GenericIterable::Dict(_)
-            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorType::SetType, self)),
+            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorTypeDefaults::SetType, self)),
             other => Ok(other),
         }
     }
@@ -458,19 +510,19 @@ impl<'a> Input<'a> for PyAny {
     fn strict_frozenset(&'a self) -> ValResult<GenericIterable<'a>> {
         match self.lax_frozenset()? {
             GenericIterable::FrozenSet(iter) => Ok(GenericIterable::FrozenSet(iter)),
-            _ => Err(ValError::new(ErrorType::FrozenSetType, self)),
+            _ => Err(ValError::new(ErrorTypeDefaults::FrozenSetType, self)),
         }
     }
 
     fn lax_frozenset(&'a self) -> ValResult<GenericIterable<'a>> {
         match self
             .extract_generic_iterable()
-            .map_err(|_| ValError::new(ErrorType::FrozenSetType, self))?
+            .map_err(|_| ValError::new(ErrorTypeDefaults::FrozenSetType, self))?
         {
             GenericIterable::PyString(_)
             | GenericIterable::Bytes(_)
             | GenericIterable::Dict(_)
-            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorType::FrozenSetType, self)),
+            | GenericIterable::Mapping(_) => Err(ValError::new(ErrorTypeDefaults::FrozenSetType, self)),
             other => Ok(other),
         }
     }
@@ -506,7 +558,7 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(iterable) = self.iter() {
             Ok(GenericIterable::Iterator(iterable))
         } else {
-            Err(ValError::new(ErrorType::IterableType, self))
+            Err(ValError::new(ErrorTypeDefaults::IterableType, self))
         }
     }
 
@@ -514,18 +566,18 @@ impl<'a> Input<'a> for PyAny {
         if self.iter().is_ok() {
             Ok(self.into())
         } else {
-            Err(ValError::new(ErrorType::IterableType, self))
+            Err(ValError::new(ErrorTypeDefaults::IterableType, self))
         }
     }
 
     fn strict_date(&self) -> ValResult<EitherDate> {
         if PyDateTime::is_type_of(self) {
             // have to check if it's a datetime first, otherwise the line below converts to a date
-            Err(ValError::new(ErrorType::DateType, self))
+            Err(ValError::new(ErrorTypeDefaults::DateType, self))
         } else if let Ok(date) = self.downcast::<PyDate>() {
             Ok(date.into())
         } else {
-            Err(ValError::new(ErrorType::DateType, self))
+            Err(ValError::new(ErrorTypeDefaults::DateType, self))
         }
     }
 
@@ -533,7 +585,7 @@ impl<'a> Input<'a> for PyAny {
         if PyDateTime::is_type_of(self) {
             // have to check if it's a datetime first, otherwise the line below converts to a date
             // even if we later try coercion from a datetime, we don't want to return a datetime now
-            Err(ValError::new(ErrorType::DateType, self))
+            Err(ValError::new(ErrorTypeDefaults::DateType, self))
         } else if let Ok(date) = self.downcast::<PyDate>() {
             Ok(date.into())
         } else if let Ok(py_str) = self.downcast::<PyString>() {
@@ -542,55 +594,64 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
             bytes_as_date(self, py_bytes.as_bytes())
         } else {
-            Err(ValError::new(ErrorType::DateType, self))
+            Err(ValError::new(ErrorTypeDefaults::DateType, self))
         }
     }
 
-    fn strict_time(&self) -> ValResult<EitherTime> {
+    fn strict_time(
+        &self,
+        _microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTime> {
         if let Ok(time) = self.downcast::<PyTime>() {
             Ok(time.into())
         } else {
-            Err(ValError::new(ErrorType::TimeType, self))
+            Err(ValError::new(ErrorTypeDefaults::TimeType, self))
         }
     }
 
-    fn lax_time(&self) -> ValResult<EitherTime> {
+    fn lax_time(&self, microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior) -> ValResult<EitherTime> {
         if let Ok(time) = self.downcast::<PyTime>() {
             Ok(time.into())
         } else if let Ok(py_str) = self.downcast::<PyString>() {
             let str = py_string_str(py_str)?;
-            bytes_as_time(self, str.as_bytes())
+            bytes_as_time(self, str.as_bytes(), microseconds_overflow_behavior)
         } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            bytes_as_time(self, py_bytes.as_bytes())
+            bytes_as_time(self, py_bytes.as_bytes(), microseconds_overflow_behavior)
         } else if PyBool::is_exact_type_of(self) {
-            Err(ValError::new(ErrorType::TimeType, self))
+            Err(ValError::new(ErrorTypeDefaults::TimeType, self))
         } else if let Ok(int) = extract_i64(self) {
             int_as_time(self, int, 0)
         } else if let Ok(float) = self.extract::<f64>() {
             float_as_time(self, float)
         } else {
-            Err(ValError::new(ErrorType::TimeType, self))
+            Err(ValError::new(ErrorTypeDefaults::TimeType, self))
         }
     }
 
-    fn strict_datetime(&self) -> ValResult<EitherDateTime> {
+    fn strict_datetime(
+        &self,
+        _microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherDateTime> {
         if let Ok(dt) = self.downcast::<PyDateTime>() {
             Ok(dt.into())
         } else {
-            Err(ValError::new(ErrorType::DatetimeType, self))
+            Err(ValError::new(ErrorTypeDefaults::DatetimeType, self))
         }
     }
 
-    fn lax_datetime(&self) -> ValResult<EitherDateTime> {
+    fn lax_datetime(
+        &self,
+        microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherDateTime> {
         if let Ok(dt) = self.downcast::<PyDateTime>() {
             Ok(dt.into())
         } else if let Ok(py_str) = self.downcast::<PyString>() {
             let str = py_string_str(py_str)?;
-            bytes_as_datetime(self, str.as_bytes())
+            bytes_as_datetime(self, str.as_bytes(), microseconds_overflow_behavior)
         } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            bytes_as_datetime(self, py_bytes.as_bytes())
+            bytes_as_datetime(self, py_bytes.as_bytes(), microseconds_overflow_behavior)
         } else if PyBool::is_exact_type_of(self) {
-            Err(ValError::new(ErrorType::DatetimeType, self))
+            Err(ValError::new(ErrorTypeDefaults::DatetimeType, self))
         } else if let Ok(int) = extract_i64(self) {
             int_as_datetime(self, int, 0)
         } else if let Ok(float) = self.extract::<f64>() {
@@ -598,32 +659,38 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(date) = self.downcast::<PyDate>() {
             Ok(date_as_datetime(date)?)
         } else {
-            Err(ValError::new(ErrorType::DatetimeType, self))
+            Err(ValError::new(ErrorTypeDefaults::DatetimeType, self))
         }
     }
 
-    fn strict_timedelta(&self) -> ValResult<EitherTimedelta> {
-        if let Ok(dt) = self.downcast::<PyDelta>() {
-            Ok(dt.into())
+    fn strict_timedelta(
+        &self,
+        _microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTimedelta> {
+        if let Ok(either_dt) = EitherTimedelta::try_from(self) {
+            Ok(either_dt)
         } else {
-            Err(ValError::new(ErrorType::TimeDeltaType, self))
+            Err(ValError::new(ErrorTypeDefaults::TimeDeltaType, self))
         }
     }
 
-    fn lax_timedelta(&self) -> ValResult<EitherTimedelta> {
-        if let Ok(dt) = self.downcast::<PyDelta>() {
-            Ok(dt.into())
+    fn lax_timedelta(
+        &self,
+        microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<EitherTimedelta> {
+        if let Ok(either_dt) = EitherTimedelta::try_from(self) {
+            Ok(either_dt)
         } else if let Ok(py_str) = self.downcast::<PyString>() {
             let str = py_string_str(py_str)?;
-            bytes_as_timedelta(self, str.as_bytes())
+            bytes_as_timedelta(self, str.as_bytes(), microseconds_overflow_behavior)
         } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            bytes_as_timedelta(self, py_bytes.as_bytes())
+            bytes_as_timedelta(self, py_bytes.as_bytes(), microseconds_overflow_behavior)
         } else if let Ok(int) = extract_i64(self) {
             Ok(int_as_duration(self, int)?.into())
         } else if let Ok(float) = self.extract::<f64>() {
             Ok(float_as_duration(self, float)?.into())
         } else {
-            Err(ValError::new(ErrorType::TimeDeltaType, self))
+            Err(ValError::new(ErrorTypeDefaults::TimeDeltaType, self))
         }
     }
 }
