@@ -10,8 +10,9 @@ mod parse_json;
 mod return_enums;
 mod shared;
 
+pub use datetime::TzInfo;
 pub(crate) use datetime::{
-    pydate_as_date, pydatetime_as_datetime, pytime_as_time, pytimedelta_as_duration, EitherDate, EitherDateTime,
+    duration_as_pytimedelta, pydate_as_date, pydatetime_as_datetime, pytime_as_time, EitherDate, EitherDateTime,
     EitherTime, EitherTimedelta,
 };
 pub(crate) use input_abstract::{Input, InputType};
