@@ -2,6 +2,7 @@ use std::borrow::Cow;
 use std::cmp::Ordering;
 use std::ops::Rem;
 use std::slice::Iter as SliceIter;
+use std::str::FromStr;
 
 use num_bigint::BigInt;
 
@@ -9,7 +10,8 @@ use pyo3::exceptions::PyTypeError;
 use pyo3::prelude::*;
 use pyo3::types::iter::PyDictIterator;
 use pyo3::types::{
-    PyByteArray, PyBytes, PyDict, PyFrozenSet, PyIterator, PyList, PyMapping, PySequence, PySet, PyString, PyTuple,
+    PyByteArray, PyBytes, PyDict, PyFloat, PyFrozenSet, PyIterator, PyList, PyMapping, PySequence, PySet, PyString,
+    PyTuple,
 };
 use pyo3::{ffi, intern, AsPyPointer, PyNativeType};
 
@@ -17,11 +19,11 @@ use pyo3::{ffi, intern, AsPyPointer, PyNativeType};
 use pyo3::types::PyFunction;
 #[cfg(not(PyPy))]
 use pyo3::PyTypeInfo;
+use serde::{ser::Error, Serialize, Serializer};
 
-use crate::errors::{py_err_string, ErrorType, InputValue, ValError, ValLineError, ValResult};
-use crate::recursion_guard::RecursionGuard;
+use crate::errors::{py_err_string, ErrorType, ErrorTypeDefaults, InputValue, ValError, ValLineError, ValResult};
 use crate::tools::py_err;
-use crate::validators::{CombinedValidator, Extra, Validator};
+use crate::validators::{CombinedValidator, ValidationState, Validator};
 
 use super::parse_json::{JsonArray, JsonInput, JsonObject};
 use super::{py_error_on_minusone, Input};
@@ -124,6 +126,7 @@ impl<'a, INPUT: Input<'a>> MaxLengthCheck<'a, INPUT> {
                         field_type: self.field_type.to_string(),
                         max_length,
                         actual_length: self.current_length,
+                        context: None,
                     },
                     self.input,
                 ));
@@ -138,6 +141,7 @@ macro_rules! any_next_error {
         ValError::new_with_loc(
             ErrorType::IterationError {
                 error: py_err_string($py, $err),
+                context: None,
             },
             $input,
             $index,
@@ -152,15 +156,13 @@ fn validate_iter_to_vec<'a, 's>(
     capacity: usize,
     mut max_length_check: MaxLengthCheck<'a, impl Input<'a>>,
     validator: &'s CombinedValidator,
-    extra: &Extra,
-    definitions: &'a [CombinedValidator],
-    recursion_guard: &'s mut RecursionGuard,
+    state: &mut ValidationState,
 ) -> ValResult<'a, Vec<PyObject>> {
     let mut output: Vec<PyObject> = Vec::with_capacity(capacity);
     let mut errors: Vec<ValLineError> = Vec::new();
     for (index, item_result) in iter.enumerate() {
         let item = item_result.map_err(|e| any_next_error!(py, e, max_length_check.input, index))?;
-        match validator.validate(py, item, extra, definitions, recursion_guard) {
+        match validator.validate(py, item, state) {
             Ok(item) => {
                 max_length_check.incr()?;
                 output.push(item);
@@ -221,14 +223,12 @@ fn validate_iter_to_set<'a, 's>(
     field_type: &'static str,
     max_length: Option<usize>,
     validator: &'s CombinedValidator,
-    extra: &Extra,
-    definitions: &'a [CombinedValidator],
-    recursion_guard: &'s mut RecursionGuard,
+    state: &mut ValidationState,
 ) -> ValResult<'a, ()> {
     let mut errors: Vec<ValLineError> = Vec::new();
     for (index, item_result) in iter.enumerate() {
         let item = item_result.map_err(|e| any_next_error!(py, e, input, index))?;
-        match validator.validate(py, item, extra, definitions, recursion_guard) {
+        match validator.validate(py, item, state) {
             Ok(item) => {
                 set.build_add(item)?;
                 if let Some(max_length) = max_length {
@@ -239,6 +239,7 @@ fn validate_iter_to_set<'a, 's>(
                                 field_type: field_type.to_string(),
                                 max_length,
                                 actual_length,
+                                context: None,
                             },
                             input,
                         ));
@@ -309,9 +310,7 @@ impl<'a> GenericIterable<'a> {
         max_length: Option<usize>,
         field_type: &'static str,
         validator: &'s CombinedValidator,
-        extra: &Extra,
-        definitions: &'a [CombinedValidator],
-        recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'a, Vec<PyObject>> {
         let capacity = self
             .generic_len()
@@ -320,16 +319,7 @@ impl<'a> GenericIterable<'a> {
 
         macro_rules! validate {
             ($iter:expr) => {
-                validate_iter_to_vec(
-                    py,
-                    $iter,
-                    capacity,
-                    max_length_check,
-                    validator,
-                    extra,
-                    definitions,
-                    recursion_guard,
-                )
+                validate_iter_to_vec(py, $iter, capacity, max_length_check, validator, state)
             };
         }
 
@@ -354,24 +344,11 @@ impl<'a> GenericIterable<'a> {
         max_length: Option<usize>,
         field_type: &'static str,
         validator: &'s CombinedValidator,
-        extra: &Extra,
-        definitions: &'a [CombinedValidator],
-        recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'a, ()> {
         macro_rules! validate_set {
             ($iter:expr) => {
-                validate_iter_to_set(
-                    py,
-                    set,
-                    $iter,
-                    input,
-                    field_type,
-                    max_length,
-                    validator,
-                    extra,
-                    definitions,
-                    recursion_guard,
-                )
+                validate_iter_to_set(py, set, $iter, input, field_type, max_length, validator, state)
             };
         }
 
@@ -464,6 +441,7 @@ fn mapping_err<'py>(err: PyErr, py: Python<'py>, input: &'py impl Input<'py>) ->
     ValError::new(
         ErrorType::MappingType {
             error: py_err_string(py, err).into(),
+            context: None,
         },
         input,
     )
@@ -499,6 +477,7 @@ impl<'py> Iterator for MappingGenericIterator<'py> {
                 return Some(Err(ValError::new(
                     ErrorType::MappingType {
                         error: MAPPING_TUPLE_ERROR.into(),
+                        context: None,
                     },
                     self.input,
                 )))
@@ -508,6 +487,7 @@ impl<'py> Iterator for MappingGenericIterator<'py> {
             return Some(Err(ValError::new(
                 ErrorType::MappingType {
                     error: MAPPING_TUPLE_ERROR.into(),
+                    context: None,
                 },
                 self.input,
             )));
@@ -779,7 +759,7 @@ impl<'a> IntoPy<PyObject> for EitherString<'a> {
 pub fn py_string_str(py_str: &PyString) -> ValResult<&str> {
     py_str
         .to_str()
-        .map_err(|_| ValError::new_custom_input(ErrorType::StringUnicode, InputValue::PyAny(py_str as &PyAny)))
+        .map_err(|_| ValError::new_custom_input(ErrorTypeDefaults::StringUnicode, InputValue::PyAny(py_str as &PyAny)))
 }
 
 #[cfg_attr(debug_assertions, derive(Debug))]
@@ -807,6 +787,13 @@ impl<'a> From<&'a PyBytes> for EitherBytes<'a> {
 }
 
 impl<'a> EitherBytes<'a> {
+    pub fn as_slice(&'a self) -> &[u8] {
+        match self {
+            EitherBytes::Cow(bytes) => bytes,
+            EitherBytes::Py(py_bytes) => py_bytes.as_bytes(),
+        }
+    }
+
     pub fn len(&'a self) -> PyResult<usize> {
         match self {
             EitherBytes::Cow(bytes) => Ok(bytes.len()),
@@ -838,16 +825,21 @@ impl<'a> EitherInt<'a> {
             EitherInt::I64(i) => Ok(i),
             EitherInt::U64(u) => match i64::try_from(u) {
                 Ok(u) => Ok(u),
-                Err(_) => Err(ValError::new(ErrorType::IntParsingSize, u.into_py(py).into_ref(py))),
+                Err(_) => Err(ValError::new(
+                    ErrorTypeDefaults::IntParsingSize,
+                    u.into_py(py).into_ref(py),
+                )),
             },
             EitherInt::BigInt(u) => match i64::try_from(u) {
                 Ok(u) => Ok(u),
                 Err(e) => Err(ValError::new(
-                    ErrorType::IntParsingSize,
+                    ErrorTypeDefaults::IntParsingSize,
                     e.into_original().into_py(py).into_ref(py),
                 )),
             },
-            EitherInt::Py(i) => i.extract().map_err(|_| ValError::new(ErrorType::IntParsingSize, i)),
+            EitherInt::Py(i) => i
+                .extract()
+                .map_err(|_| ValError::new(ErrorTypeDefaults::IntParsingSize, i)),
         }
     }
 
@@ -859,7 +851,9 @@ impl<'a> EitherInt<'a> {
                 Err(_) => Ok(Int::Big(BigInt::from(*u))),
             },
             EitherInt::BigInt(b) => Ok(Int::Big(b.clone())),
-            EitherInt::Py(i) => i.extract().map_err(|_| ValError::new(ErrorType::IntParsingSize, *i)),
+            EitherInt::Py(i) => i
+                .extract()
+                .map_err(|_| ValError::new(ErrorTypeDefaults::IntParsingSize, *i)),
         }
     }
 
@@ -901,18 +895,23 @@ impl<'a> IntoPy<PyObject> for EitherInt<'a> {
 }
 
 #[cfg_attr(debug_assertions, derive(Debug))]
+#[derive(Copy, Clone)]
 pub enum EitherFloat<'a> {
     F64(f64),
-    Py(&'a PyAny),
+    Py(&'a PyFloat),
 }
 
-impl<'a> TryInto<f64> for EitherFloat<'a> {
-    type Error = ValError<'a>;
-
-    fn try_into(self) -> ValResult<'a, f64> {
+impl<'a> EitherFloat<'a> {
+    pub fn as_f64(self) -> f64 {
         match self {
-            EitherFloat::F64(f) => Ok(f),
-            EitherFloat::Py(i) => i.extract().map_err(|_| ValError::new(ErrorType::FloatParsing, i)),
+            EitherFloat::F64(f) => f,
+
+            EitherFloat::Py(f) => {
+                {
+                    // Safety: known to be a python float
+                    unsafe { ffi::PyFloat_AS_DOUBLE(f.as_ptr()) }
+                }
+            }
         }
     }
 }
@@ -926,12 +925,26 @@ impl<'a> IntoPy<PyObject> for EitherFloat<'a> {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Serialize)]
+#[serde(untagged)]
 pub enum Int {
     I64(i64),
+    #[serde(serialize_with = "serialize_bigint_as_number")]
     Big(BigInt),
 }
 
+// The default serialization for BigInt is some internal representation which roundtrips efficiently
+// but is not the JSON value which users would expect to see.
+fn serialize_bigint_as_number<S>(big_int: &BigInt, serializer: S) -> Result<S::Ok, S::Error>
+where
+    S: Serializer,
+{
+    serde_json::Number::from_str(&big_int.to_string())
+        .map_err(S::Error::custom)
+        .expect("a valid number")
+        .serialize(serializer)
+}
+
 impl PartialOrd for Int {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         match (self, other) {
@@ -973,3 +986,12 @@ impl<'a> FromPyObject<'a> for Int {
         }
     }
 }
+
+impl ToPyObject for Int {
+    fn to_object(&self, py: Python<'_>) -> PyObject {
+        match self {
+            Self::I64(i) => i.to_object(py),
+            Self::Big(big_i) => big_i.to_object(py),
+        }
+    }
+}
