@@ -1,14 +1,15 @@
 use num_bigint::BigInt;
 
-use crate::errors::{ErrorType, ValError, ValResult};
+use crate::errors::{ErrorType, ErrorTypeDefaults, ValError, ValResult};
 use crate::input::EitherInt;
 
-use super::Input;
+use super::{EitherFloat, Input};
 
 pub fn map_json_err<'a>(input: &'a impl Input<'a>, error: serde_json::Error) -> ValError<'a> {
     ValError::new(
         ErrorType::JsonInvalid {
             error: error.to_string(),
+            context: None,
         },
         input,
     )
@@ -32,7 +33,7 @@ pub fn str_as_bool<'a>(input: &'a impl Input<'a>, str: &str) -> ValResult<'a, bo
     {
         Ok(true)
     } else {
-        Err(ValError::new(ErrorType::BoolParsing, input))
+        Err(ValError::new(ErrorTypeDefaults::BoolParsing, input))
     }
 }
 
@@ -42,10 +43,25 @@ pub fn int_as_bool<'a>(input: &'a impl Input<'a>, int: i64) -> ValResult<'a, boo
     } else if int == 1 {
         Ok(true)
     } else {
-        Err(ValError::new(ErrorType::BoolParsing, input))
+        Err(ValError::new(ErrorTypeDefaults::BoolParsing, input))
     }
 }
 
+/// Strip underscores from strings so that 1_000 can be parsed to 1000
+/// Ignore any unicode stuff since this has to be digits and underscores
+/// and if it's not subsequent parsing will just fail
+fn strip_underscores(s: &str) -> Option<String> {
+    // Leading and trailing underscores are not valid in Python (e.g. `int('__1__')` fails)
+    // so we match that behavior here.
+    // Double consecutive underscores are also not valid
+    // If there are no underscores at all, no need to replace anything
+    if s.starts_with('_') || s.ends_with('_') || !s.contains('_') || s.contains("__") {
+        // no underscores to strip
+        return None;
+    }
+    Some(s.replace('_', ""))
+}
+
 /// parse a string as an int
 ///
 /// max length of the input is 4300, see
@@ -54,17 +70,34 @@ pub fn int_as_bool<'a>(input: &'a impl Input<'a>, int: i64) -> ValResult<'a, boo
 pub fn str_as_int<'s, 'l>(input: &'s impl Input<'s>, str: &'l str) -> ValResult<'s, EitherInt<'s>> {
     let len = str.len();
     if len > 4300 {
-        Err(ValError::new(ErrorType::IntParsingSize, input))
+        Err(ValError::new(ErrorTypeDefaults::IntParsingSize, input))
     } else if let Some(int) = _parse_str(input, str, len) {
         Ok(int)
     } else if let Some(str_stripped) = strip_decimal_zeros(str) {
         if let Some(int) = _parse_str(input, str_stripped, len) {
             Ok(int)
         } else {
-            Err(ValError::new(ErrorType::IntParsing, input))
+            Err(ValError::new(ErrorTypeDefaults::IntParsing, input))
+        }
+    } else if let Some(str_stripped) = strip_underscores(str) {
+        if let Some(int) = _parse_str(input, &str_stripped, len) {
+            Ok(int)
+        } else {
+            Err(ValError::new(ErrorTypeDefaults::IntParsing, input))
         }
     } else {
-        Err(ValError::new(ErrorType::IntParsing, input))
+        Err(ValError::new(ErrorTypeDefaults::IntParsing, input))
+    }
+}
+
+/// parse a float as a float
+pub fn str_as_float<'s, 'l>(input: &'s impl Input<'s>, str: &'l str) -> ValResult<'s, EitherFloat<'s>> {
+    match str.parse() {
+        Ok(float) => Ok(EitherFloat::F64(float)),
+        Err(_) => match strip_underscores(str).and_then(|stripped| stripped.parse().ok()) {
+            Some(float) => Ok(EitherFloat::F64(float)),
+            None => Err(ValError::new(ErrorTypeDefaults::FloatParsing, input)),
+        },
     }
 }
 
@@ -93,13 +126,13 @@ fn strip_decimal_zeros(s: &str) -> Option<&str> {
 }
 
 pub fn float_as_int<'a>(input: &'a impl Input<'a>, float: f64) -> ValResult<'a, EitherInt<'a>> {
-    if float == f64::INFINITY || float == f64::NEG_INFINITY || float.is_nan() {
-        Err(ValError::new(ErrorType::FiniteNumber, input))
+    if float.is_infinite() || float.is_nan() {
+        Err(ValError::new(ErrorTypeDefaults::FiniteNumber, input))
     } else if float % 1.0 != 0.0 {
-        Err(ValError::new(ErrorType::IntFromFloat, input))
+        Err(ValError::new(ErrorTypeDefaults::IntFromFloat, input))
     } else if (i64::MIN as f64) < float && float < (i64::MAX as f64) {
         Ok(EitherInt::I64(float as i64))
     } else {
-        Err(ValError::new(ErrorType::IntParsingSize, input))
+        Err(ValError::new(ErrorTypeDefaults::IntParsingSize, input))
     }
 }
