@@ -2,11 +2,13 @@
 
 extern crate core;
 
-use pyo3::prelude::*;
+use std::sync::OnceLock;
 
-#[cfg(feature = "mimalloc")]
-#[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+use pyo3::{prelude::*, sync::GILOnceCell};
+
+// parse this first to get access to the contained macro
+#[macro_use]
+mod py_gc;
 
 mod argument_markers;
 mod build_tools;
@@ -22,6 +24,7 @@ mod url;
 mod validators;
 
 // required for benchmarks
+pub use self::input::TzInfo;
 pub use self::url::{PyMultiHostUrl, PyUrl};
 pub use argument_markers::{ArgsKwargs, PydanticUndefinedType};
 pub use build_tools::SchemaError;
@@ -33,20 +36,47 @@ pub use serializers::{
 };
 pub use validators::{PySome, SchemaValidator};
 
-pub fn get_version() -> String {
-    let version = env!("CARGO_PKG_VERSION");
-    // cargo uses "1.0-alpha1" etc. while python uses "1.0.0a1", this is not full compatibility,
-    // but it's good enough for now
-    // see https://docs.rs/semver/1.0.9/semver/struct.Version.html#method.parse for rust spec
-    // see https://peps.python.org/pep-0440/ for python spec
-    // it seems the dot after "alpha/beta" e.g. "-alpha.1" is not necessary, hence why this works
-    version.replace("-alpha", "a").replace("-beta", "b")
+pub fn get_pydantic_core_version() -> &'static str {
+    static PYDANTIC_CORE_VERSION: OnceLock<String> = OnceLock::new();
+
+    PYDANTIC_CORE_VERSION.get_or_init(|| {
+        let version = env!("CARGO_PKG_VERSION");
+        // cargo uses "1.0-alpha1" etc. while python uses "1.0.0a1", this is not full compatibility,
+        // but it's good enough for now
+        // see https://docs.rs/semver/1.0.9/semver/struct.Version.html#method.parse for rust spec
+        // see https://peps.python.org/pep-0440/ for python spec
+        // it seems the dot after "alpha/beta" e.g. "-alpha.1" is not necessary, hence why this works
+        version.replace("-alpha", "a").replace("-beta", "b")
+    })
+}
+
+/// Returns the installed version of pydantic.
+fn get_pydantic_version(py: Python<'_>) -> Option<&'static str> {
+    static PYDANTIC_VERSION: GILOnceCell<Option<String>> = GILOnceCell::new();
+
+    PYDANTIC_VERSION
+        .get_or_init(py, || {
+            py.import("pydantic")
+                .and_then(|pydantic| pydantic.getattr("__version__")?.extract())
+                .ok()
+        })
+        .as_deref()
+}
+
+pub fn build_info() -> String {
+    format!(
+        "profile={} pgo={}",
+        env!("PROFILE"),
+        option_env!("RUSTFLAGS").unwrap_or("").contains("-Cprofile-use="),
+    )
 }
 
 #[pymodule]
 fn _pydantic_core(py: Python, m: &PyModule) -> PyResult<()> {
-    m.add("__version__", get_version())?;
+    m.add("__version__", get_pydantic_core_version())?;
     m.add("build_profile", env!("PROFILE"))?;
+    m.add("build_info", build_info())?;
+    m.add("_recursion_limit", recursion_guard::RECURSION_GUARD_LIMIT)?;
     m.add("PydanticUndefined", PydanticUndefinedType::new(py))?;
     m.add_class::<PydanticUndefinedType>()?;
     m.add_class::<PySome>()?;
@@ -63,12 +93,9 @@ fn _pydantic_core(py: Python, m: &PyModule) -> PyResult<()> {
     m.add_class::<PyMultiHostUrl>()?;
     m.add_class::<ArgsKwargs>()?;
     m.add_class::<SchemaSerializer>()?;
+    m.add_class::<TzInfo>()?;
     m.add_function(wrap_pyfunction!(to_json, m)?)?;
     m.add_function(wrap_pyfunction!(to_jsonable_python, m)?)?;
     m.add_function(wrap_pyfunction!(list_all_errors, m)?)?;
-
-    #[cfg(not(feature = "mimalloc"))]
-    m.setattr("__pydantic_core_default_allocator__", true)?; // uses setattr so this is not in __all__
-
     Ok(())
 }
