@@ -1,6 +1,7 @@
 use std::borrow::Cow;
 use std::str::{from_utf8, FromStr, Utf8Error};
 
+use base64::Engine;
 use pyo3::prelude::*;
 use pyo3::types::{PyDelta, PyDict};
 use pyo3::{intern, PyNativeType};
@@ -8,7 +9,7 @@ use pyo3::{intern, PyNativeType};
 use serde::ser::Error;
 
 use crate::build_tools::py_schema_err;
-use crate::input::pytimedelta_as_duration;
+use crate::input::EitherTimedelta;
 use crate::tools::SchemaDict;
 
 use super::errors::py_err_se_err;
@@ -61,7 +62,9 @@ impl FromStr for TimedeltaMode {
 
 impl TimedeltaMode {
     pub fn from_config(config: Option<&PyDict>) -> PyResult<Self> {
-        let Some(config_dict) = config else { return Ok(Self::default()) };
+        let Some(config_dict) = config else {
+            return Ok(Self::default());
+        };
         let raw_mode = config_dict.get_as::<&str>(intern!(config_dict.py(), "ser_json_timedelta"))?;
         raw_mode.map_or_else(|| Ok(Self::default()), Self::from_str)
     }
@@ -70,27 +73,30 @@ impl TimedeltaMode {
         py_timedelta.call_method0(intern!(py_timedelta.py(), "total_seconds"))
     }
 
-    pub fn timedelta_to_json(&self, py_timedelta: &PyDelta) -> PyResult<PyObject> {
-        let py = py_timedelta.py();
+    pub fn either_delta_to_json(&self, py: Python, either_delta: &EitherTimedelta) -> PyResult<PyObject> {
         match self {
             Self::Iso8601 => {
-                let d = pytimedelta_as_duration(py_timedelta);
+                let d = either_delta.to_duration()?;
                 Ok(d.to_string().into_py(py))
             }
             Self::Float => {
+                // convert to int via a py timedelta not duration since we know this this case the input would have
+                // been a py timedelta
+                let py_timedelta = either_delta.try_into_py(py)?;
                 let seconds = Self::total_seconds(py_timedelta)?;
                 Ok(seconds.into_py(py))
             }
         }
     }
 
-    pub fn json_key<'py>(&self, py_timedelta: &PyDelta) -> PyResult<Cow<'py, str>> {
+    pub fn json_key<'py>(&self, py: Python, either_delta: &EitherTimedelta) -> PyResult<Cow<'py, str>> {
         match self {
             Self::Iso8601 => {
-                let d = pytimedelta_as_duration(py_timedelta);
+                let d = either_delta.to_duration()?;
                 Ok(d.to_string().into())
             }
             Self::Float => {
+                let py_timedelta = either_delta.try_into_py(py)?;
                 let seconds: f64 = Self::total_seconds(py_timedelta)?.extract()?;
                 Ok(seconds.to_string().into())
             }
@@ -99,15 +105,17 @@ impl TimedeltaMode {
 
     pub fn timedelta_serialize<S: serde::ser::Serializer>(
         &self,
-        py_timedelta: &PyDelta,
+        py: Python,
+        either_delta: &EitherTimedelta,
         serializer: S,
     ) -> Result<S::Ok, S::Error> {
         match self {
             Self::Iso8601 => {
-                let d = pytimedelta_as_duration(py_timedelta);
+                let d = either_delta.to_duration().map_err(py_err_se_err)?;
                 serializer.serialize_str(&d.to_string())
             }
             Self::Float => {
+                let py_timedelta = either_delta.try_into_py(py).map_err(py_err_se_err)?;
                 let seconds = Self::total_seconds(py_timedelta).map_err(py_err_se_err)?;
                 let seconds: f64 = seconds.extract().map_err(py_err_se_err)?;
                 serializer.serialize_f64(seconds)
@@ -117,48 +125,49 @@ impl TimedeltaMode {
 }
 
 #[derive(Default, Debug, Clone)]
-pub(crate) struct BytesMode {
-    base64_config: Option<base64::Config>,
+pub(crate) enum BytesMode {
+    #[default]
+    Utf8,
+    Base64,
 }
 
 impl FromStr for BytesMode {
     type Err = PyErr;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        let base64_config = match s {
-            "utf8" => None,
-            "base64" => Some(base64::Config::new(base64::CharacterSet::UrlSafe, true)),
-            s => return py_schema_err!("Invalid bytes serialization mode: `{}`, expected `utf8` or `base64`", s),
-        };
-        Ok(Self { base64_config })
+        match s {
+            "utf8" => Ok(Self::Utf8),
+            "base64" => Ok(Self::Base64),
+            s => py_schema_err!("Invalid bytes serialization mode: `{}`, expected `utf8` or `base64`", s),
+        }
     }
 }
 
 impl BytesMode {
     pub fn from_config(config: Option<&PyDict>) -> PyResult<Self> {
-        let Some(config_dict) = config else { return Ok(Self::default()) };
+        let Some(config_dict) = config else {
+            return Ok(Self::default());
+        };
         let raw_mode = config_dict.get_as::<&str>(intern!(config_dict.py(), "ser_json_bytes"))?;
         raw_mode.map_or_else(|| Ok(Self::default()), Self::from_str)
     }
 
     pub fn bytes_to_string<'py>(&self, py: Python, bytes: &'py [u8]) -> PyResult<Cow<'py, str>> {
-        if let Some(config) = self.base64_config {
-            Ok(Cow::Owned(base64::encode_config(bytes, config)))
-        } else {
-            from_utf8(bytes)
+        match self {
+            Self::Utf8 => from_utf8(bytes)
                 .map_err(|err| utf8_py_error(py, err, bytes))
-                .map(Cow::Borrowed)
+                .map(Cow::Borrowed),
+            Self::Base64 => Ok(Cow::Owned(base64::engine::general_purpose::URL_SAFE.encode(bytes))),
         }
     }
 
     pub fn serialize_bytes<S: serde::ser::Serializer>(&self, bytes: &[u8], serializer: S) -> Result<S::Ok, S::Error> {
-        if let Some(config) = self.base64_config {
-            serializer.serialize_str(&base64::encode_config(bytes, config))
-        } else {
-            match from_utf8(bytes) {
+        match self {
+            Self::Utf8 => match from_utf8(bytes) {
                 Ok(s) => serializer.serialize_str(s),
                 Err(e) => Err(Error::custom(e.to_string())),
-            }
+            },
+            Self::Base64 => serializer.serialize_str(&base64::engine::general_purpose::URL_SAFE.encode(bytes)),
         }
     }
 }
