@@ -28,6 +28,8 @@ pub(super) struct SerField {
     pub required: bool,
 }
 
+impl_py_gc_traverse!(SerField { serializer });
+
 impl SerField {
     pub fn new(
         py: Python,
@@ -142,6 +144,11 @@ macro_rules! option_length {
     };
 }
 
+impl_py_gc_traverse!(GeneralFieldsSerializer {
+    fields,
+    computed_fields
+});
+
 impl TypeSerializer for GeneralFieldsSerializer {
     fn to_python(
         &self,
@@ -205,12 +212,17 @@ impl TypeSerializer for GeneralFieldsSerializer {
                 }
             }
         }
-        if td_extra.check.enabled() && self.required_fields != used_req_fields {
+        if td_extra.check.enabled()
+            // If any of these are true we can't count fields
+            && !(extra.exclude_defaults || extra.exclude_unset || extra.exclude_none)
+            // Check for missing fields, we can't have extra fields here
+            && self.required_fields > used_req_fields
+        {
             return Err(PydanticSerializationUnexpectedValue::new_err(None));
         }
         // this is used to include `__pydantic_extra__` in serialization on models
         if let Some(extra_dict) = extra_dict {
-            for (key, value) in extra_dict.iter() {
+            for (key, value) in extra_dict {
                 if extra.exclude_none && value.is_none() {
                     continue;
                 }
@@ -292,7 +304,7 @@ impl TypeSerializer for GeneralFieldsSerializer {
         }
         // this is used to include `__pydantic_extra__` in serialization on models
         if let Some(extra_dict) = extra_dict {
-            for (key, value) in extra_dict.iter() {
+            for (key, value) in extra_dict {
                 if extra.exclude_none && value.is_none() {
                     continue;
                 }
