@@ -4,12 +4,12 @@ use pyo3::exceptions::PyTypeError;
 use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{
-    PyByteArray, PyBytes, PyDate, PyDateTime, PyDelta, PyDict, PyFrozenSet, PyIterator, PyList, PySet, PyString,
-    PyTime, PyTuple,
+    PyByteArray, PyBytes, PyDate, PyDateTime, PyDict, PyFrozenSet, PyIterator, PyList, PySet, PyString, PyTime, PyTuple,
 };
 
 use serde::ser::{Error, Serialize, SerializeMap, SerializeSeq, Serializer};
 
+use crate::input::{EitherTimedelta, Int};
 use crate::serializers::errors::SERIALIZATION_ERR_MARKER;
 use crate::serializers::filter::SchemaFilter;
 use crate::serializers::shared::{PydanticSerializer, TypeSerializer};
@@ -29,7 +29,7 @@ pub(crate) fn infer_to_python(
     exclude: Option<&PyAny>,
     extra: &Extra,
 ) -> PyResult<PyObject> {
-    infer_to_python_known(&extra.ob_type_lookup.get_type(value), value, include, exclude, extra)
+    infer_to_python_known(extra.ob_type_lookup.get_type(value), value, include, exclude, extra)
 }
 
 // arbitrary ids to identify that we recursed through infer_to_{python,json}_known
@@ -37,7 +37,7 @@ pub(crate) fn infer_to_python(
 const INFER_DEF_REF_ID: usize = usize::MAX;
 
 pub(crate) fn infer_to_python_known(
-    ob_type: &ObType,
+    ob_type: ObType,
     value: &PyAny,
     include: Option<&PyAny>,
     exclude: Option<&PyAny>,
@@ -100,7 +100,7 @@ pub(crate) fn infer_to_python_known(
 
     let serialize_with_serializer = || {
         let py_serializer = value.getattr(intern!(py, "__pydantic_serializer__"))?;
-        let serializer: SchemaSerializer = py_serializer.extract()?;
+        let serializer: PyRef<SchemaSerializer> = py_serializer.extract()?;
         let extra = serializer.build_extra(
             py,
             extra.mode,
@@ -175,8 +175,11 @@ pub(crate) fn infer_to_python_known(
                 iso_time.into_py(py)
             }
             ObType::Timedelta => {
-                let py_timedelta: &PyDelta = value.downcast()?;
-                extra.config.timedelta_mode.timedelta_to_json(py_timedelta)?
+                let either_delta = EitherTimedelta::try_from(value)?;
+                extra
+                    .config
+                    .timedelta_mode
+                    .either_delta_to_json(value.py(), &either_delta)?
             }
             ObType::Url => {
                 let py_url: PyUrl = value.extract()?;
@@ -186,6 +189,11 @@ pub(crate) fn infer_to_python_known(
                 let py_url: PyMultiHostUrl = value.extract()?;
                 py_url.__str__().into_py(py)
             }
+            ObType::Uuid => {
+                let py_uuid: &PyAny = value.downcast()?;
+                let uuid = super::type_serializers::uuid::uuid_to_string(py_uuid)?;
+                uuid.into_py(py)
+            }
             ObType::PydanticSerializable => serialize_with_serializer()?,
             ObType::Dataclass => serialize_dict(dataclass_to_dict(value)?)?,
             ObType::Enum => {
@@ -257,7 +265,7 @@ pub(crate) fn infer_to_python_known(
             ObType::Generator => {
                 let iter = super::type_serializers::generator::SerializationIterator::new(
                     value.downcast()?,
-                    super::type_serializers::any::AnySerializer::default().into(),
+                    super::type_serializers::any::AnySerializer.into(),
                     SchemaFilter::default(),
                     include,
                     exclude,
@@ -307,7 +315,7 @@ impl<'py> SerializeInfer<'py> {
 impl<'py> Serialize for SerializeInfer<'py> {
     fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
         let ob_type = self.extra.ob_type_lookup.get_type(self.value);
-        infer_serialize_known(&ob_type, self.value, serializer, self.include, self.exclude, self.extra)
+        infer_serialize_known(ob_type, self.value, serializer, self.include, self.exclude, self.extra)
     }
 }
 
@@ -319,7 +327,7 @@ pub(crate) fn infer_serialize<S: Serializer>(
     extra: &Extra,
 ) -> Result<S::Ok, S::Error> {
     infer_serialize_known(
-        &extra.ob_type_lookup.get_type(value),
+        extra.ob_type_lookup.get_type(value),
         value,
         serializer,
         include,
@@ -329,7 +337,7 @@ pub(crate) fn infer_serialize<S: Serializer>(
 }
 
 pub(crate) fn infer_serialize_known<S: Serializer>(
-    ob_type: &ObType,
+    ob_type: ObType,
     value: &PyAny,
     serializer: S,
     include: Option<&PyAny>,
@@ -406,7 +414,7 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
 
     let ser_result = match ob_type {
         ObType::None => serializer.serialize_none(),
-        ObType::Int | ObType::IntSubclass => serialize!(i64),
+        ObType::Int | ObType::IntSubclass => serialize!(Int),
         ObType::Bool => serialize!(bool),
         ObType::Float | ObType::FloatSubclass => serialize!(f64),
         ObType::Decimal => value.to_string().serialize(serializer),
@@ -444,11 +452,11 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
             serializer.serialize_str(&iso_time)
         }
         ObType::Timedelta => {
-            let py_timedelta: &PyDelta = value.downcast().map_err(py_err_se_err)?;
+            let either_delta = EitherTimedelta::try_from(value).map_err(py_err_se_err)?;
             extra
                 .config
                 .timedelta_mode
-                .timedelta_serialize(py_timedelta, serializer)
+                .timedelta_serialize(value.py(), &either_delta, serializer)
         }
         ObType::Url => {
             let py_url: PyUrl = value.extract().map_err(py_err_se_err)?;
@@ -463,7 +471,7 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
             let py_serializer = value
                 .getattr(intern!(py, "__pydantic_serializer__"))
                 .map_err(py_err_se_err)?;
-            let extracted_serializer: SchemaSerializer = py_serializer.extract().map_err(py_err_se_err)?;
+            let extracted_serializer: PyRef<SchemaSerializer> = py_serializer.extract().map_err(py_err_se_err)?;
             let extra = extracted_serializer.build_extra(
                 py,
                 extra.mode,
@@ -482,6 +490,11 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
             pydantic_serializer.serialize(serializer)
         }
         ObType::Dataclass => serialize_dict!(dataclass_to_dict(value).map_err(py_err_se_err)?),
+        ObType::Uuid => {
+            let py_uuid: &PyAny = value.downcast().map_err(py_err_se_err)?;
+            let uuid = super::type_serializers::uuid::uuid_to_string(py_uuid).map_err(py_err_se_err)?;
+            serializer.serialize_str(&uuid)
+        }
         ObType::Enum => {
             let v = value.getattr(intern!(value.py(), "value")).map_err(py_err_se_err)?;
             infer_serialize(v, serializer, include, exclude, extra)
@@ -518,7 +531,7 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
                 let msg = format!(
                     "{}Unable to serialize unknown type: {}",
                     SERIALIZATION_ERR_MARKER,
-                    safe_repr(value)
+                    safe_repr(value.get_type()),
                 );
                 return Err(S::Error::custom(msg));
             }
@@ -529,7 +542,10 @@ pub(crate) fn infer_serialize_known<S: Serializer>(
 }
 
 fn unknown_type_error(value: &PyAny) -> PyErr {
-    PydanticSerializationError::new_err(format!("Unable to serialize unknown type: {}", safe_repr(value)))
+    PydanticSerializationError::new_err(format!(
+        "Unable to serialize unknown type: {}",
+        safe_repr(value.get_type())
+    ))
 }
 
 fn serialize_unknown(value: &PyAny) -> Cow<str> {
@@ -544,10 +560,10 @@ fn serialize_unknown(value: &PyAny) -> Cow<str> {
 
 pub(crate) fn infer_json_key<'py>(key: &'py PyAny, extra: &Extra) -> PyResult<Cow<'py, str>> {
     let ob_type = extra.ob_type_lookup.get_type(key);
-    infer_json_key_known(&ob_type, key, extra)
+    infer_json_key_known(ob_type, key, extra)
 }
 
-pub(crate) fn infer_json_key_known<'py>(ob_type: &ObType, key: &'py PyAny, extra: &Extra) -> PyResult<Cow<'py, str>> {
+pub(crate) fn infer_json_key_known<'py>(ob_type: ObType, key: &'py PyAny, extra: &Extra) -> PyResult<Cow<'py, str>> {
     match ob_type {
         ObType::None => super::type_serializers::simple::none_json_key(),
         ObType::Int | ObType::IntSubclass | ObType::Float | ObType::FloatSubclass => {
@@ -583,9 +599,14 @@ pub(crate) fn infer_json_key_known<'py>(ob_type: &ObType, key: &'py PyAny, extra
             let iso_time = super::type_serializers::datetime_etc::time_to_string(py_time)?;
             Ok(Cow::Owned(iso_time))
         }
+        ObType::Uuid => {
+            let py_uuid: &PyAny = key.downcast()?;
+            let uuid = super::type_serializers::uuid::uuid_to_string(py_uuid)?;
+            Ok(Cow::Owned(uuid))
+        }
         ObType::Timedelta => {
-            let py_timedelta: &PyDelta = key.downcast()?;
-            extra.config.timedelta_mode.json_key(py_timedelta)
+            let either_delta = EitherTimedelta::try_from(key)?;
+            extra.config.timedelta_mode.json_key(key.py(), &either_delta)
         }
         ObType::Url => {
             let py_url: PyUrl = key.extract()?;
@@ -597,7 +618,7 @@ pub(crate) fn infer_json_key_known<'py>(ob_type: &ObType, key: &'py PyAny, extra
         }
         ObType::Tuple => {
             let mut key_build = super::type_serializers::tuple::KeyBuilder::new();
-            for element in key.downcast::<PyTuple>()?.iter() {
+            for element in key.downcast::<PyTuple>()? {
                 key_build.push(&infer_json_key(element, extra)?);
             }
             Ok(Cow::Owned(key_build.finish()))
