@@ -14,6 +14,7 @@ use serde_json::ser::PrettyFormatter;
 use crate::build_tools::py_schema_err;
 use crate::build_tools::py_schema_error_type;
 use crate::definitions::DefinitionsBuilder;
+use crate::py_gc::PyGcTraverse;
 use crate::tools::{py_err, SchemaDict};
 
 use super::errors::se_err_py_err;
@@ -113,6 +114,7 @@ combined_serializer! {
         Int: super::type_serializers::simple::IntSerializer;
         Bool: super::type_serializers::simple::BoolSerializer;
         Float: super::type_serializers::simple::FloatSerializer;
+        Decimal: super::type_serializers::decimal::DecimalSerializer;
         Str: super::type_serializers::string::StrSerializer;
         Bytes: super::type_serializers::bytes::BytesSerializer;
         Datetime: super::type_serializers::datetime_etc::DatetimeSerializer;
@@ -128,6 +130,7 @@ combined_serializer! {
         Dataclass: super::type_serializers::dataclass::DataclassSerializer;
         Url: super::type_serializers::url::UrlSerializer;
         MultiHostUrl: super::type_serializers::url::MultiHostUrlSerializer;
+        Uuid: super::type_serializers::uuid::UuidSerializer;
         Any: super::type_serializers::any::AnySerializer;
         Format: super::type_serializers::format::FormatSerializer;
         ToString: super::type_serializers::format::ToStringSerializer;
@@ -215,12 +218,52 @@ impl BuildSerializer for CombinedSerializer {
     }
 }
 
+// Implemented by hand because `enum_dispatch` fails with a proc macro compile error =/
+impl PyGcTraverse for CombinedSerializer {
+    fn py_gc_traverse(&self, visit: &PyVisit<'_>) -> Result<(), PyTraverseError> {
+        match self {
+            CombinedSerializer::Function(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::FunctionWrap(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Fields(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::None(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Nullable(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Int(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Bool(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Float(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Decimal(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Str(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Bytes(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Datetime(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::TimeDelta(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Date(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Time(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::List(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Set(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::FrozenSet(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Generator(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Dict(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Model(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Dataclass(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Url(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::MultiHostUrl(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Any(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Format(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::ToString(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::WithDefault(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Json(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::JsonOrPython(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Union(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Literal(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Recursive(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::TuplePositional(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::TupleVariable(inner) => inner.py_gc_traverse(visit),
+            CombinedSerializer::Uuid(inner) => inner.py_gc_traverse(visit),
+        }
+    }
+}
+
 #[enum_dispatch(CombinedSerializer)]
 pub(crate) trait TypeSerializer: Send + Sync + Clone + Debug {
-    fn py_gc_traverse(&self, _visit: &PyVisit<'_>) -> Result<(), PyTraverseError> {
-        Ok(())
-    }
-    fn py_gc_clear(&mut self) {}
     fn to_python(
         &self,
         value: &PyAny,
@@ -308,11 +351,11 @@ pub(crate) fn to_json_bytes(
     exclude: Option<&PyAny>,
     extra: &Extra,
     indent: Option<usize>,
-    json_size: usize,
+    expected_json_size: usize,
 ) -> PyResult<Vec<u8>> {
     let serializer = PydanticSerializer::new(value, serializer, include, exclude, extra);
 
-    let writer: Vec<u8> = Vec::with_capacity(json_size);
+    let writer: Vec<u8> = Vec::with_capacity(expected_json_size);
     let bytes = match indent {
         Some(indent) => {
             let indent = vec![b' '; indent];
@@ -347,7 +390,7 @@ pub(super) fn dataclass_to_dict(dc: &PyAny) -> PyResult<&PyDict> {
     let dict = PyDict::new(py);
 
     let field_type_marker = get_field_marker(py)?;
-    for (field_name, field) in dc_fields.iter() {
+    for (field_name, field) in dc_fields {
         let field_type = field.getattr(intern!(py, "_field_type"))?;
         if field_type.is(field_type_marker) {
             let field_name: &PyString = field_name.downcast()?;
