@@ -1,8 +1,8 @@
 use std::borrow::Cow;
 
+use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PySet, PyString, PyType};
-use pyo3::{intern, PyTraverseError, PyVisit};
 
 use ahash::AHashMap;
 
@@ -38,7 +38,7 @@ impl BuildSerializer for ModelFieldsBuilder {
         let fields_dict: &PyDict = schema.get_as_req(intern!(py, "fields"))?;
         let mut fields: AHashMap<String, SerField> = AHashMap::with_capacity(fields_dict.len());
 
-        for (key, value) in fields_dict.iter() {
+        for (key, value) in fields_dict {
             let key_py: &PyString = key.downcast()?;
             let key: String = key_py.extract()?;
             let field_info: &PyDict = value.downcast()?;
@@ -144,13 +144,9 @@ impl ModelSerializer {
     }
 }
 
-impl TypeSerializer for ModelSerializer {
-    fn py_gc_traverse(&self, visit: &PyVisit<'_>) -> Result<(), PyTraverseError> {
-        visit.call(&self.class)?;
-        self.serializer.py_gc_traverse(visit)?;
-        Ok(())
-    }
+impl_py_gc_traverse!(ModelSerializer { class, serializer });
 
+impl TypeSerializer for ModelSerializer {
     fn to_python(
         &self,
         value: &PyAny,
@@ -185,7 +181,7 @@ impl TypeSerializer for ModelSerializer {
 
     fn json_key<'py>(&self, key: &'py PyAny, extra: &Extra) -> PyResult<Cow<'py, str>> {
         if self.allow_value(key, extra)? {
-            infer_json_key_known(&ObType::PydanticSerializable, key, extra)
+            infer_json_key_known(ObType::PydanticSerializable, key, extra)
         } else {
             extra.warnings.on_fallback_py(&self.name, key, extra)?;
             infer_json_key(key, extra)
