@@ -47,6 +47,8 @@ impl BuildSerializer for TupleVariableSerializer {
     }
 }
 
+impl_py_gc_traverse!(TupleVariableSerializer { item_serializer });
+
 impl TypeSerializer for TupleVariableSerializer {
     fn to_python(
         &self,
@@ -87,7 +89,7 @@ impl TypeSerializer for TupleVariableSerializer {
                 let item_serializer = self.item_serializer.as_ref();
 
                 let mut key_builder = KeyBuilder::new();
-                for element in py_tuple.iter() {
+                for element in py_tuple {
                     key_builder.push(&item_serializer.json_key(element, extra)?);
                 }
                 Ok(Cow::Owned(key_builder.finish()))
@@ -181,6 +183,11 @@ impl BuildSerializer for TuplePositionalSerializer {
     }
 }
 
+impl_py_gc_traverse!(TuplePositionalSerializer {
+    items_serializers,
+    extra_serializer
+});
+
 impl TypeSerializer for TuplePositionalSerializer {
     fn to_python(
         &self,
