@@ -5,10 +5,11 @@ use pyo3::types::{PyDict, PyTuple};
 
 use crate::errors::ValResult;
 use crate::input::Input;
-use crate::recursion_guard::RecursionGuard;
+
 use crate::tools::SchemaDict;
 
-use super::{build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+use super::validation_state::ValidationState;
+use super::{build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, Validator};
 
 #[derive(Debug, Clone)]
 pub struct CallValidator {
@@ -65,18 +66,19 @@ impl BuildValidator for CallValidator {
     }
 }
 
+impl_py_gc_traverse!(CallValidator {
+    arguments_validator,
+    return_validator
+});
+
 impl Validator for CallValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        definitions: &'data Definitions<CombinedValidator>,
-        recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let args = self
-            .arguments_validator
-            .validate(py, input, extra, definitions, recursion_guard)?;
+        let args = self.arguments_validator.validate(py, input, state)?;
 
         let return_value = if let Ok((args, kwargs)) = args.extract::<(&PyTuple, &PyDict)>(py) {
             self.function.call(py, args, Some(kwargs))?
@@ -89,7 +91,7 @@ impl Validator for CallValidator {
 
         if let Some(return_validator) = &self.return_validator {
             return_validator
-                .validate(py, return_value.into_ref(py), extra, definitions, recursion_guard)
+                .validate(py, return_value.into_ref(py), state)
                 .map_err(|e| e.with_outer_location("return".into()))
         } else {
             Ok(return_value.to_object(py))
