@@ -1,11 +1,10 @@
 use pyo3::prelude::*;
 use pyo3::types::PyDict;
 
-use crate::errors::{ErrorType, ValError, ValResult};
+use crate::errors::{ErrorTypeDefaults, ValError, ValResult};
 use crate::input::Input;
-use crate::recursion_guard::RecursionGuard;
 
-use super::{BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
 #[derive(Debug, Clone)]
 pub struct CallableValidator;
@@ -22,18 +21,18 @@ impl BuildValidator for CallableValidator {
     }
 }
 
+impl_py_gc_traverse!(CallableValidator {});
+
 impl Validator for CallableValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        _extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        _state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         match input.callable() {
             true => Ok(input.to_object(py)),
-            false => Err(ValError::new(ErrorType::CallableType, input)),
+            false => Err(ValError::new(ErrorTypeDefaults::CallableType, input)),
         }
     }
 
