@@ -1,32 +1,52 @@
-use pyo3::exceptions::{PyAssertionError, PyAttributeError, PyRuntimeError, PyTypeError, PyValueError};
-use pyo3::intern;
+use pyo3::exceptions::{PyAssertionError, PyAttributeError, PyTypeError, PyValueError};
 use pyo3::prelude::*;
 use pyo3::types::{PyAny, PyDict, PyString};
+use pyo3::{intern, PyTraverseError, PyVisit};
 
 use crate::errors::{
     ErrorType, LocItem, PydanticCustomError, PydanticKnownError, PydanticOmit, ValError, ValResult, ValidationError,
 };
 use crate::input::Input;
-use crate::recursion_guard::RecursionGuard;
+use crate::py_gc::PyGcTraverse;
 use crate::tools::{function_name, py_err, SchemaDict};
 use crate::PydanticUseDefault;
 
 use super::generator::InternalValidator;
 use super::{
-    build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, InputType, Validator,
+    build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, Extra, InputType, ValidationState,
+    Validator,
 };
 
-fn destructure_function_schema(schema: &PyDict) -> PyResult<(bool, bool, &PyAny)> {
+struct FunctionInfo {
+    /// The actual function object that will get called
+    pub function: Py<PyAny>,
+    pub field_name: Option<Py<PyString>>,
+    pub info_arg: bool,
+}
+
+fn destructure_function_schema(schema: &PyDict) -> PyResult<FunctionInfo> {
     let func_dict: &PyDict = schema.get_as_req(intern!(schema.py(), "function"))?;
     let function: &PyAny = func_dict.get_as_req(intern!(schema.py(), "function"))?;
     let func_type: &str = func_dict.get_as_req(intern!(schema.py(), "type"))?;
-    let (is_field_serializer, info_arg) = match func_type {
+    let (is_field_validator, info_arg) = match func_type {
         "field" => (true, true),
         "general" => (false, true),
         "no-info" => (false, false),
         _ => unreachable!(),
     };
-    Ok((is_field_serializer, info_arg, function))
+    let field_name: Option<Py<PyString>> = match is_field_validator {
+        true => Some(
+            func_dict
+                .get_as_req::<&PyString>(intern!(schema.py(), "field_name"))?
+                .into(),
+        ),
+        false => None,
+    };
+    Ok(FunctionInfo {
+        function: function.into(),
+        field_name,
+        info_arg,
+    })
 }
 
 macro_rules! impl_build {
@@ -40,23 +60,23 @@ macro_rules! impl_build {
             ) -> PyResult<CombinedValidator> {
                 let py = schema.py();
                 let validator = build_validator(schema.get_as_req(intern!(py, "schema"))?, config, definitions)?;
-                let (is_field_validator, info_arg, function) = destructure_function_schema(schema)?;
+                let func_info = destructure_function_schema(schema)?;
                 let name = format!(
                     "{}[{}(), {}]",
                     $name,
-                    function_name(function)?,
+                    function_name(func_info.function.as_ref(py))?,
                     validator.get_name()
                 );
                 Ok(Self {
                     validator: Box::new(validator),
-                    func: function.into_py(py),
+                    func: func_info.function,
                     config: match config {
                         Some(c) => c.into(),
                         None => py.None(),
                     },
                     name,
-                    is_field_validator,
-                    info_arg,
+                    field_name: func_info.field_name,
+                    info_arg: func_info.info_arg,
                 }
                 .into())
             }
@@ -66,34 +86,35 @@ macro_rules! impl_build {
 
 macro_rules! impl_validator {
     ($name:ident) => {
+        impl_py_gc_traverse!($name {
+            validator,
+            func,
+            config
+        });
+
         impl Validator for $name {
-            fn validate<'s, 'data>(
-                &'s self,
+            fn validate<'data>(
+                &self,
                 py: Python<'data>,
                 input: &'data impl Input<'data>,
-                extra: &Extra,
-                definitions: &'data Definitions<CombinedValidator>,
-                recursion_guard: &'s mut RecursionGuard,
+                state: &mut ValidationState<'_>,
             ) -> ValResult<'data, PyObject> {
-                let validate =
-                    move |v: &'data PyAny, e: &Extra| self.validator.validate(py, v, e, definitions, recursion_guard);
-                self._validate(validate, py, input.to_object(py).into_ref(py), extra)
+                let validate = |v, s: &mut ValidationState<'_>| self.validator.validate(py, v, s);
+                self._validate(validate, py, input.to_object(py).into_ref(py), state)
             }
-            fn validate_assignment<'s, 'data: 's>(
-                &'s self,
+            fn validate_assignment<'data>(
+                &self,
                 py: Python<'data>,
                 obj: &'data PyAny,
                 field_name: &'data str,
                 field_value: &'data PyAny,
-                extra: &Extra,
-                definitions: &'data Definitions<CombinedValidator>,
-                recursion_guard: &'s mut RecursionGuard,
+                state: &mut ValidationState,
             ) -> ValResult<'data, PyObject> {
-                let validate = move |v: &'data PyAny, e: &Extra| {
+                let validate = move |v, s: &mut ValidationState<'_>| {
                     self.validator
-                        .validate_assignment(py, v, field_name, field_value, e, definitions, recursion_guard)
+                        .validate_assignment(py, v, field_name, field_value, s)
                 };
-                self._validate(validate, py, obj, extra)
+                self._validate(validate, py, obj, state)
             }
 
             fn different_strict_behavior(
@@ -126,7 +147,7 @@ pub struct FunctionBeforeValidator {
     func: PyObject,
     config: PyObject,
     name: String,
-    is_field_validator: bool,
+    field_name: Option<Py<PyString>>,
     info_arg: bool,
 }
 
@@ -135,19 +156,19 @@ impl_build!(FunctionBeforeValidator, "function-before");
 impl FunctionBeforeValidator {
     fn _validate<'s, 'data>(
         &'s self,
-        mut call: impl FnMut(&'data PyAny, &Extra) -> ValResult<'data, PyObject>,
+        call: impl FnOnce(&'data PyAny, &mut ValidationState<'_>) -> ValResult<'data, PyObject>,
         py: Python<'data>,
         input: &'data PyAny,
-        extra: &Extra,
+        state: &'s mut ValidationState<'_>,
     ) -> ValResult<'data, PyObject> {
         let r = if self.info_arg {
-            let info = ValidationInfo::new(py, extra, &self.config, self.is_field_validator)?;
+            let info = ValidationInfo::new(py, state.extra(), &self.config, self.field_name.clone());
             self.func.call1(py, (input.to_object(py), info))
         } else {
             self.func.call1(py, (input.to_object(py),))
         };
         let value = r.map_err(|e| convert_err(py, e, input))?;
-        call(value.into_ref(py), extra)
+        call(value.into_ref(py), state)
     }
 }
 
@@ -159,7 +180,7 @@ pub struct FunctionAfterValidator {
     func: PyObject,
     config: PyObject,
     name: String,
-    is_field_validator: bool,
+    field_name: Option<Py<PyString>>,
     info_arg: bool,
 }
 
@@ -168,14 +189,14 @@ impl_build!(FunctionAfterValidator, "function-after");
 impl FunctionAfterValidator {
     fn _validate<'s, 'data>(
         &'s self,
-        mut call: impl FnMut(&'data PyAny, &Extra) -> ValResult<'data, PyObject>,
+        call: impl FnOnce(&'data PyAny, &mut ValidationState<'_>) -> ValResult<'data, PyObject>,
         py: Python<'data>,
         input: &'data PyAny,
-        extra: &Extra,
+        state: &mut ValidationState<'_>,
     ) -> ValResult<'data, PyObject> {
-        let v = call(input, extra)?;
+        let v = call(input, state)?;
         let r = if self.info_arg {
-            let info = ValidationInfo::new(py, extra, &self.config, self.is_field_validator)?;
+            let info = ValidationInfo::new(py, state.extra(), &self.config, self.field_name.clone());
             self.func.call1(py, (v.to_object(py), info))
         } else {
             self.func.call1(py, (v.to_object(py),))
@@ -191,7 +212,7 @@ pub struct FunctionPlainValidator {
     func: PyObject,
     config: PyObject,
     name: String,
-    is_field_validator: bool,
+    field_name: Option<Py<PyString>>,
     info_arg: bool,
 }
 
@@ -204,32 +225,35 @@ impl BuildValidator for FunctionPlainValidator {
         _definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
-        let (is_field_validator, info_arg, function) = destructure_function_schema(schema)?;
+        let function_info = destructure_function_schema(schema)?;
         Ok(Self {
-            func: function.into_py(py),
+            func: function_info.function.clone(),
             config: match config {
                 Some(c) => c.into(),
                 None => py.None(),
             },
-            name: format!("function-plain[{}()]", function_name(function)?),
-            is_field_validator,
-            info_arg,
+            name: format!(
+                "function-plain[{}()]",
+                function_name(function_info.function.as_ref(py))?
+            ),
+            field_name: function_info.field_name.clone(),
+            info_arg: function_info.info_arg,
         }
         .into())
     }
 }
 
+impl_py_gc_traverse!(FunctionPlainValidator { func, config });
+
 impl Validator for FunctionPlainValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         let r = if self.info_arg {
-            let info = ValidationInfo::new(py, extra, &self.config, self.is_field_validator)?;
+            let info = ValidationInfo::new(py, state.extra(), &self.config, self.field_name.clone());
             self.func.call1(py, (input.to_object(py), info))
         } else {
             self.func.call1(py, (input.to_object(py),))
@@ -261,7 +285,7 @@ pub struct FunctionWrapValidator {
     func: PyObject,
     config: PyObject,
     name: String,
-    is_field_validator: bool,
+    field_name: Option<Py<PyString>>,
     info_arg: bool,
     hide_input_in_errors: bool,
 }
@@ -276,18 +300,18 @@ impl BuildValidator for FunctionWrapValidator {
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
         let validator = build_validator(schema.get_as_req(intern!(py, "schema"))?, config, definitions)?;
-        let (is_field_validator, info_arg, function) = destructure_function_schema(schema)?;
+        let function_info = destructure_function_schema(schema)?;
         let hide_input_in_errors: bool = config.get_as(intern!(py, "hide_input_in_errors"))?.unwrap_or(false);
         Ok(Self {
             validator: Box::new(validator),
-            func: function.into_py(py),
+            func: function_info.function.clone(),
             config: match config {
                 Some(c) => c.into(),
                 None => py.None(),
             },
-            name: format!("function-wrap[{}()]", function_name(function)?),
-            is_field_validator,
-            info_arg,
+            name: format!("function-wrap[{}()]", function_name(function_info.function.as_ref(py))?),
+            field_name: function_info.field_name.clone(),
+            info_arg: function_info.info_arg,
             hide_input_in_errors,
         }
         .into())
@@ -300,10 +324,10 @@ impl FunctionWrapValidator {
         handler: &'s PyAny,
         py: Python<'data>,
         input: &'data PyAny,
-        extra: &Extra,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         let r = if self.info_arg {
-            let info = ValidationInfo::new(py, extra, &self.config, self.is_field_validator)?;
+            let info = ValidationInfo::new(py, state.extra(), &self.config, self.field_name.clone());
             self.func.call1(py, (input.to_object(py), handler, info))
         } else {
             self.func.call1(py, (input.to_object(py), handler))
@@ -312,23 +336,25 @@ impl FunctionWrapValidator {
     }
 }
 
+impl_py_gc_traverse!(FunctionWrapValidator {
+    validator,
+    func,
+    config
+});
+
 impl Validator for FunctionWrapValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        definitions: &'data Definitions<CombinedValidator>,
-        recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         let handler = ValidatorCallable {
             validator: InternalValidator::new(
                 py,
                 "ValidatorCallable",
                 &self.validator,
-                definitions,
-                extra,
-                recursion_guard,
+                state,
                 self.hide_input_in_errors,
             ),
         };
@@ -336,34 +362,30 @@ impl Validator for FunctionWrapValidator {
             Py::new(py, handler)?.into_ref(py),
             py,
             input.to_object(py).into_ref(py),
-            extra,
+            state,
         )
     }
 
-    fn validate_assignment<'s, 'data: 's>(
-        &'s self,
+    fn validate_assignment<'data>(
+        &self,
         py: Python<'data>,
         obj: &'data PyAny,
         field_name: &'data str,
         field_value: &'data PyAny,
-        extra: &Extra,
-        definitions: &'data Definitions<CombinedValidator>,
-        recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         let handler = AssignmentValidatorCallable {
             validator: InternalValidator::new(
                 py,
-                "ValidatorCallable",
+                "AssignmentValidatorCallable",
                 &self.validator,
-                definitions,
-                extra,
-                recursion_guard,
+                state,
                 self.hide_input_in_errors,
             ),
             updated_field_name: field_name.to_string(),
             updated_field_value: field_value.to_object(py),
         };
-        self._validate(Py::new(py, handler)?.into_ref(py), py, obj, extra)
+        self._validate(Py::new(py, handler)?.into_ref(py), py, obj, state)
     }
 
     fn different_strict_behavior(
@@ -413,6 +435,10 @@ impl ValidatorCallable {
     fn __str__(&self) -> String {
         self.__repr__()
     }
+
+    fn __traverse__(&self, visit: PyVisit) -> Result<(), PyTraverseError> {
+        self.validator.py_gc_traverse(&visit)
+    }
 }
 
 #[pyclass(module = "pydantic_core._pydantic_core")]
@@ -455,13 +481,13 @@ macro_rules! py_err_string {
     ($error_value:expr, $type_member:ident, $input:ident) => {
         match $error_value.str() {
             Ok(py_string) => match py_string.to_str() {
-                Ok(s) => {
-                    let error = match s.is_empty() {
-                        true => "Unknown error".to_string(),
-                        false => s.to_string(),
-                    };
-                    ValError::new(ErrorType::$type_member { error }, $input)
-                }
+                Ok(_) => ValError::new(
+                    ErrorType::$type_member {
+                        error: Some($error_value.into()),
+                        context: None,
+                    },
+                    $input,
+                ),
                 Err(e) => ValError::InternalErr(e),
             },
             Err(e) => ValError::InternalErr(e),
@@ -500,34 +526,19 @@ pub struct ValidationInfo {
     #[pyo3(get)]
     context: Option<PyObject>,
     data: Option<Py<PyDict>>,
-    field_name: Option<String>,
+    field_name: Option<Py<PyString>>,
     #[pyo3(get)]
     mode: InputType,
 }
 
 impl ValidationInfo {
-    fn new(py: Python, extra: &Extra, config: &PyObject, is_field_validator: bool) -> PyResult<Self> {
-        if is_field_validator {
-            match extra.field_name {
-                Some(field_name) => Ok(
-                    Self {
-                        config: config.clone_ref(py),
-                        context: extra.context.map(Into::into),
-                        field_name: Some(field_name.to_string()),
-                        data: extra.data.map(Into::into),
-                        mode: extra.mode,
-                    }
-                ),
-                _ => Err(PyRuntimeError::new_err("This validator expected to be run inside the context of a model field but no model field was found")),
-            }
-        } else {
-            Ok(Self {
-                config: config.clone_ref(py),
-                context: extra.context.map(Into::into),
-                field_name: None,
-                data: None,
-                mode: extra.mode,
-            })
+    fn new(py: Python, extra: &Extra, config: &PyObject, field_name: Option<Py<PyString>>) -> Self {
+        Self {
+            config: config.clone_ref(py),
+            context: extra.context.map(Into::into),
+            field_name,
+            data: extra.data.map(Into::into),
+            mode: extra.mode,
         }
     }
 }
@@ -536,16 +547,16 @@ impl ValidationInfo {
 impl ValidationInfo {
     #[getter]
     fn get_data(&self, py: Python) -> PyResult<Py<PyDict>> {
-        match self.data {
-            Some(ref data) => Ok(data.clone_ref(py)),
-            None => Err(PyAttributeError::new_err("No attribute named 'data'")),
+        match (&self.data, &self.field_name) {
+            (Some(data), Some(_)) => Ok(data.clone_ref(py)),
+            _ => Err(PyAttributeError::new_err("No attribute named 'data'")),
         }
     }
 
     #[getter]
-    fn get_field_name<'py>(&self, py: Python<'py>) -> PyResult<&'py PyString> {
+    fn get_field_name(&self) -> PyResult<Py<PyString>> {
         match self.field_name {
-            Some(ref field_name) => Ok(PyString::new(py, field_name)),
+            Some(ref field_name) => Ok(field_name.clone()),
             None => Err(PyAttributeError::new_err("No attribute named 'field_name'")),
         }
     }
@@ -556,11 +567,15 @@ impl ValidationInfo {
             None => "None",
         };
         let config = self.config.as_ref(py).repr()?;
-        let mut s = format!("ValidationInfo(config={config}, context={context}");
-        if let Some(ref data) = self.data {
+        let mut s = if self.field_name.is_some() {
+            format!("FieldValidationInfo(config={config}, context={context}")
+        } else {
+            format!("ValidationInfo(config={config}, context={context}")
+        };
+        if let Ok(data) = self.get_data(py) {
             s += &format!(", data={}", data.as_ref(py).repr()?);
         }
-        if let Some(ref field_name) = self.field_name {
+        if let Ok(field_name) = self.get_field_name() {
             s += &format!(", field_name='{field_name}'");
         }
         s += ")";
