@@ -6,11 +6,10 @@ use pyo3::types::{PyDict, PyType};
 use crate::build_tools::py_schema_err;
 use crate::errors::{ErrorType, ValError, ValResult};
 use crate::input::Input;
-use crate::recursion_guard::RecursionGuard;
 use crate::tools::SchemaDict;
 
-use super::InputType;
-use super::{BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+use super::ValidationState;
+use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, Validator};
 
 #[derive(Debug, Clone)]
 pub struct IsInstanceValidator {
@@ -55,32 +54,32 @@ impl BuildValidator for IsInstanceValidator {
     }
 }
 
+impl_py_gc_traverse!(IsInstanceValidator { class });
+
 impl Validator for IsInstanceValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        _state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        match extra.mode {
-            InputType::Json => Err(ValError::InternalErr(PyNotImplementedError::new_err(
-                "Cannot check isinstance when validating from json,\
+        if !input.is_python() {
+            return Err(ValError::InternalErr(PyNotImplementedError::new_err(
+                "Cannot check isinstance when validating from json, \
                             use a JsonOrPython validator instead.",
-            ))),
-            InputType::Python => {
-                let ob = input.to_object(py);
-                match ob.as_ref(py).is_instance(self.class.as_ref(py))? {
-                    true => Ok(ob),
-                    false => Err(ValError::new(
-                        ErrorType::IsInstanceOf {
-                            class: self.class_repr.clone(),
-                        },
-                        input,
-                    )),
-                }
-            }
+            )));
+        }
+
+        let ob = input.to_object(py);
+        match ob.as_ref(py).is_instance(self.class.as_ref(py))? {
+            true => Ok(ob),
+            false => Err(ValError::new(
+                ErrorType::IsInstanceOf {
+                    class: self.class_repr.clone(),
+                    context: None,
+                },
+                input,
+            )),
         }
     }
 
