@@ -3,24 +3,31 @@
 use core::fmt::Debug;
 
 use ahash::AHashMap;
-use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList};
+use pyo3::{intern, PyTraverseError, PyVisit};
 
 use crate::build_tools::{py_schema_err, py_schema_error_type};
 use crate::errors::{ErrorType, ValError, ValResult};
 use crate::input::Input;
-use crate::recursion_guard::RecursionGuard;
+use crate::py_gc::PyGcTraverse;
 use crate::tools::SchemaDict;
 
-use super::{BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
+
+#[derive(Debug, Clone)]
+struct BoolLiteral {
+    pub true_id: Option<usize>,
+    pub false_id: Option<usize>,
+}
 
 #[derive(Debug, Clone)]
 pub struct LiteralLookup<T: Clone + Debug> {
-    // Specialized lookups for ints and strings because they
+    // Specialized lookups for ints, bools and strings because they
     // (1) are easy to convert between Rust and Python
     // (2) hashing them in Rust is very fast
     // (3) are the most commonly used things in Literal[...]
+    expected_bool: Option<BoolLiteral>,
     expected_int: Option<AHashMap<i64, usize>>,
     expected_str: Option<AHashMap<String, usize>>,
     // Catch all for Enum and bytes (the latter only because it is seldom used)
@@ -31,12 +38,23 @@ pub struct LiteralLookup<T: Clone + Debug> {
 impl<T: Clone + Debug> LiteralLookup<T> {
     pub fn new<'py>(py: Python<'py>, expected: impl Iterator<Item = (&'py PyAny, T)>) -> PyResult<Self> {
         let mut expected_int = AHashMap::new();
-        let mut expected_str = AHashMap::new();
+        let mut expected_str: AHashMap<String, usize> = AHashMap::new();
+        let mut expected_bool = BoolLiteral {
+            true_id: None,
+            false_id: None,
+        };
         let expected_py = PyDict::new(py);
         let mut values = Vec::new();
         for (k, v) in expected {
             let id = values.len();
             values.push(v);
+            if let Ok(bool) = k.strict_bool() {
+                if bool {
+                    expected_bool.true_id = Some(id);
+                } else {
+                    expected_bool.false_id = Some(id);
+                }
+            }
             if let Ok(either_int) = k.exact_int() {
                 let int = either_int
                     .into_i64(py)
@@ -53,6 +71,10 @@ impl<T: Clone + Debug> LiteralLookup<T> {
         }
 
         Ok(Self {
+            expected_bool: match expected_bool.true_id.is_some() || expected_bool.false_id.is_some() {
+                true => Some(expected_bool),
+                false => None,
+            },
             expected_int: match expected_int.is_empty() {
                 true => None,
                 false => Some(expected_int),
@@ -74,7 +96,17 @@ impl<T: Clone + Debug> LiteralLookup<T> {
         py: Python<'data>,
         input: &'data I,
     ) -> ValResult<'data, Option<(&'data I, &T)>> {
-        // dbg!(input.to_object(py).as_ref(py).repr().unwrap());
+        if let Some(expected_bool) = &self.expected_bool {
+            if let Ok(bool_value) = input.strict_bool() {
+                if bool_value {
+                    if let Some(true_value) = &expected_bool.true_id {
+                        return Ok(Some((input, &self.values[*true_value])));
+                    }
+                } else if let Some(false_value) = &expected_bool.false_id {
+                    return Ok(Some((input, &self.values[*false_value])));
+                }
+            }
+        }
         if let Some(expected_ints) = &self.expected_int {
             if let Ok(either_int) = input.exact_int() {
                 let int = either_int.into_i64(py)?;
@@ -103,6 +135,14 @@ impl<T: Clone + Debug> LiteralLookup<T> {
     }
 }
 
+impl<T: PyGcTraverse + Clone + Debug> PyGcTraverse for LiteralLookup<T> {
+    fn py_gc_traverse(&self, visit: &PyVisit<'_>) -> Result<(), PyTraverseError> {
+        self.expected_py.py_gc_traverse(visit)?;
+        self.values.py_gc_traverse(visit)?;
+        Ok(())
+    }
+}
+
 #[derive(Debug, Clone)]
 pub struct LiteralValidator {
     lookup: LiteralLookup<PyObject>,
@@ -124,7 +164,7 @@ impl BuildValidator for LiteralValidator {
         }
         let py = expected.py();
         let mut repr_args: Vec<String> = Vec::new();
-        for item in expected.iter() {
+        for item in expected {
             repr_args.push(item.repr()?.extract()?);
         }
         let (expected_repr, name) = expected_repr_name(repr_args, "literal");
@@ -137,20 +177,21 @@ impl BuildValidator for LiteralValidator {
     }
 }
 
+impl_py_gc_traverse!(LiteralValidator { lookup });
+
 impl Validator for LiteralValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        _extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        _state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         match self.lookup.validate(py, input)? {
             Some((_, v)) => Ok(v.clone()),
             None => Err(ValError::new(
                 ErrorType::LiteralError {
                     expected: self.expected_repr.clone(),
+                    context: None,
                 },
                 input,
             )),
