@@ -9,9 +9,10 @@ use pyo3::types::{PyAny, PyDict, PyTuple, PyType};
 use pyo3::{intern, PyTraverseError, PyVisit};
 
 use crate::build_tools::{py_schema_err, py_schema_error_type, SchemaError};
-use crate::definitions::{Definitions, DefinitionsBuilder};
+use crate::definitions::DefinitionsBuilder;
 use crate::errors::{ErrorMode, LocItem, ValError, ValResult, ValidationError};
 use crate::input::{Input, InputType};
+use crate::py_gc::PyGcTraverse;
 use crate::recursion_guard::RecursionGuard;
 use crate::tools::SchemaDict;
 
@@ -26,6 +27,7 @@ mod custom_error;
 mod dataclass;
 mod date;
 mod datetime;
+pub(crate) mod decimal;
 mod definitions;
 mod dict;
 mod float;
@@ -52,11 +54,14 @@ mod tuple;
 mod typed_dict;
 mod union;
 mod url;
+mod uuid;
+mod validation_state;
 mod with_default;
 
 pub use with_default::DefaultType;
 
 use self::definitions::DefinitionRefValidator;
+pub use self::validation_state::ValidationState;
 
 #[pyclass(module = "pydantic_core._pydantic_core", name = "Some")]
 pub struct PySome {
@@ -153,7 +158,7 @@ impl SchemaValidator {
         context: Option<&PyAny>,
         self_instance: Option<&PyAny>,
     ) -> PyResult<PyObject> {
-        let r = self._validate(
+        self._validate(
             py,
             input,
             InputType::Python,
@@ -161,8 +166,9 @@ impl SchemaValidator {
             from_attributes,
             context,
             self_instance,
-        );
-        r.map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Python))
+            &mut RecursionGuard::default(),
+        )
+        .map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Python))
     }
 
     #[pyo3(signature = (input, *, strict=None, from_attributes=None, context=None, self_instance=None))]
@@ -183,6 +189,7 @@ impl SchemaValidator {
             from_attributes,
             context,
             self_instance,
+            &mut RecursionGuard::default(),
         ) {
             Ok(_) => Ok(true),
             Err(ValError::InternalErr(err)) => Err(err),
@@ -201,11 +208,20 @@ impl SchemaValidator {
         context: Option<&PyAny>,
         self_instance: Option<&PyAny>,
     ) -> PyResult<PyObject> {
+        let recursion_guard = &mut RecursionGuard::default();
         match input.parse_json() {
-            Ok(input) => {
-                let r = self._validate(py, &input, InputType::Json, strict, None, context, self_instance);
-                r.map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Json))
-            }
+            Ok(input) => self
+                ._validate(
+                    py,
+                    &input,
+                    InputType::Json,
+                    strict,
+                    None,
+                    context,
+                    self_instance,
+                    recursion_guard,
+                )
+                .map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Json)),
             Err(err) => Err(self.prepare_validation_err(py, err, ErrorMode::Json)),
         }
     }
@@ -229,13 +245,13 @@ impl SchemaValidator {
             from_attributes,
             ultra_strict: false,
             context,
-            field_name: None,
             self_instance: None,
         };
 
         let guard = &mut RecursionGuard::default();
+        let mut state = ValidationState::new(extra, &self.definitions, guard);
         self.validator
-            .validate_assignment(py, obj, field_name, field_value, &extra, &self.definitions, guard)
+            .validate_assignment(py, obj, field_name, field_value, &mut state)
             .map_err(|e| self.prepare_validation_err(py, e, ErrorMode::Python))
     }
 
@@ -248,13 +264,11 @@ impl SchemaValidator {
             from_attributes: None,
             ultra_strict: false,
             context,
-            field_name: None,
             self_instance: None,
         };
         let recursion_guard = &mut RecursionGuard::default();
-        let r = self
-            .validator
-            .default_value(py, None::<i64>, &extra, &self.definitions, recursion_guard);
+        let mut state = ValidationState::new(extra, &self.definitions, recursion_guard);
+        let r = self.validator.default_value(py, None::<i64>, &mut state);
         match r {
             Ok(maybe_default) => match maybe_default {
                 Some(v) => Ok(PySome::new(v).into_py(py)),
@@ -281,13 +295,6 @@ impl SchemaValidator {
         }
         Ok(())
     }
-
-    fn __clear__(&mut self) {
-        self.validator.py_gc_clear();
-        for slot in &mut self.definitions {
-            slot.py_gc_clear();
-        }
-    }
 }
 
 impl SchemaValidator {
@@ -301,17 +308,17 @@ impl SchemaValidator {
         from_attributes: Option<bool>,
         context: Option<&'data PyAny>,
         self_instance: Option<&PyAny>,
+        recursion_guard: &'data mut RecursionGuard,
     ) -> ValResult<'data, PyObject>
     where
         's: 'data,
     {
-        self.validator.validate(
-            py,
-            input,
-            &Extra::new(strict, from_attributes, context, self_instance, mode),
+        let mut state = ValidationState::new(
+            Extra::new(strict, from_attributes, context, self_instance, mode),
             &self.definitions,
-            &mut RecursionGuard::default(),
-        )
+            recursion_guard,
+        );
+        self.validator.validate(py, input, &mut state)
     }
 
     fn prepare_validation_err(&self, py: Python, error: ValError, error_mode: ErrorMode) -> PyErr {
@@ -343,14 +350,13 @@ impl<'py> SelfValidator<'py> {
     }
 
     pub fn validate_schema(&self, py: Python<'py>, schema: &'py PyAny) -> PyResult<&'py PyAny> {
-        let extra = Extra::new(None, None, None, None, InputType::Python);
-        match self.validator.validator.validate(
-            py,
-            schema,
-            &extra,
+        let mut recursion_guard = RecursionGuard::default();
+        let mut state = ValidationState::new(
+            Extra::new(None, None, None, None, InputType::Python),
             &self.validator.definitions,
-            &mut RecursionGuard::default(),
-        ) {
+            &mut recursion_guard,
+        );
+        match self.validator.validator.validate(py, schema, &mut state) {
             Ok(schema_obj) => Ok(schema_obj.into_ref(py)),
             Err(e) => Err(SchemaError::from_val_error(py, e)),
         }
@@ -458,6 +464,8 @@ pub fn build_validator<'a>(
         bool::BoolValidator,
         // floats
         float::FloatBuilder,
+        // decimals
+        decimal::DecimalValidator,
         // tuples
         tuple::TuplePositionalValidator,
         tuple::TupleVariableValidator,
@@ -515,6 +523,8 @@ pub fn build_validator<'a>(
         // url types
         url::UrlValidator,
         url::MultiHostUrlValidator,
+        // uuid types
+        uuid::UuidValidator,
         // recursive (self-referencing) models
         definitions::DefinitionRefValidator,
         definitions::DefinitionsValidatorBuilder,
@@ -529,9 +539,6 @@ pub struct Extra<'a> {
     pub mode: InputType,
     /// This is used as the `data` kwargs to validator functions
     pub data: Option<&'a PyDict>,
-    /// Represents the fields of the model we are currently validating
-    /// If there is no model this will be None
-    pub field_name: Option<&'a str>,
     /// whether we're in strict or lax mode
     pub strict: Option<bool>,
     /// whether we're in ultra-strict mode, only used occasionally in unions
@@ -555,7 +562,6 @@ impl<'a> Extra<'a> {
         Extra {
             mode,
             data: None,
-            field_name: None,
             strict,
             ultra_strict: false,
             from_attributes,
@@ -574,14 +580,13 @@ impl<'a> Extra<'a> {
             ultra_strict,
             from_attributes: self.from_attributes,
             context: self.context,
-            field_name: self.field_name,
             self_instance: self.self_instance,
         }
     }
 }
 
 #[derive(Debug, Clone)]
-#[enum_dispatch]
+#[enum_dispatch(PyGcTraverse)]
 pub enum CombinedValidator {
     // typed dict e.g. heterogeneous dicts or simply a model
     TypedDict(typed_dict::TypedDictValidator),
@@ -607,6 +612,8 @@ pub enum CombinedValidator {
     // floats
     Float(float::FloatValidator),
     ConstrainedFloat(float::ConstrainedFloatValidator),
+    // decimals
+    Decimal(decimal::DecimalValidator),
     // lists
     List(list::ListValidator),
     // sets - unique lists
@@ -663,6 +670,8 @@ pub enum CombinedValidator {
     // url types
     Url(url::UrlValidator),
     MultiHostUrl(url::MultiHostUrlValidator),
+    // uuid types
+    Uuid(uuid::UuidValidator),
     // reference to definition, useful for recursive (self-referencing) models
     DefinitionRef(definitions::DefinitionRefValidator),
     // input dependent
@@ -673,44 +682,33 @@ pub enum CombinedValidator {
 /// validators defined in `build_validator` also need `EXPECTED_TYPE` as a const, but that can't be part of the trait
 #[enum_dispatch(CombinedValidator)]
 pub trait Validator: Send + Sync + Clone + Debug {
-    fn py_gc_traverse(&self, _visit: &PyVisit<'_>) -> Result<(), PyTraverseError> {
-        Ok(())
-    }
-    fn py_gc_clear(&mut self) {}
-
     /// Do the actual validation for this schema/type
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        definitions: &'data Definitions<CombinedValidator>,
-        recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject>;
 
     /// Get a default value, currently only used by `WithDefaultValidator`
-    fn default_value<'s, 'data>(
-        &'s self,
+    fn default_value<'data>(
+        &self,
         _py: Python<'data>,
         _outer_loc: Option<impl Into<LocItem>>,
-        _extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        _state: &mut ValidationState,
     ) -> ValResult<'data, Option<PyObject>> {
         Ok(None)
     }
 
     /// Validate assignment to a field of a model
     #[allow(clippy::too_many_arguments)]
-    fn validate_assignment<'s, 'data: 's>(
-        &'s self,
+    fn validate_assignment<'data>(
+        &self,
         _py: Python<'data>,
         _obj: &'data PyAny,
         _field_name: &'data str,
         _field_value: &'data PyAny,
-        _extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        _state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         let py_err = PyTypeError::new_err(format!("validate_assignment is not supported for {}", self.get_name()));
         Err(py_err.into())
