@@ -7,16 +7,18 @@ use ahash::AHashSet;
 
 use crate::build_tools::py_schema_err;
 use crate::build_tools::{is_strict, schema_or_config_same, ExtraBehavior};
-use crate::errors::{py_err_string, ErrorType, ValError, ValLineError, ValResult};
+use crate::errors::{py_err_string, ErrorType, ErrorTypeDefaults, ValError, ValLineError, ValResult};
 use crate::input::{
     AttributesGenericIterator, DictGenericIterator, GenericMapping, Input, JsonObjectGenericIterator,
     MappingGenericIterator,
 };
 use crate::lookup_key::LookupKey;
-use crate::recursion_guard::RecursionGuard;
 use crate::tools::SchemaDict;
 
-use super::{build_validator, BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+use super::ValidationState;
+use super::{build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, Extra, Validator};
+
+use std::ops::ControlFlow;
 
 #[derive(Debug, Clone)]
 struct Field {
@@ -27,6 +29,8 @@ struct Field {
     frozen: bool,
 }
 
+impl_py_gc_traverse!(Field { validator });
+
 #[derive(Debug, Clone)]
 pub struct ModelFieldsValidator {
     fields: Vec<Field>,
@@ -66,7 +70,7 @@ impl BuildValidator for ModelFieldsValidator {
         let fields_dict: &PyDict = schema.get_as_req(intern!(py, "fields"))?;
         let mut fields: Vec<Field> = Vec::with_capacity(fields_dict.len());
 
-        for (key, value) in fields_dict.iter() {
+        for (key, value) in fields_dict {
             let field_info: &PyDict = value.downcast()?;
             let field_name: &str = key.extract()?;
 
@@ -107,17 +111,20 @@ impl BuildValidator for ModelFieldsValidator {
     }
 }
 
+impl_py_gc_traverse!(ModelFieldsValidator {
+    fields,
+    extra_validator
+});
+
 impl Validator for ModelFieldsValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        definitions: &'data Definitions<CombinedValidator>,
-        recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let strict = extra.strict.unwrap_or(self.strict);
-        let from_attributes = extra.from_attributes.unwrap_or(self.from_attributes);
+        let strict = state.strict_or(self.strict);
+        let from_attributes = state.extra().from_attributes.unwrap_or(self.from_attributes);
 
         // we convert the DictType error to a ModelType error
         let dict = match input.validate_model_fields(strict, from_attributes) {
@@ -126,10 +133,11 @@ impl Validator for ModelFieldsValidator {
                 let errors: Vec<ValLineError> = errors
                     .into_iter()
                     .map(|e| match e.error_type {
-                        ErrorType::DictType => {
+                        ErrorType::DictType { .. } => {
                             let mut e = e;
                             e.error_type = ErrorType::ModelType {
                                 class_name: self.model_name.clone(),
+                                context: None,
                             };
                             e
                         }
@@ -154,60 +162,74 @@ impl Validator for ModelFieldsValidator {
             _ => None,
         };
 
+        macro_rules! control_flow {
+            ($e: expr) => {
+                match $e {
+                    Ok(v) => ControlFlow::Continue(v),
+                    Err(err) => ControlFlow::Break(ValError::from(err)),
+                }
+            };
+        }
+
         macro_rules! process {
             ($dict:ident, $get_method:ident, $iter:ty $(,$kwargs:ident)?) => {{
-                for field in &self.fields {
-                    let extra = Extra {
-                        data: Some(model_dict),
-                        field_name: Some(&field.name),
-                        ..*extra
-                    };
-                    let op_key_value = match field.lookup_key.$get_method($dict $(, $kwargs )? ) {
-                        Ok(v) => v,
-                        Err(err) => {
-                            errors.push(ValLineError::new_with_loc(
-                                ErrorType::GetAttributeError {
-                                    error: py_err_string(py, err),
-                                },
-                                input,
-                                field.name.clone(),
-                            ));
-                            continue;
-                        }
-                    };
-                    if let Some((lookup_path, value)) = op_key_value {
-                        if let Some(ref mut used_keys) = used_keys {
-                            // key is "used" whether or not validation passes, since we want to skip this key in
-                            // extra logic either way
-                            used_keys.insert(lookup_path.first_key());
-                        }
-                        match field
-                            .validator
-                            .validate(py, value, &extra, definitions, recursion_guard)
-                        {
-                            Ok(value) => {
-                                model_dict.set_item(&field.name_py, value)?;
-                                fields_set_vec.push(field.name_py.clone_ref(py));
+                match state.with_new_extra(Extra {
+                    data: Some(model_dict),
+                    ..*state.extra()
+                }, |state| {
+                    for field in &self.fields {
+                        let op_key_value = match field.lookup_key.$get_method($dict $(, $kwargs )? ) {
+                            Ok(v) => v,
+                            Err(err) => {
+                                errors.push(ValLineError::new_with_loc(
+                                    ErrorType::GetAttributeError {
+                                        error: py_err_string(py, err),
+                                        context: None,
+                                    },
+                                    input,
+                                    field.name.clone(),
+                                ));
+                                continue;
                             }
-                            Err(ValError::Omit) => continue,
-                            Err(ValError::LineErrors(line_errors)) => {
-                                for err in line_errors {
-                                    errors.push(lookup_path.apply_error_loc(err, self.loc_by_alias, &field.name));
+                        };
+                        if let Some((lookup_path, value)) = op_key_value {
+                            if let Some(ref mut used_keys) = used_keys {
+                                // key is "used" whether or not validation passes, since we want to skip this key in
+                                // extra logic either way
+                                used_keys.insert(lookup_path.first_key());
+                            }
+                            match field
+                                .validator
+                                .validate(py, value, state)
+                            {
+                                Ok(value) => {
+                                    control_flow!(model_dict.set_item(&field.name_py, value))?;
+                                    fields_set_vec.push(field.name_py.clone_ref(py));
                                 }
+                                Err(ValError::Omit) => continue,
+                                Err(ValError::LineErrors(line_errors)) => {
+                                    for err in line_errors {
+                                        errors.push(lookup_path.apply_error_loc(err, self.loc_by_alias, &field.name));
+                                    }
+                                }
+                                Err(err) => return ControlFlow::Break(err),
                             }
-                            Err(err) => return Err(err),
+                            continue;
+                        } else if let Some(value) = control_flow!(field.validator.default_value(py, Some(field.name.as_str()), state))? {
+                            control_flow!(model_dict.set_item(&field.name_py, value))?;
+                        } else {
+                            errors.push(field.lookup_key.error(
+                                ErrorTypeDefaults::Missing,
+                                input,
+                                self.loc_by_alias,
+                                &field.name
+                            ));
                         }
-                        continue;
-                    } else if let Some(value) = field.validator.default_value(py, Some(field.name.as_str()), &extra, definitions, recursion_guard)? {
-                        model_dict.set_item(&field.name_py, value)?;
-                    } else {
-                        errors.push(field.lookup_key.error(
-                            ErrorType::Missing,
-                            input,
-                            self.loc_by_alias,
-                            &field.name
-                        ));
                     }
+                    ControlFlow::Continue(())
+                }) {
+                    ControlFlow::Continue(()) => {}
+                    ControlFlow::Break(err) => return Err(err),
                 }
 
                 if let Some(ref mut used_keys) = used_keys {
@@ -220,7 +242,7 @@ impl Validator for ModelFieldsValidator {
                                 for err in line_errors {
                                     errors.push(
                                         err.with_outer_location(raw_key.as_loc_item())
-                                            .with_type(ErrorType::InvalidKey),
+                                            .with_type(ErrorTypeDefaults::InvalidKey),
                                     );
                                 }
                                 continue;
@@ -235,7 +257,7 @@ impl Validator for ModelFieldsValidator {
                         match self.extra_behavior {
                             ExtraBehavior::Forbid => {
                                 errors.push(ValLineError::new_with_loc(
-                                    ErrorType::ExtraForbidden,
+                                    ErrorTypeDefaults::ExtraForbidden,
                                     value,
                                     raw_key.as_loc_item(),
                                 ));
@@ -244,7 +266,7 @@ impl Validator for ModelFieldsValidator {
                             ExtraBehavior::Allow => {
                             let py_key = either_str.as_py_string(py);
                                 if let Some(ref validator) = self.extra_validator {
-                                    match validator.validate(py, value, &extra, definitions, recursion_guard) {
+                                    match validator.validate(py, value, state) {
                                         Ok(value) => {
                                             model_extra_dict.set_item(py_key, value)?;
                                             fields_set_vec.push(py_key.into_py(py));
@@ -263,7 +285,9 @@ impl Validator for ModelFieldsValidator {
                             }
                         }
                     }
-                    model_extra_dict_op = Some(model_extra_dict);
+                    if matches!(self.extra_behavior, ExtraBehavior::Allow) {
+                        model_extra_dict_op = Some(model_extra_dict);
+                    }
                 }
             }};
         }
@@ -279,9 +303,9 @@ impl Validator for ModelFieldsValidator {
         } else {
             let fields_set = PySet::new(py, &fields_set_vec)?;
 
-            // if we have extra=allow, but we didn't create a dict because we were validate attributes, set it now
-            // so __pydantic_extra__ is always a dict if extra=allow
-            if model_extra_dict_op.is_none() && matches!(self.extra_behavior, ExtraBehavior::Allow) {
+            // if we have extra=allow, but we didn't create a dict because we were validating
+            // from attributes, set it now so __pydantic_extra__ is always a dict if extra=allow
+            if matches!(self.extra_behavior, ExtraBehavior::Allow) && model_extra_dict_op.is_none() {
                 model_extra_dict_op = Some(PyDict::new(py));
             };
 
@@ -289,25 +313,23 @@ impl Validator for ModelFieldsValidator {
         }
     }
 
-    fn validate_assignment<'s, 'data: 's>(
-        &'s self,
+    fn validate_assignment<'data>(
+        &self,
         py: Python<'data>,
         obj: &'data PyAny,
         field_name: &'data str,
         field_value: &'data PyAny,
-        extra: &Extra,
-        definitions: &'data Definitions<CombinedValidator>,
-        recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         let dict: &PyDict = obj.downcast()?;
 
-        let ok = |output: PyObject| {
+        let get_updated_dict = |output: PyObject| {
             dict.set_item(field_name, output)?;
-            Ok(dict.to_object(py))
+            Ok(dict)
         };
 
         let prepare_result = |result: ValResult<'data, PyObject>| match result {
-            Ok(output) => ok(output),
+            Ok(output) => get_updated_dict(output),
             Err(ValError::LineErrors(line_errors)) => {
                 let errors = line_errors
                     .into_iter()
@@ -327,24 +349,21 @@ impl Validator for ModelFieldsValidator {
             }
         }
 
-        let extra = Extra {
+        let new_extra = Extra {
             data: Some(data_dict),
-            field_name: Some(field_name),
-            ..*extra
+            ..*state.extra()
         };
 
         let new_data = if let Some(field) = self.fields.iter().find(|f| f.name == field_name) {
             if field.frozen {
                 Err(ValError::new_with_loc(
-                    ErrorType::FrozenField,
+                    ErrorTypeDefaults::FrozenField,
                     field_value,
                     field.name.to_string(),
                 ))
             } else {
                 prepare_result(
-                    field
-                        .validator
-                        .validate(py, field_value, &extra, definitions, recursion_guard),
+                    state.with_new_extra(new_extra, |state| field.validator.validate(py, field_value, state)),
                 )
             }
         } else {
@@ -355,15 +374,16 @@ impl Validator for ModelFieldsValidator {
             // unless the user explicitly set extra_behavior to 'allow'
             match self.extra_behavior {
                 ExtraBehavior::Allow => match self.extra_validator {
-                    Some(ref validator) => {
-                        prepare_result(validator.validate(py, field_value, &extra, definitions, recursion_guard))
-                    }
-                    None => ok(field_value.to_object(py)),
+                    Some(ref validator) => prepare_result(
+                        state.with_new_extra(new_extra, |state| validator.validate(py, field_value, state)),
+                    ),
+                    None => get_updated_dict(field_value.to_object(py)),
                 },
                 ExtraBehavior::Forbid | ExtraBehavior::Ignore => {
                     return Err(ValError::new_with_loc(
                         ErrorType::NoSuchAttribute {
                             attribute: field_name.to_string(),
+                            context: None,
                         },
                         field_value,
                         field_name.to_string(),
@@ -372,8 +392,24 @@ impl Validator for ModelFieldsValidator {
             }
         }?;
 
+        let new_extra = match &self.extra_behavior {
+            ExtraBehavior::Allow => {
+                let non_extra_data = PyDict::new(py);
+                self.fields.iter().for_each(|f| {
+                    let popped_value = PyAny::get_item(new_data, &f.name).unwrap();
+                    new_data.del_item(&f.name).unwrap();
+                    non_extra_data.set_item(&f.name, popped_value).unwrap();
+                });
+                let new_extra = new_data.copy()?;
+                new_data.clear();
+                new_data.update(non_extra_data.as_mapping())?;
+                new_extra.to_object(py)
+            }
+            _ => py.None(),
+        };
+
         let fields_set: &PySet = PySet::new(py, &[field_name.to_string()])?;
-        Ok((new_data, py.None(), fields_set.to_object(py)).to_object(py))
+        Ok((new_data.to_object(py), new_extra, fields_set.to_object(py)).to_object(py))
     }
 
     fn different_strict_behavior(
