@@ -6,10 +6,9 @@ use regex::Regex;
 use crate::build_tools::{is_strict, py_schema_error_type, schema_or_config};
 use crate::errors::{ErrorType, ValError, ValResult};
 use crate::input::Input;
-use crate::recursion_guard::RecursionGuard;
 use crate::tools::SchemaDict;
 
-use super::{BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
 #[derive(Debug, Clone)]
 pub struct StrValidator {
@@ -37,16 +36,17 @@ impl BuildValidator for StrValidator {
     }
 }
 
+impl_py_gc_traverse!(StrValidator {});
+
 impl Validator for StrValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        Ok(input.validate_str(extra.strict.unwrap_or(self.strict))?.into_py(py))
+        let either_str = input.validate_str(state.strict_or(self.strict))?;
+        Ok(either_str.into_py(py))
     }
 
     fn different_strict_behavior(
@@ -78,37 +78,56 @@ pub struct StrConstrainedValidator {
     to_upper: bool,
 }
 
+impl_py_gc_traverse!(StrConstrainedValidator {});
+
 impl Validator for StrConstrainedValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let either_str = input.validate_str(extra.strict.unwrap_or(self.strict))?;
+        let either_str = input.validate_str(state.strict_or(self.strict))?;
         let cow = either_str.as_cow()?;
         let mut str = cow.as_ref();
         if self.strip_whitespace {
             str = str.trim();
         }
+
+        let str_len: Option<usize> = if self.min_length.is_some() | self.max_length.is_some() {
+            Some(str.chars().count())
+        } else {
+            None
+        };
         if let Some(min_length) = self.min_length {
-            if str.len() < min_length {
-                // return py_schema_err!("{} is shorter than {}", str, min_length);
-                return Err(ValError::new(ErrorType::StringTooShort { min_length }, input));
+            if str_len.unwrap() < min_length {
+                return Err(ValError::new(
+                    ErrorType::StringTooShort {
+                        min_length,
+                        context: None,
+                    },
+                    input,
+                ));
             }
         }
         if let Some(max_length) = self.max_length {
-            if str.len() > max_length {
-                return Err(ValError::new(ErrorType::StringTooLong { max_length }, input));
+            if str_len.unwrap() > max_length {
+                return Err(ValError::new(
+                    ErrorType::StringTooLong {
+                        max_length,
+                        context: None,
+                    },
+                    input,
+                ));
             }
         }
+
         if let Some(pattern) = &self.pattern {
             if !pattern.is_match(str) {
                 return Err(ValError::new(
                     ErrorType::StringPatternMismatch {
                         pattern: pattern.to_string(),
+                        context: None,
                     },
                     input,
                 ));
