@@ -1,20 +1,19 @@
-use pyo3::intern;
 use pyo3::prelude::*;
-use pyo3::types::{PyDelta, PyDict, PyString};
+use pyo3::types::PyDict;
 use speedate::Duration;
 
 use crate::build_tools::is_strict;
 use crate::errors::{ErrorType, ValError, ValResult};
-use crate::input::{EitherTimedelta, Input};
-use crate::recursion_guard::RecursionGuard;
-use crate::tools::SchemaDict;
+use crate::input::{duration_as_pytimedelta, EitherTimedelta, Input};
 
-use super::{BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+use super::datetime::extract_microseconds_precision;
+use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
 #[derive(Debug, Clone)]
 pub struct TimeDeltaValidator {
     strict: bool,
     constraints: Option<TimedeltaConstraints>,
+    microseconds_precision: speedate::MicrosecondsPrecisionOverflowBehavior,
 }
 
 #[derive(Debug, Clone)]
@@ -24,6 +23,17 @@ struct TimedeltaConstraints {
     ge: Option<Duration>,
     gt: Option<Duration>,
 }
+
+fn get_constraint(schema: &PyDict, key: &str) -> PyResult<Option<Duration>> {
+    match schema.get_item(key) {
+        Some(value) => {
+            let either_timedelta = EitherTimedelta::try_from(value)?;
+            Ok(Some(either_timedelta.to_duration()?))
+        }
+        None => Ok(None),
+    }
+}
+
 impl BuildValidator for TimeDeltaValidator {
     const EXPECTED_TYPE: &'static str = "timedelta";
 
@@ -32,40 +42,39 @@ impl BuildValidator for TimeDeltaValidator {
         config: Option<&PyDict>,
         _definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
-        let py = schema.py();
-        let has_constraints = schema.get_item(intern!(py, "le")).is_some()
-            || schema.get_item(intern!(py, "lt")).is_some()
-            || schema.get_item(intern!(py, "ge")).is_some()
-            || schema.get_item(intern!(py, "gt")).is_some();
+        let constraints = TimedeltaConstraints {
+            le: get_constraint(schema, "le")?,
+            lt: get_constraint(schema, "lt")?,
+            ge: get_constraint(schema, "ge")?,
+            gt: get_constraint(schema, "gt")?,
+        };
 
         Ok(Self {
             strict: is_strict(schema, config)?,
-            constraints: match has_constraints {
-                true => Some(TimedeltaConstraints {
-                    le: py_timedelta_as_timedelta(schema, intern!(py, "le"))?,
-                    lt: py_timedelta_as_timedelta(schema, intern!(py, "lt"))?,
-                    ge: py_timedelta_as_timedelta(schema, intern!(py, "ge"))?,
-                    gt: py_timedelta_as_timedelta(schema, intern!(py, "gt"))?,
-                }),
-                false => None,
-            },
+            constraints: (constraints.le.is_some()
+                || constraints.lt.is_some()
+                || constraints.ge.is_some()
+                || constraints.gt.is_some())
+            .then_some(constraints),
+            microseconds_precision: extract_microseconds_precision(schema, config)?,
         }
         .into())
     }
 }
 
+impl_py_gc_traverse!(TimeDeltaValidator {});
+
 impl Validator for TimeDeltaValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let timedelta = input.validate_timedelta(extra.strict.unwrap_or(self.strict))?;
+        let timedelta = input.validate_timedelta(state.strict_or(self.strict), self.microseconds_precision)?;
+        let py_timedelta = timedelta.try_into_py(py)?;
         if let Some(constraints) = &self.constraints {
-            let raw_timedelta = timedelta.as_raw();
+            let raw_timedelta = timedelta.to_duration()?;
 
             macro_rules! check_constraint {
                 ($constraint:ident, $error:ident) => {
@@ -73,9 +82,13 @@ impl Validator for TimeDeltaValidator {
                         if !raw_timedelta.$constraint(constraint) {
                             return Err(ValError::new(
                                 ErrorType::$error {
-                                    $constraint: constraint.to_string().into(),
+                                    context: None,
+                                    $constraint: duration_as_pytimedelta(py, constraint)?
+                                        .repr()?
+                                        .to_string()
+                                        .into(),
                                 },
-                                input,
+                                py_timedelta.as_ref(),
                             ));
                         }
                     }
@@ -87,7 +100,7 @@ impl Validator for TimeDeltaValidator {
             check_constraint!(ge, GreaterThanEqual);
             check_constraint!(gt, GreaterThan);
         }
-        Ok(timedelta.try_into_py(py)?)
+        Ok(py_timedelta.into())
     }
 
     fn different_strict_behavior(
@@ -106,10 +119,3 @@ impl Validator for TimeDeltaValidator {
         Ok(())
     }
 }
-
-fn py_timedelta_as_timedelta(schema: &PyDict, field: &PyString) -> PyResult<Option<Duration>> {
-    match schema.get_as::<&PyDelta>(field)? {
-        Some(timedelta) => Ok(Some(EitherTimedelta::Py(timedelta).as_raw())),
-        None => Ok(None),
-    }
-}
