@@ -10,14 +10,13 @@ use ahash::AHashSet;
 use url::{ParseError, SyntaxViolation, Url};
 
 use crate::build_tools::{is_strict, py_schema_err};
-use crate::errors::{ErrorType, ValError, ValResult};
+use crate::errors::{ErrorType, ErrorTypeDefaults, ValError, ValResult};
 use crate::input::Input;
-use crate::recursion_guard::RecursionGuard;
 use crate::tools::SchemaDict;
 use crate::url::{schema_is_special, PyMultiHostUrl, PyUrl};
 
 use super::literal::expected_repr_name;
-use super::{BuildValidator, CombinedValidator, Definitions, DefinitionsBuilder, Extra, Validator};
+use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
 type AllowedSchemas = Option<(AHashSet<String>, String)>;
 
@@ -57,21 +56,27 @@ impl BuildValidator for UrlValidator {
     }
 }
 
+impl_py_gc_traverse!(UrlValidator {});
+
 impl Validator for UrlValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let mut lib_url = self.get_url(input, extra.strict.unwrap_or(self.strict))?;
+        let mut lib_url = self.get_url(input, state.strict_or(self.strict))?;
 
         if let Some((ref allowed_schemes, ref expected_schemes_repr)) = self.allowed_schemes {
             if !allowed_schemes.contains(lib_url.scheme()) {
                 let expected_schemes = expected_schemes_repr.clone();
-                return Err(ValError::new(ErrorType::UrlScheme { expected_schemes }, input));
+                return Err(ValError::new(
+                    ErrorType::UrlScheme {
+                        expected_schemes,
+                        context: None,
+                    },
+                    input,
+                ));
             }
         }
 
@@ -128,7 +133,7 @@ impl UrlValidator {
 
                     parse_url(&url_str, input, strict)
                 } else {
-                    Err(ValError::new(ErrorType::UrlType, input))
+                    Err(ValError::new(ErrorTypeDefaults::UrlType, input))
                 }
             }
         }
@@ -137,7 +142,13 @@ impl UrlValidator {
     fn check_length<'s, 'data>(&self, input: &'data impl Input<'data>, url_str: &str) -> ValResult<'data, ()> {
         if let Some(max_length) = self.max_length {
             if url_str.len() > max_length {
-                return Err(ValError::new(ErrorType::UrlTooLong { max_length }, input));
+                return Err(ValError::new(
+                    ErrorType::UrlTooLong {
+                        max_length,
+                        context: None,
+                    },
+                    input,
+                ));
             }
         }
         Ok(())
@@ -186,21 +197,27 @@ impl BuildValidator for MultiHostUrlValidator {
     }
 }
 
+impl_py_gc_traverse!(MultiHostUrlValidator {});
+
 impl Validator for MultiHostUrlValidator {
-    fn validate<'s, 'data>(
-        &'s self,
+    fn validate<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        extra: &Extra,
-        _definitions: &'data Definitions<CombinedValidator>,
-        _recursion_guard: &'s mut RecursionGuard,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let mut multi_url = self.get_url(input, extra.strict.unwrap_or(self.strict))?;
+        let mut multi_url = self.get_url(input, state.strict_or(self.strict))?;
 
         if let Some((ref allowed_schemes, ref expected_schemes_repr)) = self.allowed_schemes {
             if !allowed_schemes.contains(multi_url.scheme()) {
                 let expected_schemes = expected_schemes_repr.clone();
-                return Err(ValError::new(ErrorType::UrlScheme { expected_schemes }, input));
+                return Err(ValError::new(
+                    ErrorType::UrlScheme {
+                        expected_schemes,
+                        context: None,
+                    },
+                    input,
+                ));
             }
         }
         match check_sub_defaults(
@@ -254,7 +271,7 @@ impl MultiHostUrlValidator {
                     self.check_length(input, || lib_url.as_str().len())?;
                     Ok(PyMultiHostUrl::new(lib_url, None))
                 } else {
-                    Err(ValError::new(ErrorType::UrlType, input))
+                    Err(ValError::new(ErrorTypeDefaults::UrlType, input))
                 }
             }
         }
@@ -266,7 +283,13 @@ impl MultiHostUrlValidator {
     {
         if let Some(max_length) = self.max_length {
             if func() > max_length {
-                return Err(ValError::new(ErrorType::UrlTooLong { max_length }, input));
+                return Err(ValError::new(
+                    ErrorType::UrlTooLong {
+                        max_length,
+                        context: None,
+                    },
+                    input,
+                ));
             }
         }
         Ok(())
@@ -283,6 +306,7 @@ fn parse_multihost_url<'url, 'input>(
             Err(ValError::new(
                 ErrorType::UrlParsing {
                     error: $parse_error.to_string(),
+                    context: None,
                 },
                 input,
             ))
@@ -400,6 +424,7 @@ fn parse_url<'url, 'input>(
         return Err(ValError::new(
             ErrorType::UrlParsing {
                 error: EMPTY_INPUT.into(),
+                context: None,
             },
             input,
         ));
@@ -426,6 +451,7 @@ fn parse_url<'url, 'input>(
                     Err(ValError::new(
                         ErrorType::UrlSyntaxViolation {
                             error: vio.description().into(),
+                            context: None,
                         },
                         input,
                     ))
@@ -433,10 +459,24 @@ fn parse_url<'url, 'input>(
                     Ok(url)
                 }
             }
-            Err(e) => Err(ValError::new(ErrorType::UrlParsing { error: e.to_string() }, input)),
+            Err(e) => Err(ValError::new(
+                ErrorType::UrlParsing {
+                    error: e.to_string(),
+                    context: None,
+                },
+                input,
+            )),
         }
     } else {
-        Url::parse(url_str).map_err(move |e| ValError::new(ErrorType::UrlParsing { error: e.to_string() }, input))
+        Url::parse(url_str).map_err(move |e| {
+            ValError::new(
+                ErrorType::UrlParsing {
+                    error: e.to_string(),
+                    context: None,
+                },
+                input,
+            )
+        })
     }
 }
 
@@ -448,13 +488,17 @@ fn check_sub_defaults(
     default_port: Option<u16>,
     default_path: &Option<String>,
 ) -> Result<(), ErrorType> {
-    let map_parse_err = |e: ParseError| ErrorType::UrlParsing { error: e.to_string() };
+    let map_parse_err = |e: ParseError| ErrorType::UrlParsing {
+        error: e.to_string(),
+        context: None,
+    };
     if !lib_url.has_host() {
         if let Some(ref default_host) = default_host {
             lib_url.set_host(Some(default_host)).map_err(map_parse_err)?;
         } else if host_required {
             return Err(ErrorType::UrlParsing {
                 error: ParseError::EmptyHost.to_string(),
+                context: None,
             });
         }
     }
@@ -483,7 +527,7 @@ fn get_allowed_schemas(schema: &PyDict, name: &'static str) -> PyResult<(Allowed
 
             let mut expected: AHashSet<String> = AHashSet::new();
             let mut repr_args = Vec::new();
-            for item in list.iter() {
+            for item in list {
                 let str = item.extract()?;
                 repr_args.push(format!("'{str}'"));
                 expected.insert(str);
