@@ -1,3 +1,6 @@
+from decimal import Decimal
+
+
 def schema(*, strict: bool = False) -> dict:
     class MyModel:
         # __slots__ is not required, but it avoids __pydantic_fields_set__ falling into __dict__
@@ -31,6 +34,7 @@ def schema(*, strict: bool = False) -> dict:
                     'type': 'model-field',
                     'schema': {'type': 'float', 'ge': 1.0, 'le': 10.0, 'multiple_of': 0.5},
                 },
+                'field_decimal': {'type': 'model-field', 'schema': {'type': 'decimal'}},
                 'field_bool': {'type': 'model-field', 'schema': {'type': 'bool'}},
                 'field_bytes': {'type': 'model-field', 'schema': {'type': 'bytes'}},
                 'field_bytes_con': {
@@ -52,6 +56,7 @@ def schema(*, strict: bool = False) -> dict:
                     'type': 'model-field',
                     'schema': {'type': 'datetime', 'ge': '2000-01-01T06:00:00', 'lt': '2020-01-02T12:13:14'},
                 },
+                'field_uuid': {'type': 'model-field', 'schema': {'type': 'uuid'}},
                 'field_list_any': {'type': 'model-field', 'schema': {'type': 'list'}},
                 'field_list_str': {'type': 'model-field', 'schema': {'type': 'list', 'items_schema': {'type': 'str'}}},
                 'field_list_str_con': {
@@ -209,91 +214,6 @@ def schema(*, strict: bool = False) -> dict:
     }
 
 
-def pydantic_model():
-    from datetime import date, datetime, time
-    from typing import Any, Literal, Union
-
-    try:
-        from pydantic import BaseModel, conbytes, confloat, confrozenset, conint, conlist, conset, constr, validator
-    except ImportError:
-        return None
-
-    class UnionModel1(BaseModel):
-        field_str: str
-        field_int: int
-        field_float: float
-
-    class UnionModel2(BaseModel):
-        field_float: float
-        field_bytes: bytes
-        field_date: date
-
-    class FunctionModel(BaseModel):
-        field_before: str
-        field_after: str
-        field_wrap: str
-        field_plain: Any
-
-        @validator('field_before', pre=True, allow_reuse=True)
-        def append_before(cls, v):
-            return f'{v} Changed'
-
-        @validator('field_after', 'field_wrap', 'field_plain', allow_reuse=True)  # best attempts at wrap and plain
-        def append_after(cls, v):
-            return f'{v} Changed'
-
-        @validator('field_wrap', pre=True, allow_reuse=True)  # other part of wrap
-        def wrap_before(cls, v):
-            return f'Input {v}'
-
-    class BranchModel(BaseModel):
-        name: str
-        sub_branch: 'BranchModel' = None
-
-    class Model(BaseModel):
-        field_str: str
-        field_str_con: constr(min_length=3, max_length=5, regex='^[a-z]+$')
-        field_int: int
-        field_int_con: conint(gt=1, lt=10, multiple_of=2)
-        field_float: float
-        field_float_con: confloat(ge=1.0, le=10.0, multiple_of=0.5)
-        field_bool: bool
-        field_bytes: bytes
-        field_bytes_con: conbytes(min_length=6, max_length=1000)
-        field_date: date
-        field_date_con: date  # todo ge='2020-01-01', lt='2020-01-02'
-        field_time: time
-        field_time_con: time  # todo ge='06:00:00', lt='12:13:14'
-        field_datetime: datetime
-        field_datetime_con: datetime  # todo ge='2000-01-01T06:00:00', lt='2020-01-02T12:13:14'
-        field_list_any: list
-        field_list_str: list[str]
-        field_list_str_con: conlist(str, min_items=3, max_items=42)
-        field_set_any: set
-        field_set_int: set[int]
-        field_set_int_con: conset(int, min_items=3, max_items=42)
-        field_frozenset_any: frozenset
-        field_frozenset_bytes: frozenset[bytes]
-        field_frozenset_bytes_con: confrozenset(bytes, min_items=3, max_items=42)
-        field_tuple_var_len_any: tuple[Any, ...]
-        field_tuple_var_len_float: tuple[float, ...]
-        field_tuple_var_len_float_con: tuple[float, ...]  # todo min_items=3, max_items=42
-        field_tuple_fix_len: tuple[str, int, float, bool]
-        field_dict_any: dict
-        field_dict_str_float: dict[str, float]
-        field_literal_1_int: Literal[1]
-        field_literal_1_str: Literal['foobar']
-        field_literal_mult_int: Literal[1, 2, 3]
-        field_literal_mult_str: Literal['foo', 'bar', 'baz']
-        field_literal_assorted: Literal[1, 'foo', True]
-        field_list_nullable_int: list[int | None]
-        field_union: Union[str, UnionModel1, UnionModel2]
-        field_functions_model: FunctionModel
-        field_recursive: BranchModel
-
-    return Model
-
-
 def input_data_lax():
     return {
         'field_str': 'fo',
@@ -302,6 +222,7 @@ def input_data_lax():
         'field_int_con': 8,
         'field_float': 1.0,
         'field_float_con': 10.0,
+        'field_decimal': 42.0,
         'field_bool': True,
         'field_bytes': b'foobar',
         'field_bytes_con': b'foobar',
@@ -311,6 +232,7 @@ def input_data_lax():
         'field_time_con': '12:00:00',
         'field_datetime': '2020-01-01T12:13:14',
         'field_datetime_con': '2020-01-01T00:00:00',
+        'field_uuid': '12345678-1234-5678-1234-567812345678',
         'field_list_any': ['a', b'b', True, 1.0, None] * 10,
         'field_list_str': ['a', 'b', 'c'] * 10,
         'field_list_str_con': ['a', 'b', 'c'] * 10,
@@ -348,6 +270,7 @@ def input_data_lax():
 
 def input_data_strict():
     from datetime import date, datetime, time
+    from uuid import UUID
 
     input_data = input_data_lax()
     input_data.update(
@@ -357,6 +280,8 @@ def input_data_strict():
         field_time_con=time(12, 0, 0),
         field_datetime=datetime(2020, 1, 1, 12, 13, 14),
         field_datetime_con=datetime(2020, 1, 1),
+        field_uuid=UUID('12345678-1234-5678-1234-567812345678'),
+        field_decimal=Decimal('42.0'),
     )
     return input_data
 
@@ -369,6 +294,7 @@ def input_data_wrong():
         'field_int_con': 11,
         'field_float': False,
         'field_float_con': 10.1,
+        'field_decimal': 'wrong',
         'field_bool': 4,
         'field_bytes': 42,
         'field_bytes_con': b'foo',
@@ -378,6 +304,7 @@ def input_data_wrong():
         'field_time_con': '23:00:00',
         'field_datetime': b'smash',
         'field_datetime_con': '1900-01-01T00:00:00',
+        'field_uuid': '12345678-1234-5678-1234-567812345678',
         'field_list_any': {1: 2, 3: 4},
         'field_list_str': [(i,) for i in range(100)],
         'field_list_str_con': ['a', 'b'],
