@@ -2,17 +2,15 @@
 General benchmarks that attempt to cover all field types, through by no means all uses of all field types.
 """
 import json
-import sys
 from datetime import date, datetime, time
+from decimal import Decimal
+from uuid import UUID
 
 import pytest
 
 from pydantic_core import SchemaValidator, ValidationError
 
-from .complete_schema import input_data_lax, input_data_strict, input_data_wrong, pydantic_model, schema
-from .test_micro_benchmarks import skip_pydantic
-
-pytestmark = pytest.mark.skipif(sys.version_info < (3, 10), reason='requires python3.10 or higher')
+from .complete_schema import input_data_lax, input_data_strict, input_data_wrong, schema
 
 
 def test_complete_valid():
@@ -21,7 +19,7 @@ def test_complete_valid():
     lax_validator = SchemaValidator(lax_schema)
     output = lax_validator.validate_python(input_data_lax())
     assert isinstance(output, cls)
-    assert len(output.__pydantic_fields_set__) == 39
+    assert len(output.__pydantic_fields_set__) == 41
     output_dict = output.__dict__
     assert output_dict == {
         'field_str': 'fo',
@@ -30,6 +28,7 @@ def test_complete_valid():
         'field_int_con': 8,
         'field_float': 1.0,
         'field_float_con': 10.0,
+        'field_decimal': Decimal('42.0'),
         'field_bool': True,
         'field_bytes': b'foobar',
         'field_bytes_con': b'foobar',
@@ -39,6 +38,7 @@ def test_complete_valid():
         'field_time_con': time(12, 0),
         'field_datetime': datetime(2020, 1, 1, 12, 13, 14),
         'field_datetime_con': datetime(2020, 1, 1),
+        'field_uuid': UUID('12345678-1234-5678-1234-567812345678'),
         'field_list_any': ['a', b'b', True, 1.0, None] * 10,
         'field_list_str': ['a', 'b', 'c'] * 10,
         'field_list_str_con': ['a', 'b', 'c'] * 10,
@@ -77,32 +77,13 @@ def test_complete_valid():
     output2 = strict_validator.validate_python(input_data_strict())
     assert output_dict == output2.__dict__
 
-    model = pydantic_model()
-    if model is None:
-        print('pydantic is not installed, skipping pydantic tests')
-        return
-
-    output_pydantic = model.parse_obj(input_data_lax())
-    assert output_pydantic.dict() == output_dict
-
 
 def test_complete_invalid():
     lax_schema = schema()
     lax_validator = SchemaValidator(lax_schema)
     with pytest.raises(ValidationError) as exc_info:
         lax_validator.validate_python(input_data_wrong())
-    assert len(exc_info.value.errors(include_url=False)) == 738
-
-    model = pydantic_model()
-    if model is None:
-        print('pydantic is not installed, skipping pydantic tests')
-        return
-
-    from pydantic import ValidationError as PydanticValidationError
-
-    with pytest.raises(PydanticValidationError) as exc_info:
-        model.parse_obj(input_data_wrong())
-    assert len(exc_info.value.errors()) == 530
+    assert len(exc_info.value.errors(include_url=False)) == 739
 
 
 @pytest.mark.benchmark(group='complete')
@@ -117,14 +98,6 @@ def test_complete_core_strict(benchmark):
     benchmark(v.validate_python, input_data_strict())
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='complete')
-def test_complete_pyd(benchmark):
-    model = pydantic_model()
-    assert model is not None
-    benchmark(model.parse_obj, input_data_lax())
-
-
 @pytest.mark.benchmark(group='complete-wrong')
 def test_complete_core_error(benchmark):
     v = SchemaValidator(schema())
@@ -151,23 +124,6 @@ def test_complete_core_isinstance(benchmark):
         v.isinstance_python(data)
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='complete-wrong')
-def test_complete_pyd_error(benchmark):
-    model = pydantic_model()
-    assert model is not None
-    data = input_data_wrong()
-
-    @benchmark
-    def f():
-        try:
-            model.parse_obj(data)
-        except ValueError:
-            pass
-        else:
-            raise RuntimeError('expected ValueError')
-
-
 def default_json_encoder(obj):
     if isinstance(obj, bytes):
         return obj.decode('utf-8')
@@ -184,18 +140,6 @@ def test_complete_core_json(benchmark):
     benchmark(v.validate_json, json_data)
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='complete-json')
-def test_complete_pyd_json(benchmark):
-    model = pydantic_model()
-    assert model is not None
-    json_data = json.dumps(input_data_lax(), default=default_json_encoder)
-
-    @benchmark
-    def t():
-        model.parse_raw(json_data, content_type='application/json')
-
-
 @pytest.mark.benchmark(group='build')
 def test_build_schema(benchmark):
     lax_schema = schema()
