@@ -1,43 +1,41 @@
 """
 Numerous benchmarks of specific functionality.
 """
+import decimal
 import json
-import os
 import platform
+import sys
 from datetime import date, datetime, timedelta, timezone
 from decimal import Decimal
 from enum import Enum
-from typing import Any, Dict, FrozenSet, List, Optional, Set, Union
+from typing import Any, List
+from uuid import UUID
 
 import pytest
 from dirty_equals import IsStr
 
-from pydantic_core import ArgsKwargs, PydanticCustomError, SchemaValidator, ValidationError, core_schema
+import pydantic_core
+from pydantic_core import (
+    ArgsKwargs,
+    PydanticCustomError,
+    PydanticKnownError,
+    SchemaValidator,
+    ValidationError,
+    core_schema,
+)
 from pydantic_core import ValidationError as CoreValidationError
 
-if os.getenv('BENCHMARK_VS_PYDANTIC'):
-    try:
-        from pydantic import BaseModel, StrictBool, StrictInt, StrictStr
-        from pydantic import ValidationError as PydanticValidationError
-    except ImportError:
-        BaseModel = None
-else:
-    BaseModel = None
+skip_pypy_deep_stack = pytest.mark.skipif(
+    platform.python_implementation() == 'PyPy' and pydantic_core._pydantic_core.build_profile == 'debug',
+    reason='PyPy does not have enough stack space for Rust debug builds to recurse very deep',
+)
 
-skip_pydantic = pytest.mark.skipif(BaseModel is None, reason='skipping benchmarks vs. pydantic')
+skip_wasm_deep_stack = pytest.mark.skipif(
+    sys.platform == 'emscripten', reason='wasm does not have enough stack space to recurse very deep'
+)
 
 
 class TestBenchmarkSimpleModel:
-    @pytest.fixture(scope='class')
-    def pydantic_model(self):
-        class PydanticModel(BaseModel):
-            name: str
-            age: int
-            friends: List[int]
-            settings: Dict[str, float]
-
-        return PydanticModel
-
     @pytest.fixture(scope='class')
     def core_validator_fs(self):
         class CoreModel:
@@ -68,11 +66,6 @@ class TestBenchmarkSimpleModel:
 
     data = {'name': 'John', 'age': 42, 'friends': list(range(200)), 'settings': {f'v_{i}': i / 2.0 for i in range(50)}}
 
-    @skip_pydantic
-    @pytest.mark.benchmark(group='simple model - python')
-    def test_pyd_python(self, pydantic_model, benchmark):
-        benchmark(pydantic_model.parse_obj, self.data)
-
     @pytest.mark.benchmark(group='simple model - python')
     def test_core_python_fs(self, core_validator_fs, benchmark):
         m = core_validator_fs.validate_python(self.data)
@@ -81,16 +74,6 @@ class TestBenchmarkSimpleModel:
         assert m.__pydantic_fields_set__ == {'name', 'age', 'friends', 'settings'}
         benchmark(core_validator_fs.validate_python, self.data)
 
-    @skip_pydantic
-    @pytest.mark.benchmark(group='simple model - JSON')
-    def test_pyd_json(self, pydantic_model, benchmark):
-        json_data = json.dumps(self.data)
-
-        @benchmark
-        def pydantic_json():
-            obj = json.loads(json_data)
-            return pydantic_model.parse_obj(obj)
-
     @pytest.mark.benchmark(group='simple model - JSON')
     def test_core_json_fs(self, core_validator_fs, benchmark):
         json_data = json.dumps(self.data)
@@ -141,18 +124,6 @@ class TestModelLarge:
 bool_cases = [True, False, 0, 1, '0', '1', 'true', 'false', 'True', 'False']
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='bool')
-def test_bool_pyd(benchmark):
-    class PydanticModel(BaseModel):
-        value: bool
-
-    @benchmark
-    def t():
-        for case in bool_cases:
-            PydanticModel(value=case)
-
-
 @pytest.mark.benchmark(group='bool')
 def test_bool_core(benchmark):
     schema_validator = SchemaValidator({'type': 'bool'})
@@ -166,16 +137,6 @@ def test_bool_core(benchmark):
 small_class_data = {'name': 'John', 'age': 42}
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='create small model')
-def test_small_class_pyd(benchmark):
-    class PydanticModel(BaseModel):
-        name: str
-        age: int
-
-    benchmark(PydanticModel.parse_obj, small_class_data)
-
-
 @pytest.mark.benchmark(group='create small model')
 def test_small_class_core_dict(benchmark):
     model_schema = {
@@ -322,24 +283,14 @@ def definition_model_data():
     data = {'width': -1}
 
     _data = data
-    for i in range(100):
+    for i in range(pydantic_core._pydantic_core._recursion_limit - 2):
         _data['branch'] = {'width': i}
         _data = _data['branch']
     return data
 
 
-@pytest.mark.skipif(platform.python_implementation() == 'PyPy', reason='crashes on pypy due to recursion depth')
-@skip_pydantic
-@pytest.mark.benchmark(group='recursive model')
-def test_definition_model_pyd(definition_model_data, benchmark):
-    class PydanticBranch(BaseModel):
-        width: int
-        branch: Optional['PydanticBranch'] = None
-
-    benchmark(PydanticBranch.parse_obj, definition_model_data)
-
-
-@pytest.mark.skipif(platform.python_implementation() == 'PyPy', reason='crashes on pypy due to recursion depth')
+@skip_pypy_deep_stack
+@skip_wasm_deep_stack
 @pytest.mark.benchmark(group='recursive model')
 def test_definition_model_core(definition_model_data, benchmark):
     class CoreBranch:
@@ -373,19 +324,6 @@ def test_definition_model_core(definition_model_data, benchmark):
     benchmark(v.validate_python, definition_model_data)
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='List[TypedDict]')
-def test_list_of_dict_models_pyd(benchmark):
-    class PydanticBranch(BaseModel):
-        width: int
-
-    class PydanticRoot(BaseModel):
-        __root__: List[PydanticBranch]
-
-    data = [{'width': i} for i in range(100)]
-    benchmark(PydanticRoot.parse_obj, data)
-
-
 @pytest.mark.benchmark(group='List[TypedDict]')
 def test_list_of_dict_models_core(benchmark):
     v = SchemaValidator(
@@ -405,18 +343,6 @@ def test_list_of_dict_models_core(benchmark):
 list_of_ints_data = ([i for i in range(1000)], [str(i) for i in range(1000)])
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='List[int]')
-def test_list_of_ints_pyd_py(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: List[int]
-
-    @benchmark
-    def t():
-        PydanticModel.parse_obj(list_of_ints_data[0])
-        PydanticModel.parse_obj(list_of_ints_data[1])
-
-
 @pytest.mark.benchmark(group='List[int]')
 def test_list_of_ints_core_py(benchmark):
     v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}})
@@ -427,20 +353,6 @@ def test_list_of_ints_core_py(benchmark):
         v.validate_python(list_of_ints_data[1])
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='List[int] JSON')
-def test_list_of_ints_pyd_json(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: List[int]
-
-    json_data = [json.dumps(d) for d in list_of_ints_data]
-
-    @benchmark
-    def t():
-        PydanticModel.parse_obj(json.loads(json_data[0]))
-        PydanticModel.parse_obj(json.loads(json_data[1]))
-
-
 @pytest.mark.benchmark(group='List[int] JSON')
 def test_list_of_ints_core_json(benchmark):
     v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}})
@@ -453,18 +365,6 @@ def test_list_of_ints_core_json(benchmark):
         v.validate_json(json_data[1])
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='List[Any]')
-def test_list_of_any_pyd_py(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: list
-
-    @benchmark
-    def t():
-        PydanticModel.parse_obj(list_of_ints_data[0])
-        PydanticModel.parse_obj(list_of_ints_data[1])
-
-
 @pytest.mark.benchmark(group='List[Any]')
 def test_list_of_any_core_py(benchmark):
     v = SchemaValidator({'type': 'list'})
@@ -479,18 +379,6 @@ set_of_ints_data = ({i for i in range(1000)}, {str(i) for i in range(1000)})
 set_of_ints_duplicates = ([i for i in range(100)] * 10, [str(i) for i in range(100)] * 10)
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='Set[int]')
-def test_set_of_ints_pyd(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: Set[int]
-
-    @benchmark
-    def t():
-        PydanticModel.parse_obj(set_of_ints_data[0])
-        PydanticModel.parse_obj(set_of_ints_data[1])
-
-
 @pytest.mark.benchmark(group='Set[int]')
 def test_set_of_ints_core(benchmark):
     v = SchemaValidator({'type': 'set', 'items_schema': {'type': 'int'}})
@@ -521,20 +409,6 @@ def test_set_of_ints_core_length(benchmark):
         v.validate_python(set_of_ints_data[1])
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='Set[int] JSON')
-def test_set_of_ints_pyd_json(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: Set[int]
-
-    json_data = [json.dumps(list(d)) for d in set_of_ints_data]
-
-    @benchmark
-    def t():
-        PydanticModel.parse_obj(json.loads(json_data[0]))
-        PydanticModel.parse_obj(json.loads(json_data[1]))
-
-
 @pytest.mark.benchmark(group='Set[int] JSON')
 def test_set_of_ints_core_json(benchmark):
     v = SchemaValidator({'type': 'set', 'items_schema': {'type': 'int'}})
@@ -563,15 +437,6 @@ frozenset_of_ints = frozenset({i for i in range(1000)})
 frozenset_of_ints_duplicates = [i for i in range(100)] * 10
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='FrozenSet[int]')
-def test_frozenset_of_ints_pyd(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: FrozenSet[int]
-
-    benchmark(PydanticModel.parse_obj, frozenset_of_ints)
-
-
 @pytest.mark.benchmark(group='FrozenSet[int]')
 def test_frozenset_of_ints_core(benchmark):
     v = SchemaValidator({'type': 'frozenset', 'items_schema': {'type': 'int'}})
@@ -589,18 +454,6 @@ def test_frozenset_of_ints_duplicates_core(benchmark):
 dict_of_ints_data = ({str(i): i for i in range(1000)}, {str(i): str(i) for i in range(1000)})
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='Dict[str, int]')
-def test_dict_of_ints_pyd(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: Dict[str, int]
-
-    @benchmark
-    def t():
-        PydanticModel.parse_obj(dict_of_ints_data[0])
-        PydanticModel.parse_obj(dict_of_ints_data[1])
-
-
 @pytest.mark.benchmark(group='Dict[str, int]')
 def test_dict_of_ints_core(benchmark):
     v = SchemaValidator({'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'int'}})
@@ -621,20 +474,6 @@ def test_dict_of_any_core(benchmark):
         v.validate_python(dict_of_ints_data[1])
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='Dict[str, int] JSON')
-def test_dict_of_ints_pyd_json(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: Dict[str, int]
-
-    json_data = [json.dumps(d) for d in dict_of_ints_data]
-
-    @benchmark
-    def t():
-        PydanticModel.parse_obj(json.loads(json_data[0]))
-        PydanticModel.parse_obj(json.loads(json_data[1]))
-
-
 @pytest.mark.benchmark(group='Dict[str, int] JSON')
 def test_dict_of_ints_core_json(benchmark):
     v = SchemaValidator({'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'int'}})
@@ -650,18 +489,6 @@ def test_dict_of_ints_core_json(benchmark):
 many_models_data = [{'age': i} for i in range(1000)]
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='List[SimpleMode]')
-def test_many_models_pyd(benchmark):
-    class SimpleMode(BaseModel):
-        age: int
-
-    class PydanticModel(BaseModel):
-        __root__: List[SimpleMode]
-
-    benchmark(PydanticModel.parse_obj, many_models_data)
-
-
 @pytest.mark.benchmark(group='List[DictSimpleMode]')
 def test_many_models_core_dict(benchmark):
     model_schema = {
@@ -699,15 +526,6 @@ def test_many_models_core_model(benchmark):
 list_of_nullable_data = [None if i % 2 else i for i in range(1000)]
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='List[Nullable[int]]')
-def test_list_of_nullable_pyd(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: List[Optional[int]]
-
-    benchmark(PydanticModel.parse_obj, list_of_nullable_data)
-
-
 @pytest.mark.benchmark(group='List[Nullable[int]]')
 def test_list_of_nullable_core(benchmark):
     v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'nullable', 'schema': {'type': 'int'}}})
@@ -725,23 +543,7 @@ def test_bytes_core(benchmark):
     benchmark(v.validate_python, some_bytes)
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='bytes')
-def test_bytes_pyd(benchmark):
-    class PydanticModel(BaseModel):
-        __root__: bytes
-
-    benchmark(PydanticModel.parse_obj, some_bytes)
-
-
 class TestBenchmarkDateTime:
-    @pytest.fixture(scope='class')
-    def pydantic_model(self):
-        class PydanticModel(BaseModel):
-            dt: datetime
-
-        return PydanticModel
-
     @pytest.fixture(scope='class')
     def core_validator(self):
         class CoreModel:
@@ -760,7 +562,7 @@ class TestBenchmarkDateTime:
 
     @pytest.fixture(scope='class')
     def datetime_raw(self):
-        return datetime.utcnow().replace(tzinfo=timezone.utc) + timedelta(days=1)
+        return datetime.now(timezone.utc) + timedelta(days=1)
 
     @pytest.fixture(scope='class')
     def datetime_str(self, datetime_raw):
@@ -774,23 +576,10 @@ class TestBenchmarkDateTime:
     def json_dict_data(self, datetime_str):
         return json.dumps({'dt': datetime_str})
 
-    @skip_pydantic
-    @pytest.mark.benchmark(group='datetime model - python')
-    def test_pyd_python(self, pydantic_model, benchmark, python_data_dict):
-        benchmark(pydantic_model.parse_obj, python_data_dict)
-
     @pytest.mark.benchmark(group='datetime model - python')
     def test_core_python(self, core_validator, benchmark, python_data_dict):
         benchmark(core_validator.validate_python, python_data_dict)
 
-    @skip_pydantic
-    @pytest.mark.benchmark(group='datetime model - JSON')
-    def test_model_pyd_json(self, pydantic_model, benchmark, json_dict_data):
-        @benchmark
-        def pydantic_json():
-            obj = json.loads(json_dict_data)
-            return pydantic_model.parse_obj(obj)
-
     @pytest.mark.benchmark(group='datetime model - JSON')
     def test_model_core_json(self, core_validator, benchmark, json_dict_data):
         benchmark(core_validator.validate_json, json_dict_data)
@@ -861,29 +650,12 @@ class TestBenchmarkUnion:
 
         benchmark(v.validate_python, 1)
 
-    @skip_pydantic
-    @pytest.mark.benchmark(group='smart-union')
-    def test_smart_union_pyd(self, benchmark):
-        # default pydantic-core behavior matches pydantic one with `Config.smart_union`
-        class PydanticModel(BaseModel, smart_union=True):
-            __root__: Union[bool, int, str]
-
-        benchmark(PydanticModel.parse_obj, 1)
-
     @pytest.mark.benchmark(group='smart-union-coerce')
     def test_smart_union_coerce_core(self, benchmark):
         v = SchemaValidator({'type': 'union', 'choices': [{'type': 'bool'}, {'type': 'str'}]})
 
         benchmark(v.validate_python, 1)  # will be True
 
-    @skip_pydantic
-    @pytest.mark.benchmark(group='smart-union-coerce')
-    def test_smart_union_coerce_pyd(self, benchmark):
-        class PydanticModel(BaseModel, smart_union=True):
-            __root__: Union[bool, str]
-
-        benchmark(PydanticModel.parse_obj, 1)  # will be True
-
     @pytest.mark.benchmark(group='strict-union')
     def test_strict_union_core(self, benchmark):
         v = SchemaValidator(
@@ -892,14 +664,6 @@ class TestBenchmarkUnion:
 
         benchmark(v.validate_python, 1)
 
-    @skip_pydantic
-    @pytest.mark.benchmark(group='strict-union')
-    def test_strict_union_pyd(self, benchmark):
-        class PydanticModel(BaseModel):
-            __root__: Union[StrictBool, StrictInt, StrictStr]
-
-        benchmark(PydanticModel.parse_obj, 1)  # will be True
-
     @pytest.mark.benchmark(group='strict-union-error')
     def test_strict_union_error_core(self, benchmark):
         v = SchemaValidator({'type': 'union', 'strict': True, 'choices': [{'type': 'bool'}, {'type': 'str'}]})
@@ -913,20 +677,113 @@ class TestBenchmarkUnion:
 
         benchmark(validate_with_expected_error)
 
-    @skip_pydantic
-    @pytest.mark.benchmark(group='strict-union-error')
-    def test_strict_union_error_pyd(self, benchmark):
-        class PydanticModel(BaseModel):
-            __root__: Union[StrictBool, StrictStr]
 
-        def validate_with_expected_error():
+class TestBenchmarkUUID:
+    @pytest.fixture(scope='class')
+    def core_validator(self):
+        class CoreModel:
+            __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
+
+        return SchemaValidator(
+            {
+                'type': 'model',
+                'cls': CoreModel,
+                'schema': {
+                    'type': 'model-fields',
+                    'fields': {'u': {'type': 'model-field', 'schema': {'type': 'uuid'}}},
+                },
+            }
+        )
+
+    @pytest.fixture(scope='class')
+    def validator(self):
+        return SchemaValidator({'type': 'uuid'})
+
+    @pytest.fixture(scope='class')
+    def pydantic_validator(self):
+        def to_UUID(v: Any) -> UUID:
+            if isinstance(v, UUID):
+                return v
             try:
-                PydanticModel.parse_obj(2)
-                assert False
-            except PydanticValidationError:
-                assert True
+                if isinstance(v, str):
+                    return UUID(v)
+                else:
+                    try:
+                        return UUID(v.decode())
+                    except ValueError:
+                        # 16 bytes in big-endian order as the bytes argument fail
+                        # the above check
+                        return UUID(bytes=v)
+            except ValueError:
+                raise PydanticCustomError(
+                    'uuid_parsing', 'Input should be a valid UUID, unable to parse string as an UUID'
+                )
 
-        benchmark(validate_with_expected_error)
+        json_schema = core_schema.no_info_after_validator_function(
+            to_UUID, core_schema.str_schema(strict=True, strip_whitespace=True)
+        )
+        schema = core_schema.json_or_python_schema(
+            json_schema=json_schema,
+            python_schema=core_schema.lax_or_strict_schema(
+                lax_schema=core_schema.union_schema([core_schema.is_instance_schema(UUID), json_schema]),
+                strict_schema=core_schema.is_instance_schema(UUID),
+            ),
+            serialization=core_schema.to_string_ser_schema(when_used='json'),
+        )
+
+        return SchemaValidator(schema)
+
+    @pytest.mark.benchmark(group='uuid from str')
+    def test_uuid_from_string_core(self, benchmark, validator):
+        benchmark(validator.validate_python, '12345678-1234-5678-1234-567812345678')
+
+    @pytest.mark.benchmark(group='uuid from str')
+    def test_uuid_from_string_pyd(self, benchmark, pydantic_validator):
+        benchmark(pydantic_validator.validate_python, '12345678-1234-5678-1234-567812345678')
+
+    @pytest.mark.benchmark(group='uuid from UUID')
+    def test_uuid_from_uuid_core(self, benchmark, validator):
+        benchmark(validator.validate_python, UUID('12345678-1234-5678-1234-567812345678'))
+
+    @pytest.mark.benchmark(group='uuid from UUID')
+    def test_uuid_from_uuid_pyd(self, benchmark, pydantic_validator):
+        benchmark(pydantic_validator.validate_python, UUID('12345678-1234-5678-1234-567812345678'))
+
+    @pytest.fixture(scope='class')
+    def uuid_raw(self):
+        return UUID('12345678-1234-5678-1234-567812345678')
+
+    @pytest.fixture(scope='class')
+    def uuid_str(self, uuid_raw):
+        return str(uuid_raw)
+
+    @pytest.fixture(scope='class')
+    def python_data_dict(self, uuid_raw):
+        return {'u': uuid_raw}
+
+    @pytest.fixture(scope='class')
+    def json_dict_data(self, uuid_str):
+        return json.dumps({'u': uuid_str})
+
+    @pytest.mark.benchmark(group='uuid model - python')
+    def test_core_python(self, core_validator, benchmark, python_data_dict):
+        benchmark(core_validator.validate_python, python_data_dict)
+
+    @pytest.mark.benchmark(group='uuid model - JSON')
+    def test_model_core_json(self, core_validator, benchmark, json_dict_data):
+        benchmark(core_validator.validate_json, json_dict_data)
+
+    @pytest.mark.benchmark(group='uuid uuid')
+    def test_core_raw(self, benchmark, uuid_raw):
+        v = SchemaValidator({'type': 'uuid'})
+
+        benchmark(v.validate_python, uuid_raw)
+
+    @pytest.mark.benchmark(group='uuid str')
+    def test_core_str(self, benchmark, uuid_str):
+        v = SchemaValidator({'type': 'uuid'})
+
+        benchmark(v.validate_python, uuid_str)
 
 
 @pytest.mark.benchmark(group='raise-error')
@@ -1371,18 +1228,6 @@ def test_core_root_model(benchmark):
     benchmark(v.validate_python, input_data)
 
 
-@skip_pydantic
-@pytest.mark.benchmark(group='root_model')
-def test_v1_root_model(benchmark):
-    class MyModel(BaseModel):
-        __root__: List[int]
-
-    assert MyModel.parse_obj([1, 2, '3']).__root__ == [1, 2, 3]
-    input_data = list(range(100))
-
-    benchmark(MyModel.parse_obj, input_data)
-
-
 @pytest.mark.benchmark(group='strict_int')
 def test_strict_int(benchmark):
     v = SchemaValidator(core_schema.int_schema(strict=True))
@@ -1450,3 +1295,84 @@ def test_tagged_union_int_keys_json(benchmark):
         v.validate_json('{"x": 1001, "y": "1"}')
 
     benchmark(v.validate_json, payload)
+
+
+@skip_pypy_deep_stack
+@skip_wasm_deep_stack
+@pytest.mark.benchmark(group='field_function_validator')
+def test_field_function_validator(benchmark) -> None:
+    def f(v: int, info: core_schema.FieldValidationInfo) -> int:
+        assert info.field_name == 'x'
+        return v + 1
+
+    schema: core_schema.CoreSchema = core_schema.int_schema()
+    limit = pydantic_core._pydantic_core._recursion_limit - 3
+
+    for _ in range(limit):
+        schema = core_schema.field_after_validator_function(f, 'x', schema)
+
+    schema = core_schema.typed_dict_schema({'x': core_schema.typed_dict_field(schema)})
+
+    v = SchemaValidator(schema)
+    payload = {'x': 0}
+    assert v.validate_python(payload) == {'x': limit}
+
+    benchmark(v.validate_python, payload)
+
+
+class TestBenchmarkDecimal:
+    @pytest.fixture(scope='class')
+    def validator(self):
+        return SchemaValidator({'type': 'decimal'})
+
+    @pytest.fixture(scope='class')
+    def pydantic_validator(self):
+        Decimal = decimal.Decimal
+
+        def to_decimal(v: str) -> decimal.Decimal:
+            try:
+                return Decimal(v)
+            except decimal.DecimalException as e:
+                raise PydanticCustomError('decimal_parsing', 'Input should be a valid decimal') from e
+
+        primitive_schema = core_schema.union_schema(
+            [
+                # if it's an int keep it like that and pass it straight to Decimal
+                # but if it's not make it a string
+                # we don't use JSON -> float because parsing to any float will cause
+                # loss of precision
+                core_schema.int_schema(strict=True),
+                core_schema.str_schema(strict=True, strip_whitespace=True),
+                core_schema.no_info_plain_validator_function(str),
+            ]
+        )
+        json_schema = core_schema.no_info_after_validator_function(to_decimal, primitive_schema)
+        schema = core_schema.json_or_python_schema(
+            json_schema=json_schema,
+            python_schema=core_schema.lax_or_strict_schema(
+                lax_schema=core_schema.union_schema([core_schema.is_instance_schema(decimal.Decimal), json_schema]),
+                strict_schema=core_schema.is_instance_schema(decimal.Decimal),
+            ),
+            serialization=core_schema.to_string_ser_schema(when_used='json'),
+        )
+
+        def check_finite(value: decimal.Decimal) -> decimal.Decimal:
+            if not value.is_finite():
+                raise PydanticKnownError('finite_number')
+            return value
+
+        schema = core_schema.no_info_after_validator_function(check_finite, schema)
+
+        return SchemaValidator(schema)
+
+    @pytest.mark.benchmark(group='decimal from str')
+    def test_decimal_from_string_core(self, benchmark, validator):
+        benchmark(validator.validate_python, '123.456789')
+
+    @pytest.mark.benchmark(group='decimal from str')
+    def test_decimal_from_string_pyd(self, benchmark, pydantic_validator):
+        benchmark(pydantic_validator.validate_python, '123.456789')
+
+    @pytest.mark.benchmark(group='decimal from str')
+    def test_decimal_from_string_limit(self, benchmark):
+        benchmark(decimal.Decimal, '123.456789')
