@@ -1,5 +1,6 @@
 import re
 from decimal import Decimal
+from typing import Any, Optional
 
 import pytest
 from dirty_equals import HasRepr, IsInstance, IsJson, IsStr
@@ -10,7 +11,6 @@ from pydantic_core import (
     PydanticOmit,
     SchemaValidator,
     ValidationError,
-    __version__,
     core_schema,
 )
 from pydantic_core._pydantic_core import list_all_errors
@@ -32,6 +32,20 @@ def test_pydantic_value_error():
     )
 
 
+@pytest.mark.parametrize(
+    'msg,result_msg', [('my custom error', 'my custom error'), ('my custom error {foo}', "my custom error {'bar': []}")]
+)
+def test_pydantic_value_error_nested_ctx(msg: str, result_msg: str):
+    ctx = {'foo': {'bar': []}}
+    e = PydanticCustomError('my_error', msg, ctx)
+    assert e.message() == result_msg
+    assert e.message_template == msg
+    assert e.type == 'my_error'
+    assert e.context == ctx
+    assert str(e) == result_msg
+    assert repr(e) == f'{result_msg} [type=my_error, context={ctx}]'
+
+
 def test_pydantic_value_error_none():
     e = PydanticCustomError('my_error', 'this is a custom error {missed}')
     assert e.message() == 'this is a custom error {missed}'
@@ -143,6 +157,16 @@ def test_pydantic_error_type():
     assert repr(e) == "Invalid JSON: Test [type=json_invalid, context={'error': 'Test'}]"
 
 
+def test_pydantic_error_type_nested_ctx():
+    ctx = {'error': 'Test', 'foo': {'bar': []}}
+    e = PydanticKnownError('json_invalid', ctx)
+    assert e.message() == 'Invalid JSON: Test'
+    assert e.type == 'json_invalid'
+    assert e.context == ctx
+    assert str(e) == 'Invalid JSON: Test'
+    assert repr(e) == f'Invalid JSON: Test [type=json_invalid, context={ctx}]'
+
+
 def test_pydantic_error_type_raise_no_ctx():
     def f(input_value, info):
         raise PydanticKnownError('finite_number')
@@ -159,9 +183,14 @@ def test_pydantic_error_type_raise_no_ctx():
     ]
 
 
-def test_pydantic_error_type_raise_ctx():
+@pytest.mark.parametrize(
+    'extra', [{}, {'foo': 1}, {'foo': {'bar': []}}, {'foo': {'bar': object()}}, {'foo': Decimal('42.1')}]
+)
+def test_pydantic_error_type_raise_ctx(extra: dict):
+    ctx = {'gt': 42, **extra}
+
     def f(input_value, info):
-        raise PydanticKnownError('greater_than', {'gt': 42})
+        raise PydanticKnownError('greater_than', ctx)
 
     v = SchemaValidator(
         {'type': 'function-before', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
@@ -171,7 +200,66 @@ def test_pydantic_error_type_raise_ctx():
         v.validate_python(4)
     # insert_assert(exc_info.value.errors(include_url=False))
     assert exc_info.value.errors(include_url=False) == [
-        {'type': 'greater_than', 'loc': (), 'msg': 'Input should be greater than 42', 'input': 4, 'ctx': {'gt': 42.0}}
+        {'type': 'greater_than', 'loc': (), 'msg': 'Input should be greater than 42', 'input': 4, 'ctx': ctx}
+    ]
+
+
+@pytest.mark.parametrize('ctx', [None, {}])
+def test_pydantic_error_type_raise_custom_no_ctx(ctx: Optional[dict]):
+    def f(input_value, info):
+        raise PydanticKnownError('int_type', ctx)
+
+    v = SchemaValidator(
+        {'type': 'function-before', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
+    )
+
+    expect_ctx = {'ctx': {}} if ctx is not None else {}
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_python(4)
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
+        {'type': 'int_type', 'loc': (), 'msg': 'Input should be a valid integer', 'input': 4, **expect_ctx}
+    ]
+
+
+@pytest.mark.parametrize(
+    'extra', [{}, {'foo': 1}, {'foo': {'bar': []}}, {'foo': {'bar': object()}}, {'foo': Decimal('42.1')}]
+)
+def test_pydantic_custom_error_type_raise_custom_ctx(extra: dict):
+    ctx = {'val': 42, **extra}
+
+    def f(input_value, info):
+        raise PydanticCustomError('my_error', 'my message with {val}', ctx)
+
+    v = SchemaValidator(
+        {'type': 'function-before', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
+    )
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_python(4)
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
+        {'type': 'my_error', 'loc': (), 'msg': 'my message with 42', 'input': 4, 'ctx': ctx}
+    ]
+
+
+@pytest.mark.parametrize('ctx', [None, {}])
+def test_pydantic_custom_error_type_raise_custom_no_ctx(ctx: Optional[dict]):
+    def f(input_value, info):
+        raise PydanticCustomError('my_error', 'my message', ctx)
+
+    v = SchemaValidator(
+        {'type': 'function-before', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
+    )
+
+    expect_ctx = {'ctx': {}} if ctx is not None else {}
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_python(4)
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
+        {'type': 'my_error', 'loc': (), 'msg': 'my message', 'input': 4, **expect_ctx}
     ]
 
 
@@ -191,6 +279,7 @@ all_errors = [
     ('invalid_key', 'Keys should be strings', None),
     ('get_attribute_error', 'Error extracting attribute: foo', {'error': 'foo'}),
     ('none_required', 'Input should be None', None),
+    ('enum', 'Input should be foo', {'expected': 'foo'}),
     ('greater_than', 'Input should be greater than 42.1', {'gt': 42.1}),
     ('greater_than', 'Input should be greater than 42.1', {'gt': '42.1'}),
     ('greater_than', 'Input should be greater than 2020-01-01', {'gt': '2020-01-01'}),
@@ -237,8 +326,8 @@ all_errors = [
     ('bytes_type', 'Input should be a valid bytes', None),
     ('bytes_too_short', 'Data should have at least 42 bytes', {'min_length': 42}),
     ('bytes_too_long', 'Data should have at most 42 bytes', {'max_length': 42}),
-    ('value_error', 'Value error, foobar', {'error': 'foobar'}),
-    ('assertion_error', 'Assertion failed, foobar', {'error': 'foobar'}),
+    ('value_error', 'Value error, foobar', {'error': ValueError('foobar')}),
+    ('assertion_error', 'Assertion failed, foobar', {'error': AssertionError('foobar')}),
     ('literal_error', 'Input should be foo', {'expected': 'foo'}),
     ('literal_error', 'Input should be foo or bar', {'expected': 'foo or bar'}),
     ('date_type', 'Input should be a valid date', None),
@@ -281,6 +370,18 @@ all_errors = [
     ('url_syntax_violation', 'Input violated strict URL syntax rules, Foobar', {'error': 'Foobar'}),
     ('url_too_long', 'URL should have at most 42 characters', {'max_length': 42}),
     ('url_scheme', 'URL scheme should be "foo", "bar" or "spam"', {'expected_schemes': '"foo", "bar" or "spam"'}),
+    ('uuid_type', 'UUID input should be a string, bytes or UUID object', None),
+    ('uuid_parsing', 'Input should be a valid UUID, Foobar', {'error': 'Foobar'}),
+    ('uuid_version', 'UUID version 42 expected', {'expected_version': 42}),
+    ('decimal_type', 'Decimal input should be an integer, float, string or Decimal object', None),
+    ('decimal_parsing', 'Input should be a valid decimal', None),
+    ('decimal_max_digits', 'Decimal input should have no more than 42 digits in total', {'max_digits': 42}),
+    ('decimal_max_places', 'Decimal input should have no more than 42 decimal places', {'decimal_places': 42}),
+    (
+        'decimal_whole_digits',
+        'Decimal input should have no more than 42 digits before the decimal point',
+        {'whole_digits': 42},
+    ),
 ]
 
 
@@ -302,7 +403,7 @@ def test_error_decimal():
     e = PydanticKnownError('greater_than', {'gt': Decimal('42.1')})
     assert e.message() == 'Input should be greater than 42.1'
     assert e.type == 'greater_than'
-    assert e.context == {'gt': 42.1}
+    assert e.context == {'gt': Decimal('42.1')}
 
 
 def test_custom_error_decimal():
@@ -347,14 +448,23 @@ def test_omit_exc_repr():
     assert str(PydanticOmit()) == 'PydanticOmit()'
 
 
-def test_type_error_error():
-    with pytest.raises(TypeError, match="^GreaterThan: 'gt' context value must be a Number$"):
-        PydanticKnownError('greater_than', {'gt': []})
+@pytest.mark.parametrize(
+    'error,ctx,expect',
+    [
+        ('greater_than', {'gt': []}, "GreaterThan: 'gt' context value must be a Number"),
+        ('model_type', {'class_name': []}, "ModelType: 'class_name' context value must be a String"),
+        ('date_parsing', {'error': []}, "DateParsing: 'error' context value must be a String"),
+        ('string_too_short', {'min_length': []}, "StringTooShort: 'min_length' context value must be a usize"),
+    ],
+)
+def test_type_error_error(error: str, ctx: dict, expect: str):
+    with pytest.raises(TypeError, match=f'^{expect}$'):
+        PydanticKnownError(error, ctx)
 
 
-def test_does_not_require_context():
-    with pytest.raises(TypeError, match="^'json_type' errors do not require context$"):
-        PydanticKnownError('json_type', {'gt': 123})
+def test_custom_context_for_simple_error():
+    err = PydanticKnownError('json_type', {'foo': 'bar'})
+    assert err.context == {'foo': 'bar'}
 
 
 def test_all_errors():
@@ -412,7 +522,7 @@ class BadRepr:
         raise RuntimeError('bad repr')
 
 
-def test_error_on_repr():
+def test_error_on_repr(pydantic_version):
     s = SchemaValidator({'type': 'int'})
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python(BadRepr())
@@ -421,7 +531,7 @@ def test_error_on_repr():
         '1 validation error for int\n'
         '  Input should be a valid integer '
         '[type=int_type, input_value=<unprintable BadRepr object>, input_type=BadRepr]\n'
-        f'    For further information visit https://errors.pydantic.dev/{__version__}/v/int_type'
+        f'    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/int_type'
     )
     assert exc_info.value.errors(include_url=False) == [
         {'type': 'int_type', 'loc': (), 'msg': 'Input should be a valid integer', 'input': IsInstance(BadRepr)}
@@ -438,7 +548,7 @@ def test_error_on_repr():
     )
 
 
-def test_error_json():
+def test_error_json(pydantic_version):
     s = SchemaValidator({'type': 'str', 'min_length': 3})
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python('12')
@@ -461,7 +571,7 @@ def test_error_json():
                 'msg': 'String should have at least 3 characters',
                 'input': '12',
                 'ctx': {'min_length': 3},
-                'url': f'https://errors.pydantic.dev/{__version__}/v/string_too_short',
+                'url': f'https://errors.pydantic.dev/{pydantic_version}/v/string_too_short',
             }
         ]
     )
@@ -472,6 +582,50 @@ def test_error_json():
     assert exc_info.value.json(indent=2).startswith('[\n  {\n    "type": "string_too_short",')
 
 
+def test_error_json_python_error(pydantic_version: str):
+    def raise_py_error(v: Any) -> Any:
+        try:
+            assert False
+        except AssertionError as e:
+            raise ValueError('Oh no!') from e
+
+    s = SchemaValidator(core_schema.no_info_plain_validator_function(raise_py_error))
+    with pytest.raises(ValidationError) as exc_info:
+        s.validate_python('anything')
+
+    exc = exc_info.value.errors()[0]['ctx']['error']  # type: ignore
+    assert isinstance(exc, ValueError)
+    assert isinstance(exc.__context__, AssertionError)
+
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
+        {
+            'type': 'value_error',
+            'loc': (),
+            'msg': 'Value error, Oh no!',
+            'input': 'anything',
+            'ctx': {'error': HasRepr(repr(ValueError('Oh no!')))},
+        }
+    ]
+    assert exc_info.value.json() == IsJson(
+        [
+            {
+                'type': 'value_error',
+                'loc': [],
+                'msg': 'Value error, Oh no!',
+                'input': 'anything',
+                'ctx': {'error': 'Oh no!'},
+                'url': f'https://errors.pydantic.dev/{pydantic_version}/v/value_error',
+            }
+        ]
+    )
+    assert exc_info.value.json(include_url=False, include_context=False) == IsJson(
+        [{'type': 'value_error', 'loc': [], 'msg': 'Value error, Oh no!', 'input': 'anything'}]
+    )
+    assert exc_info.value.json().startswith('[{"type":"value_error",')
+    assert exc_info.value.json(indent=2).startswith('[\n  {\n    "type": "value_error",')
+
+
 def test_error_json_cycle():
     s = SchemaValidator({'type': 'str', 'min_length': 3})
     cycle = []
@@ -563,7 +717,7 @@ def test_raise_validation_error():
     assert exc_info.value.errors(include_url=False) == [
         {'type': 'greater_than', 'loc': ('a', 2), 'msg': 'Input should be greater than 5', 'input': 4, 'ctx': {'gt': 5}}
     ]
-    with pytest.raises(TypeError, match='GreaterThan requires context: {gt: Number}'):
+    with pytest.raises(TypeError, match="GreaterThan: 'gt' required in context"):
         raise ValidationError.from_exception_data('Foobar', [{'type': 'greater_than', 'loc': ('a', 2), 'input': 4}])
 
 
@@ -619,7 +773,63 @@ def test_raise_validation_error_custom():
     ]
 
 
-def test_loc_with_dots():
+@pytest.mark.parametrize(
+    'msg,result_msg', [('my custom error', 'my custom error'), ('my custom error {foo}', "my custom error {'bar': []}")]
+)
+def test_raise_validation_error_custom_nested_ctx(msg: str, result_msg: str):
+    ctx = {'foo': {'bar': []}}
+    custom_error = PydanticCustomError('my_error', msg, ctx)
+    with pytest.raises(ValidationError) as exc_info:
+        raise ValidationError.from_exception_data('Foobar', [{'type': custom_error, 'input': 'x'}])
+
+    expected_error_detail = {'type': 'my_error', 'loc': (), 'msg': result_msg, 'input': 'x', 'ctx': ctx}
+
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [expected_error_detail]
+    assert exc_info.value.json(include_url=False) == IsJson([{**expected_error_detail, 'loc': []}])
+
+
+def test_raise_validation_error_known_class_ctx():
+    custom_data = Foobar()
+    ctx = {'gt': 10, 'foo': {'bar': custom_data}}
+
+    with pytest.raises(ValidationError) as exc_info:
+        raise ValidationError.from_exception_data('MyTitle', [{'type': 'greater_than', 'input': 9, 'ctx': ctx}])
+
+    expected_error_detail = {
+        'type': 'greater_than',
+        'loc': (),
+        'msg': 'Input should be greater than 10',
+        'input': 9,
+        'ctx': ctx,
+    }
+
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [expected_error_detail]
+    assert exc_info.value.json(include_url=False) == IsJson(
+        [{**expected_error_detail, 'loc': [], 'ctx': {'gt': 10, 'foo': {'bar': str(custom_data)}}}]
+    )
+
+
+def test_raise_validation_error_custom_class_ctx():
+    custom_data = Foobar()
+    ctx = {'foo': {'bar': custom_data}}
+    custom_error = PydanticCustomError('my_error', 'my message', ctx)
+    assert custom_error.context == ctx
+
+    with pytest.raises(ValidationError) as exc_info:
+        raise ValidationError.from_exception_data('MyTitle', [{'type': custom_error, 'input': 'x'}])
+
+    expected_error_detail = {'type': 'my_error', 'loc': (), 'msg': 'my message', 'input': 'x', 'ctx': ctx}
+
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [expected_error_detail]
+    assert exc_info.value.json(include_url=False) == IsJson(
+        [{**expected_error_detail, 'loc': [], 'ctx': {'foo': {'bar': str(custom_data)}}}]
+    )
+
+
+def test_loc_with_dots(pydantic_version):
     v = SchemaValidator(
         core_schema.typed_dict_schema(
             {
@@ -648,5 +858,5 @@ def test_loc_with_dots():
         "`foo.bar`.0\n"
         "  Input should be a valid integer, unable to parse string as an integer "
         "[type=int_parsing, input_value='x', input_type=str]\n"
-        f'    For further information visit https://errors.pydantic.dev/{__version__}/v/int_parsing'
+        f'    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/int_parsing'
     )
