@@ -36,7 +36,7 @@ def test_datetime_int(datetime_schema, data):
     except OverflowError:
         pytest.skip('OverflowError, see pyodide/pyodide#2841, this can happen on 32-bit systems')
     else:
-        assert datetime_schema.validate_python(data) == expected, data
+        assert datetime_schema.validate_python(data).replace(tzinfo=None) == expected, data
 
 
 @given(strategies.binary())
