@@ -1,10 +1,12 @@
 import json
+import platform
 import re
 from typing import List
 
 import pytest
 from dirty_equals import IsList
 
+import pydantic_core
 from pydantic_core import (
     PydanticSerializationError,
     SchemaSerializer,
@@ -61,6 +63,10 @@ def test_bytes():
         s.validate_json('123')
 
 
+# A number well outside of i64 range
+_BIG_NUMBER_STR = '1' + ('0' * 40)
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
@@ -68,7 +74,9 @@ def test_bytes():
         ('"123"', 123),
         ('123.0', 123),
         ('"123.0"', 123),
+        (_BIG_NUMBER_STR, int(_BIG_NUMBER_STR)),
         ('123.4', Err('Input should be a valid integer, got a number with a fractional part [type=int_from_float,')),
+        ('"123.4"', Err('Input should be a valid integer, unable to parse string as an integer [type=int_parsing,')),
         ('"string"', Err('Input should be a valid integer, unable to parse string as an integer [type=int_parsing,')),
     ],
 )
@@ -285,6 +293,10 @@ def test_cycle_same():
         to_json(f, fallback=fallback_func_passthrough)
 
 
+@pytest.mark.skipif(
+    platform.python_implementation() == 'PyPy' and pydantic_core._pydantic_core.build_profile == 'debug',
+    reason='PyPy does not have enough stack space for Rust debug builds to recurse very deep',
+)
 def test_cycle_change():
     def fallback_func_change_id(obj):
         return Foobar()
@@ -315,7 +327,12 @@ def test_json_key_fallback():
     assert to_json(x, fallback=fallback_func) == b'{"fallback:FoobarHash":1}'
 
 
-class BadRepr:
+class BedReprMeta(type):
+    def __repr__(self):
+        raise ValueError('bad repr')
+
+
+class BadRepr(metaclass=BedReprMeta):
     def __repr__(self):
         raise ValueError('bad repr')
 
@@ -326,7 +343,7 @@ class BadRepr:
 def test_bad_repr():
     b = BadRepr()
 
-    error_msg = '^Unable to serialize unknown type: <unprintable BadRepr object>$'
+    error_msg = '^Unable to serialize unknown type: <unprintable BedReprMeta object>$'
     with pytest.raises(PydanticSerializationError, match=error_msg):
         to_jsonable_python(b)
 
