@@ -56,9 +56,24 @@ all_schema_functions = [
     (core_schema.bytes_schema, args(min_length=5, ref='xx'), {'type': 'bytes', 'min_length': 5, 'ref': 'xx'}),
     (core_schema.date_schema, args(), {'type': 'date'}),
     (core_schema.date_schema, args(gt=date(2020, 1, 1)), {'type': 'date', 'gt': date(2020, 1, 1)}),
-    (core_schema.time_schema, args(), {'type': 'time'}),
-    (core_schema.datetime_schema, args(), {'type': 'datetime'}),
-    (core_schema.timedelta_schema, args(), {'type': 'timedelta'}),
+    (core_schema.time_schema, args(), {'type': 'time', 'microseconds_precision': 'truncate'}),
+    (core_schema.datetime_schema, args(), {'type': 'datetime', 'microseconds_precision': 'truncate'}),
+    (core_schema.timedelta_schema, args(), {'type': 'timedelta', 'microseconds_precision': 'truncate'}),
+    (
+        core_schema.time_schema,
+        args(microseconds_precision='error'),
+        {'type': 'time', 'microseconds_precision': 'error'},
+    ),
+    (
+        core_schema.datetime_schema,
+        args(microseconds_precision='error'),
+        {'type': 'datetime', 'microseconds_precision': 'error'},
+    ),
+    (
+        core_schema.timedelta_schema,
+        args(microseconds_precision='error'),
+        {'type': 'timedelta', 'microseconds_precision': 'error'},
+    ),
     (core_schema.literal_schema, args(['a', 'b']), {'type': 'literal', 'expected': ['a', 'b']}),
     (core_schema.is_instance_schema, args(int), {'type': 'is-instance', 'cls': int}),
     (core_schema.callable_schema, args(), {'type': 'callable'}),
@@ -263,6 +278,9 @@ all_schema_functions = [
         args(MyDataclass, {'type': 'int'}, ['foobar'], slots=True),
         {'type': 'dataclass', 'schema': {'type': 'int'}, 'fields': ['foobar'], 'cls': MyDataclass, 'slots': True},
     ),
+    (core_schema.uuid_schema, args(), {'type': 'uuid'}),
+    (core_schema.decimal_schema, args(), {'type': 'decimal'}),
+    (core_schema.decimal_schema, args(multiple_of=5, gt=1.2), {'type': 'decimal', 'multiple_of': 5, 'gt': 1.2}),
 ]
 
 
