@@ -220,7 +220,9 @@ def test_ser_function_plain():
 
     s = SchemaSerializer(
         core_schema.any_schema(
-            serialization=core_schema.plain_serializer_function_ser_schema(f, info_arg=True, json_return_type='str')
+            serialization=core_schema.plain_serializer_function_ser_schema(
+                f, info_arg=True, return_schema=core_schema.str_schema()
+            )
         )
     )
     assert s.to_python(123) == (
