@@ -6,7 +6,7 @@ from typing import Any, get_type_hints
 
 import pytest
 
-from pydantic_core import ArgsKwargs, SchemaError, SchemaValidator, ValidationError, __version__, core_schema
+from pydantic_core import ArgsKwargs, SchemaError, SchemaValidator, ValidationError, core_schema
 
 from ..conftest import Err, PyAndJson, plain_repr
 
@@ -984,7 +984,7 @@ def test_invalid_schema():
         )
 
 
-def test_error_display():
+def test_error_display(pydantic_version):
     v = SchemaValidator(
         core_schema.arguments_schema(
             [
@@ -1013,7 +1013,7 @@ def test_error_display():
         "b\n"
         "  Missing required argument [type=missing_argument, "
         "input_value=ArgsKwargs((), {'a': 1}), input_type=ArgsKwargs]\n"
-        f"    For further information visit https://errors.pydantic.dev/{__version__}/v/missing_argument"
+        f"    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/missing_argument"
     )
     # insert_assert(exc_info.value.json(include_url=False))
     assert exc_info.value.json(include_url=False) == (
