@@ -1,6 +1,9 @@
 import dataclasses
+import gc
+import platform
 import re
 import sys
+import weakref
 from typing import Any, ClassVar, Dict, List, Optional, Union
 
 import pytest
@@ -504,7 +507,7 @@ def test_dataclass_field_after_validator():
                 core_schema.dataclass_field(name='a', schema=core_schema.int_schema()),
                 core_schema.dataclass_field(
                     name='b',
-                    schema=core_schema.field_after_validator_function(Foo.validate_b, core_schema.str_schema()),
+                    schema=core_schema.field_after_validator_function(Foo.validate_b, 'b', core_schema.str_schema()),
                 ),
             ],
         ),
@@ -536,7 +539,7 @@ def test_dataclass_field_plain_validator():
             [
                 core_schema.dataclass_field(name='a', schema=core_schema.int_schema()),
                 core_schema.dataclass_field(
-                    name='b', schema=core_schema.field_plain_validator_function(Foo.validate_b)
+                    name='b', schema=core_schema.field_plain_validator_function(Foo.validate_b, 'b')
                 ),
             ],
         ),
@@ -569,7 +572,7 @@ def test_dataclass_field_before_validator():
                 core_schema.dataclass_field(name='a', schema=core_schema.int_schema()),
                 core_schema.dataclass_field(
                     name='b',
-                    schema=core_schema.field_before_validator_function(Foo.validate_b, core_schema.str_schema()),
+                    schema=core_schema.field_before_validator_function(Foo.validate_b, 'b', core_schema.str_schema()),
                 ),
             ],
         ),
@@ -605,7 +608,8 @@ def test_dataclass_field_wrap_validator1():
             [
                 core_schema.dataclass_field(name='a', schema=core_schema.int_schema()),
                 core_schema.dataclass_field(
-                    name='b', schema=core_schema.field_wrap_validator_function(Foo.validate_b, core_schema.str_schema())
+                    name='b',
+                    schema=core_schema.field_wrap_validator_function(Foo.validate_b, 'b', core_schema.str_schema()),
                 ),
             ],
         ),
@@ -639,7 +643,8 @@ def test_dataclass_field_wrap_validator2():
             [
                 core_schema.dataclass_field(name='a', schema=core_schema.int_schema()),
                 core_schema.dataclass_field(
-                    name='b', schema=core_schema.field_wrap_validator_function(Foo.validate_b, core_schema.str_schema())
+                    name='b',
+                    schema=core_schema.field_wrap_validator_function(Foo.validate_b, 'b', core_schema.str_schema()),
                 ),
             ],
         ),
@@ -865,7 +870,7 @@ def test_validate_assignment_function():
                 [
                     core_schema.dataclass_field('field_a', core_schema.str_schema()),
                     core_schema.dataclass_field(
-                        'field_b', core_schema.field_after_validator_function(func, core_schema.int_schema())
+                        'field_b', core_schema.field_after_validator_function(func, 'field_b', core_schema.int_schema())
                     ),
                     core_schema.dataclass_field('field_c', core_schema.int_schema()),
                 ],
@@ -878,14 +883,14 @@ def test_validate_assignment_function():
     assert m.field_a == 'x'
     assert m.field_b == 246
     assert m.field_c == 456
-    assert calls == ["ValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')"]
+    assert calls == ["FieldValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')"]
 
     v.validate_assignment(m, 'field_b', '111')
 
     assert m.field_b == 222
     assert calls == [
-        "ValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')",
-        "ValidationInfo(config=None, context=None, data={'field_a': 'x', 'field_c': 456}, field_name='field_b')",
+        "FieldValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')",
+        "FieldValidationInfo(config=None, context=None, data={'field_a': 'x', 'field_c': 456}, field_name='field_b')",
     ]
 
 
@@ -1280,7 +1285,7 @@ def test_dataclass_slots_field_before_validator():
                 core_schema.dataclass_field(name='a', schema=core_schema.int_schema()),
                 core_schema.dataclass_field(
                     name='b',
-                    schema=core_schema.field_before_validator_function(Foo.validate_b, core_schema.str_schema()),
+                    schema=core_schema.field_before_validator_function(Foo.validate_b, 'b', core_schema.str_schema()),
                 ),
             ],
         ),
@@ -1315,7 +1320,7 @@ def test_dataclass_slots_field_after_validator():
                 core_schema.dataclass_field(name='a', schema=core_schema.int_schema()),
                 core_schema.dataclass_field(
                     name='b',
-                    schema=core_schema.field_after_validator_function(Foo.validate_b, core_schema.str_schema()),
+                    schema=core_schema.field_after_validator_function(Foo.validate_b, 'b', core_schema.str_schema()),
                 ),
             ],
         ),
@@ -1488,3 +1493,58 @@ def test_dataclass_json():
             'type': 'dataclass_type',
         }
     ]
+
+
+@pytest.mark.xfail(
+    condition=platform.python_implementation() == 'PyPy', reason='https://foss.heptapod.net/pypy/pypy/-/issues/3899'
+)
+@pytest.mark.parametrize('validator', [None, 'field', 'dataclass'])
+def test_leak_dataclass(validator):
+    def fn():
+        @dataclasses.dataclass
+        class Dataclass:
+            a: int
+
+            @classmethod
+            def _validator(cls, v, info):
+                return v
+
+            @classmethod
+            def _wrap_validator(cls, v, validator, info):
+                return validator(v)
+
+        field_schema = core_schema.int_schema()
+        if validator == 'field':
+            field_schema = core_schema.field_before_validator_function(Dataclass._validator, 'a', field_schema)
+            field_schema = core_schema.field_wrap_validator_function(Dataclass._wrap_validator, 'a', field_schema)
+            field_schema = core_schema.field_after_validator_function(Dataclass._validator, 'a', field_schema)
+
+        dataclass_schema = core_schema.dataclass_schema(
+            Dataclass,
+            core_schema.dataclass_args_schema('Dataclass', [core_schema.dataclass_field('a', field_schema)]),
+            ['a'],
+        )
+
+        if validator == 'dataclass':
+            dataclass_schema = core_schema.general_before_validator_function(Dataclass._validator, dataclass_schema)
+            dataclass_schema = core_schema.general_wrap_validator_function(Dataclass._wrap_validator, dataclass_schema)
+            dataclass_schema = core_schema.general_after_validator_function(Dataclass._validator, dataclass_schema)
+
+        # If any of the Rust validators don't implement traversal properly,
+        # there will be an undetectable cycle created by this assignment
+        # which will keep Dataclass alive
+        Dataclass.__pydantic_validator__ = SchemaValidator(dataclass_schema)
+
+        return Dataclass
+
+    klass = fn()
+    ref = weakref.ref(klass)
+    assert ref() is not None
+
+    del klass
+    gc.collect(0)
+    gc.collect(1)
+    gc.collect(2)
+    gc.collect()
+
+    assert ref() is None
