@@ -1,9 +1,11 @@
 import copy
 import json
+import pickle
 import platform
 import re
 from datetime import date, datetime, time, timedelta, timezone, tzinfo
 from decimal import Decimal
+from typing import Dict
 
 import pytest
 import pytz
@@ -19,17 +21,17 @@ from ..conftest import Err, PyAndJson
         (datetime(2022, 6, 8, 12, 13, 14), datetime(2022, 6, 8, 12, 13, 14)),
         (date(2022, 6, 8), datetime(2022, 6, 8)),
         ('2022-06-08T12:13:14', datetime(2022, 6, 8, 12, 13, 14)),
-        ('1000000000000', datetime(2001, 9, 9, 1, 46, 40)),
+        ('1000000000000', datetime(2001, 9, 9, 1, 46, 40, tzinfo=timezone.utc)),
         (b'2022-06-08T12:13:14', datetime(2022, 6, 8, 12, 13, 14)),
         (b'2022-06-08T12:13:14Z', datetime(2022, 6, 8, 12, 13, 14, tzinfo=timezone.utc)),
         ((1,), Err('Input should be a valid datetime [type=datetime_type')),
         (time(1, 2, 3), Err('Input should be a valid datetime [type=datetime_type')),
-        (Decimal('1654646400'), datetime(2022, 6, 8)),
-        ('1654646400', datetime(2022, 6, 8)),
-        (Decimal('1654646400.123456'), datetime(2022, 6, 8, 0, 0, 0, 123456)),
-        (Decimal('1654646400.1234564'), datetime(2022, 6, 8, 0, 0, 0, 123456)),
-        (Decimal('1654646400.1234568'), datetime(2022, 6, 8, 0, 0, 0, 123457)),
-        ('1654646400.1234568', datetime(2022, 6, 8, 0, 0, 0, 123457)),
+        (Decimal('1654646400'), datetime(2022, 6, 8, tzinfo=timezone.utc)),
+        ('1654646400', datetime(2022, 6, 8, tzinfo=timezone.utc)),
+        (Decimal('1654646400.123456'), datetime(2022, 6, 8, 0, 0, 0, 123456, tzinfo=timezone.utc)),
+        (Decimal('1654646400.1234564'), datetime(2022, 6, 8, 0, 0, 0, 123456, tzinfo=timezone.utc)),
+        (Decimal('1654646400.1234568'), datetime(2022, 6, 8, 0, 0, 0, 123457, tzinfo=timezone.utc)),
+        ('1654646400.1234568', datetime(2022, 6, 8, 0, 0, 0, 123457, tzinfo=timezone.utc)),
         (253_402_300_800_000, Err('should be a valid datetime, dates after 9999 are not supported as unix timestamps')),
         (-20_000_000_000, Err('should be a valid datetime, dates before 1600 are not supported as unix timestamps')),
         (float('nan'), Err('Input should be a valid datetime, NaN values not permitted [type=datetime_parsing,')),
@@ -112,8 +114,8 @@ def test_keep_tz_bound():
             '2022-06-08T12:13:14+23:59',
             datetime(2022, 6, 8, 12, 13, 14, tzinfo=timezone(timedelta(hours=23, minutes=59))),
         ),
-        (1655205632, datetime(2022, 6, 14, 11, 20, 32)),
-        (1655205632.331557, datetime(2022, 6, 14, 11, 20, 32, microsecond=331557)),
+        (1655205632, datetime(2022, 6, 14, 11, 20, 32, tzinfo=timezone.utc)),
+        (1655205632.331557, datetime(2022, 6, 14, 11, 20, 32, microsecond=331557, tzinfo=timezone.utc)),
         (
             '2022-06-08T12:13:14+24:00',
             Err('Input should be a valid datetime, timezone offset must be less than 24 hours [type=datetime_parsing,'),
@@ -290,7 +292,7 @@ def test_invalid_constraint():
     [
         ('2022-06-08T12:13:14', datetime(2022, 6, 8, 12, 13, 14)),
         ('2022-06-08T12:13:14Z', datetime(2022, 6, 8, 12, 13, 14, tzinfo=timezone.utc)),
-        (1655205632, datetime(2022, 6, 14, 11, 20, 32)),
+        (1655205632, datetime(2022, 6, 14, 11, 20, 32, tzinfo=timezone.utc)),
         ('2068-06-08T12:13:14', Err('Input should be in the past [type=datetime_past,')),
         (3105730800, Err('Input should be in the past [type=datetime_past,')),
     ],
@@ -307,7 +309,7 @@ def test_datetime_past(py_and_json: PyAndJson, input_value, expected):
 
 def test_datetime_past_timezone():
     v = SchemaValidator(core_schema.datetime_schema(now_utc_offset=0, now_op='past'))
-    now_utc = datetime.utcnow().replace(tzinfo=timezone.utc) - timedelta(seconds=1)
+    now_utc = datetime.now(timezone.utc) - timedelta(seconds=1)
     assert v.isinstance_python(now_utc)
     # "later" in the day
     assert v.isinstance_python(now_utc.astimezone(pytz.timezone('Europe/Istanbul')))
@@ -333,7 +335,7 @@ def test_datetime_past_timezone():
     [
         ('2068-06-08T12:13:14', datetime(2068, 6, 8, 12, 13, 14)),
         ('2068-06-08T12:13:14Z', datetime(2068, 6, 8, 12, 13, 14, tzinfo=timezone.utc)),
-        (3105730800, datetime(2068, 5, 31, 23, 0)),
+        (3105730800, datetime(2068, 5, 31, 23, 0, tzinfo=timezone.utc)),
         ('2022-06-08T12:13:14', Err('Input should be in the future [type=datetime_future,')),
         (1655205632, Err('Input should be in the future [type=datetime_future,')),
     ],
@@ -350,7 +352,7 @@ def test_datetime_future(py_and_json: PyAndJson, input_value, expected):
 
 def test_datetime_future_timezone():
     v = SchemaValidator(core_schema.datetime_schema(now_utc_offset=0, now_op='future'))
-    now_utc = datetime.utcnow().replace(tzinfo=timezone.utc)
+    now_utc = datetime.now(timezone.utc)
 
     soon_utc = now_utc + timedelta(minutes=1)
     assert v.isinstance_python(soon_utc)
@@ -376,10 +378,10 @@ def test_mock_utc_offset_8_hours(mocker):
     """
     mocker.patch('time.localtime', return_value=type('time.struct_time', (), {'tm_gmtoff': 8 * 60 * 60}))
     v = SchemaValidator(core_schema.datetime_schema(now_op='future'))
-    future = datetime.utcnow() + timedelta(hours=8, minutes=1)
+    future = datetime.now(timezone.utc).replace(tzinfo=None) + timedelta(hours=8, minutes=1)
     assert v.isinstance_python(future)
 
-    future = datetime.utcnow() + timedelta(hours=7, minutes=59)
+    future = datetime.now(timezone.utc).replace(tzinfo=None) + timedelta(hours=7, minutes=59)
     assert not v.isinstance_python(future)
 
 
@@ -476,3 +478,38 @@ def test_tz_constraint_too_high():
 def test_tz_constraint_wrong():
     with pytest.raises(SchemaError, match="Input should be 'aware' or 'naive"):
         SchemaValidator(core_schema.datetime_schema(tz_constraint='wrong'))
+
+
+def test_tz_pickle() -> None:
+    """
+    https://github.com/pydantic/pydantic-core/issues/589
+    """
+    v = SchemaValidator(core_schema.datetime_schema())
+    original = datetime(2022, 6, 8, 12, 13, 14, tzinfo=timezone(timedelta(hours=-12, minutes=-15)))
+    validated = v.validate_python('2022-06-08T12:13:14-12:15')
+    assert validated == original
+    assert pickle.loads(pickle.dumps(validated)) == validated == original
+
+
+def test_tz_hash() -> None:
+    v = SchemaValidator(core_schema.datetime_schema())
+    lookup: Dict[datetime, str] = {}
+    for day in range(1, 10):
+        input_str = f'2022-06-{day:02}T12:13:14-12:15'
+        validated = v.validate_python(input_str)
+        lookup[validated] = input_str
+
+    assert len(lookup) == 9
+    assert (
+        lookup[datetime(2022, 6, 8, 12, 13, 14, tzinfo=timezone(timedelta(hours=-12, minutes=-15)))]
+        == '2022-06-08T12:13:14-12:15'
+    )
+
+
+def test_tz_cmp() -> None:
+    v = SchemaValidator(core_schema.datetime_schema())
+    validated1 = v.validate_python('2022-06-08T12:13:14-12:15')
+    validated2 = v.validate_python('2022-06-08T12:13:14-12:14')
+
+    assert validated1 > validated2
+    assert validated2 < validated1
