@@ -179,12 +179,12 @@ def test_float_repr():
     v = SchemaValidator({'type': 'float'})
     assert (
         plain_repr(v)
-        == 'SchemaValidator(title="float",validator=Float(FloatValidator{strict:false,allow_inf_nan:true}),definitions=[])'  # noqa: E501
+        == 'SchemaValidator(title="float",validator=Float(FloatValidator{strict:false,allow_inf_nan:true}),definitions=[])'
     )
     v = SchemaValidator({'type': 'float', 'strict': True})
     assert (
         plain_repr(v)
-        == 'SchemaValidator(title="float",validator=Float(FloatValidator{strict:true,allow_inf_nan:true}),definitions=[])'  # noqa: E501
+        == 'SchemaValidator(title="float",validator=Float(FloatValidator{strict:true,allow_inf_nan:true}),definitions=[])'
     )
     v = SchemaValidator({'type': 'float', 'multiple_of': 7})
     assert plain_repr(v).startswith('SchemaValidator(title="constrained-float",validator=ConstrainedFloat(')
@@ -325,3 +325,49 @@ def test_non_finite_constrained_float_values(input_value, allow_inf_nan, expecte
             v.validate_python(input_value)
     else:
         assert v.validate_python(input_value) == expected
+
+
+@pytest.mark.parametrize(
+    'input_value,expected',
+    [
+        # lower e, minus
+        ('1.0e-12', 1e-12),
+        ('1e-12', 1e-12),
+        ('12e-1', 12e-1),
+        # upper E, minus
+        ('1.0E-12', 1e-12),
+        ('1E-12', 1e-12),
+        ('12E-1', 12e-1),
+        # lower E, plus
+        ('1.0e+12', 1e12),
+        ('1e+12', 1e12),
+        ('12e+1', 12e1),
+        # upper E, plus
+        ('1.0E+12', 1e12),
+        ('1E+12', 1e12),
+        ('12E+1', 12e1),
+        # lower E, unsigned
+        ('1.0e12', 1e12),
+        ('1e12', 1e12),
+        ('12e1', 12e1),
+        # upper E, unsigned
+        ('1.0E12', 1e12),
+        ('1E12', 1e12),
+        ('12E1', 12e1),
+    ],
+)
+def test_validate_scientific_notation_from_json(input_value, expected):
+    v = SchemaValidator({'type': 'float'})
+    assert v.validate_json(input_value) == expected
+
+
+def test_string_with_underscores() -> None:
+    v = SchemaValidator({'type': 'float'})
+    assert v.validate_python('1_000_000.0') == 1_000_000.0
+    assert v.validate_json('"1_000_000.0"') == 1_000_000.0
+
+    for edge_case in ('_1', '_1.0', '1__0', '1.1__1', '1_0.0_', '1._', '1_0__0.0'):
+        with pytest.raises(ValidationError):
+            v.validate_python(edge_case)
+        with pytest.raises(ValidationError):
+            v.validate_json(f'"{edge_case}"')
