@@ -1,9 +1,10 @@
 import platform
 import re
 from copy import deepcopy
-from typing import Any, Dict, Type, Union
+from typing import Any, Dict, List, Type, Union
 
 import pytest
+from dirty_equals import HasRepr
 
 from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema
 
@@ -58,7 +59,7 @@ def test_function_before_raise():
             'loc': (),
             'msg': 'Value error, foobar',
             'input': 'input value',
-            'ctx': {'error': 'foobar'},
+            'ctx': {'error': HasRepr(repr(ValueError('foobar')))},
         }
     ]
 
@@ -350,7 +351,7 @@ def test_function_after_raise():
             'loc': (),
             'msg': 'Value error, foobar',
             'input': 'input value',
-            'ctx': {'error': 'foobar'},
+            'ctx': {'error': HasRepr(repr(ValueError('foobar')))},
         }
     ]
 
@@ -391,7 +392,7 @@ def test_function_after_config():
                     'type': 'typed-dict-field',
                     'schema': {
                         'type': 'function-after',
-                        'function': {'type': 'field', 'function': f},
+                        'function': {'type': 'field', 'function': f, 'field_name': 'test_field'},
                         'schema': {'type': 'str'},
                     },
                 }
@@ -460,6 +461,9 @@ def test_validate_assignment():
         __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
         field_a: str
 
+        def __init__(self):
+            self.__pydantic_extra__ = None  # this attribute must be present for validate_assignment
+
     v = SchemaValidator(
         core_schema.no_info_after_validator_function(
             f,
@@ -473,6 +477,7 @@ def test_validate_assignment():
     assert m.field_a == 'test'
     assert m.__pydantic_fields_set__ == {'field_a'}
     assert m.__dict__ == {'field_a': 'test', 'more': 'foobar'}
+    assert m.__pydantic_extra__ is None
 
     m2 = Model()
     m2.field_a = 'test'
@@ -551,7 +556,7 @@ def test_raise_assertion_error():
             'loc': (),
             'msg': 'Assertion failed, foobar',
             'input': 'input value',
-            'ctx': {'error': 'foobar'},
+            'ctx': {'error': HasRepr(repr(AssertionError('foobar')))},
         }
     ]
 
@@ -571,9 +576,9 @@ def test_raise_assertion_error_plain():
         {
             'type': 'assertion_error',
             'loc': (),
-            'msg': 'Assertion failed, Unknown error',
+            'msg': 'Assertion failed, ',
             'input': 'input value',
-            'ctx': {'error': 'Unknown error'},
+            'ctx': {'error': HasRepr(repr(AssertionError()))},
         }
     ]
 
@@ -614,8 +619,8 @@ def test_model_field_before_validator() -> None:
     def f(input_value: Any, info: core_schema.FieldValidationInfo) -> Any:
         assert info.field_name == 'x'
         assert info.data == {}
-        assert repr(info) == "ValidationInfo(config=None, context=None, data={}, field_name='x')"
-        assert str(info) == "ValidationInfo(config=None, context=None, data={}, field_name='x')"
+        assert repr(info) == "FieldValidationInfo(config=None, context=None, data={}, field_name='x')"
+        assert str(info) == "FieldValidationInfo(config=None, context=None, data={}, field_name='x')"
         assert isinstance(input_value, bytes)
         return f'input: {input_value.decode()}'
 
@@ -623,7 +628,11 @@ def test_model_field_before_validator() -> None:
         core_schema.model_schema(
             Model,
             core_schema.model_fields_schema(
-                {'x': core_schema.model_field(core_schema.field_before_validator_function(f, core_schema.str_schema()))}
+                {
+                    'x': core_schema.model_field(
+                        core_schema.field_before_validator_function(f, 'x', core_schema.str_schema())
+                    )
+                }
             ),
         )
     )
@@ -645,7 +654,11 @@ def test_model_field_after_validator() -> None:
         core_schema.model_schema(
             Model,
             core_schema.model_fields_schema(
-                {'x': core_schema.model_field(core_schema.field_after_validator_function(f, core_schema.str_schema()))}
+                {
+                    'x': core_schema.model_field(
+                        core_schema.field_after_validator_function(f, 'x', core_schema.str_schema())
+                    )
+                }
             ),
         )
     )
@@ -667,7 +680,7 @@ def test_model_field_plain_validator() -> None:
         core_schema.model_schema(
             Model,
             core_schema.model_fields_schema(
-                {'x': core_schema.model_field(core_schema.field_plain_validator_function(f))}
+                {'x': core_schema.model_field(core_schema.field_plain_validator_function(f, 'x'))}
             ),
         )
     )
@@ -691,7 +704,11 @@ def test_model_field_wrap_validator() -> None:
         core_schema.model_schema(
             Model,
             core_schema.model_fields_schema(
-                {'x': core_schema.model_field(core_schema.field_wrap_validator_function(f, core_schema.str_schema()))}
+                {
+                    'x': core_schema.model_field(
+                        core_schema.field_wrap_validator_function(f, 'x', core_schema.str_schema())
+                    )
+                }
             ),
         )
     )
@@ -799,19 +816,6 @@ def test_non_model_field_wrap_validator_tries_to_access_field_info() -> None:
     assert v.validate_python({'x': b'foo'}).x == 'input: foo'
 
 
-def test_method_function_no_model():
-    def f(*args: Any, **kwargs: Any) -> Any:  # pragma: no cover
-        raise AssertionError('Should not be called')
-
-    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'field', 'function': f}})
-
-    with pytest.raises(
-        RuntimeError,
-        match='This validator expected to be run inside the context of a model field but no model field was found',
-    ):
-        v.validate_python(123)
-
-
 def test_typed_dict_data() -> None:
     info_stuff = None
 
@@ -827,7 +831,7 @@ def test_typed_dict_data() -> None:
                 'a': core_schema.typed_dict_field(core_schema.int_schema()),
                 'b': core_schema.typed_dict_field(core_schema.int_schema()),
                 'c': core_schema.typed_dict_field(
-                    core_schema.field_after_validator_function(f, core_schema.str_schema())
+                    core_schema.field_after_validator_function(f, 'c', core_schema.str_schema())
                 ),
             }
         )
@@ -938,3 +942,32 @@ def test_function_validation_info_mode():
     assert v.validate_json('1') == 1
     assert calls == ['json']
     calls.clear()
+
+
+def test_reprs() -> None:
+    reprs: List[str] = []
+
+    def sample_repr(v: Any, info: core_schema.ValidationInfo) -> Any:
+        reprs.append(repr(info))
+        return v
+
+    v = SchemaValidator(
+        core_schema.chain_schema(
+            [
+                core_schema.general_plain_validator_function(sample_repr),
+                core_schema.field_plain_validator_function(sample_repr, field_name='x'),
+            ]
+        )
+    )
+
+    class Foo:
+        def __repr__(self) -> str:
+            return 'This is Foo!'
+
+    v.validate_python(Foo())
+
+    # insert_assert(reprs)
+    assert reprs == [
+        'ValidationInfo(config=None, context=None)',
+        "FieldValidationInfo(config=None, context=None, field_name='x')",
+    ]
