@@ -15,14 +15,8 @@ from ..conftest import Err, PyAndJson
         ([1, 2, '3'], [1, 2, 3]),
         ({1: 2, 3: 4}, [1, 3]),
         ('123', [1, 2, 3]),
-        (5, Err('Input should be iterable [type=iterable_type, input_value=5, input_type=int]')),
-        (
-            [1, 'wrong'],
-            Err(
-                'Input should be a valid integer, unable to parse string as an integer '
-                "[type=int_parsing, input_value='wrong', input_type=str]"
-            ),
-        ),
+        (5, Err('[type=iterable_type, input_value=5, input_type=int]')),
+        ([1, 'wrong'], Err("[type=int_parsing, input_value='wrong', input_type=str]")),
     ],
     ids=repr,
 )
@@ -36,6 +30,20 @@ def test_generator_json_int(py_and_json: PyAndJson, input_value, expected):
         assert list(v.validate_test(input_value)) == expected
 
 
+@pytest.mark.parametrize(
+    'config,input_str',
+    (
+        ({}, 'type=iterable_type, input_value=5, input_type=int'),
+        ({'hide_input_in_errors': False}, 'type=iterable_type, input_value=5, input_type=int'),
+        ({'hide_input_in_errors': True}, 'type=iterable_type'),
+    ),
+)
+def test_generator_json_hide_input(py_and_json: PyAndJson, config, input_str):
+    v = py_and_json({'type': 'generator', 'items_schema': {'type': 'int'}}, config)
+    with pytest.raises(ValidationError, match=re.escape(f'[{input_str}]')):
+        list(v.validate_test(5))
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
@@ -43,7 +51,7 @@ def test_generator_json_int(py_and_json: PyAndJson, input_value, expected):
         ([1, 2, '3'], [1, 2, '3']),
         ({'1': 2, '3': 4}, ['1', '3']),
         ('123', ['1', '2', '3']),
-        (5, Err('Input should be iterable [type=iterable_type, input_value=5, input_type=int]')),
+        (5, Err('[type=iterable_type, input_value=5, input_type=int]')),
         ([1, 'wrong'], [1, 'wrong']),
     ],
     ids=repr,
