@@ -6,7 +6,7 @@ from typing import Any, Dict
 import pytest
 from dirty_equals import IsStr
 
-from pydantic_core import SchemaValidator, ValidationError, __version__
+from pydantic_core import SchemaValidator, ValidationError
 
 from ..conftest import Err, PyAndJson, plain_repr
 
@@ -162,13 +162,7 @@ def test_negative_int(input_value, expected):
         (i64_max, i64_max),
         (i64_max + 1, i64_max + 1),
         (i64_max * 2, i64_max * 2),
-        (
-            int(1e30),
-            Err(
-                'Unable to parse input string as an integer, exceeded maximum size '
-                '[type=int_parsing_size, input_value=1e+30, input_type=float]'
-            ),
-        ),
+        (int(1e30), int(1e30)),
         (0, Err('Input should be greater than 0 [type=greater_than, input_value=0, input_type=int]')),
         (-1, Err('Input should be greater than 0 [type=greater_than, input_value=-1, input_type=int]')),
         pytest.param(
@@ -197,10 +191,7 @@ def test_positive_json(input_value, expected):
         (0, Err('Input should be less than 0 [type=less_than, input_value=0, input_type=int]')),
         (-i64_max, -i64_max),
         (-i64_max - 1, -i64_max - 1),
-        (
-            -i64_max * 2,
-            Err(' Unable to parse input string as an integer, exceeded maximum size [type=int_parsing_size'),
-        ),
+        (-i64_max * 2, -i64_max * 2),
     ],
 )
 def test_negative_json(input_value, expected):
@@ -340,7 +331,7 @@ def test_int_repr():
     assert plain_repr(v).startswith('SchemaValidator(title="constrained-int",validator=ConstrainedInt(')
 
 
-def test_too_long():
+def test_too_long(pydantic_version):
     v = SchemaValidator({'type': 'int'})
 
     with pytest.raises(ValidationError) as exc_info:
@@ -359,7 +350,7 @@ def test_too_long():
         "1 validation error for int\n"
         "  Unable to parse input string as an integer, exceeded maximum size "
         "[type=int_parsing_size, input_value='111111111111111111111111...11111111111111111111111', input_type=str]\n"
-        f"    For further information visit https://errors.pydantic.dev/{__version__}/v/int_parsing_size"
+        f"    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/int_parsing_size"
     )
 
 
@@ -391,8 +382,7 @@ def test_long_python_inequality():
 def test_long_json():
     v = SchemaValidator({'type': 'int'})
 
-    with pytest.raises(ValidationError, match=r'number out of range at line 1 column 401 \[type=json_invalid,'):
-        v.validate_json('-' + '1' * 400)
+    assert v.validate_json('-' + '1' * 400) == int('-' + '1' * 400)
 
     with pytest.raises(ValidationError, match=r'expected ident at line 1 column 2 \[type=json_invalid,'):
         v.validate_json('nan')
@@ -403,3 +393,15 @@ def test_int_key(py_and_json: PyAndJson):
     assert v.validate_test({'1': 1, '2': 2}) == {1: 1, 2: 2}
     with pytest.raises(ValidationError, match='Input should be a valid integer'):
         v.validate_test({'1': 1, '2': 2}, strict=True)
+
+
+def test_string_as_int_with_underscores() -> None:
+    v = SchemaValidator({'type': 'int'})
+    assert v.validate_python('1_000_000') == 1_000_000
+    assert v.validate_json('"1_000_000"') == 1_000_000
+
+    for edge_case in ('_1', '1__0', '1_0_', '1_0__0'):
+        with pytest.raises(ValidationError):
+            v.validate_python(edge_case)
+        with pytest.raises(ValidationError):
+            v.validate_json(f'"{edge_case}"')
