@@ -3,7 +3,7 @@ from copy import deepcopy
 from typing import Any, Callable, Dict, List, Set, Tuple
 
 import pytest
-from dirty_equals import IsInstance
+from dirty_equals import HasRepr, IsInstance
 
 from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema
 
@@ -139,7 +139,7 @@ def test_model_class_root_validator_wrap():
             'loc': (),
             'msg': 'Assertion failed, assert 456 == 123',
             'input': {'field_a': 456},
-            'ctx': {'error': 'assert 456 == 123'},
+            'ctx': {'error': HasRepr(repr(AssertionError('assert 456 == 123')))},
         }
     ]
 
@@ -173,7 +173,7 @@ def test_model_class_root_validator_before():
             'loc': (),
             'msg': 'Assertion failed, assert 456 == 123',
             'input': {'field_a': 456},
-            'ctx': {'error': 'assert 456 == 123'},
+            'ctx': {'error': HasRepr(repr(AssertionError('assert 456 == 123')))},
         }
     ]
 
@@ -208,7 +208,7 @@ def test_model_class_root_validator_after():
             'loc': (),
             'msg': 'Assertion failed, assert 456 == 123',
             'input': {'field_a': 456},
-            'ctx': {'error': 'assert 456 == 123'},
+            'ctx': {'error': HasRepr(repr(AssertionError('assert 456 == 123')))},
         }
     ]
 
@@ -876,7 +876,7 @@ def test_post_init_validation_error():
             'loc': (),
             'msg': 'Value error, this is broken: test',
             'input': {'field_a': 'test'},
-            'ctx': {'error': 'this is broken: test'},
+            'ctx': {'error': HasRepr(repr(ValueError('this is broken: test')))},
         }
     ]
 
@@ -943,6 +943,9 @@ def test_validate_assignment():
         field_a: str
         field_b: int
 
+        def __init__(self):
+            self.__pydantic_extra__ = None
+
     v = SchemaValidator(
         {
             'type': 'model',
@@ -993,7 +996,7 @@ def test_validate_assignment_function():
                 {
                     'field_a': core_schema.model_field(core_schema.str_schema()),
                     'field_b': core_schema.model_field(
-                        core_schema.field_after_validator_function(func, core_schema.int_schema())
+                        core_schema.field_after_validator_function(func, 'field_b', core_schema.int_schema())
                     ),
                     'field_c': core_schema.model_field(core_schema.int_schema()),
                 }
@@ -1006,20 +1009,23 @@ def test_validate_assignment_function():
     assert m.field_b == 246
     assert m.field_c == 456
     assert m.__pydantic_fields_set__ == {'field_a', 'field_b', 'field_c'}
-    assert calls == ["ValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')"]
+    assert calls == ["FieldValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')"]
 
     v.validate_assignment(m, 'field_b', '111')
 
     assert m.field_b == 222
     assert calls == [
-        "ValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')",
-        "ValidationInfo(config=None, context=None, data={'field_a': 'x', 'field_c': 456}, field_name='field_b')",
+        "FieldValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')",
+        "FieldValidationInfo(config=None, context=None, data={'field_a': 'x', 'field_c': 456}, field_name='field_b')",
     ]
 
 
 def test_validate_assignment_no_fields_set():
     class MyModel:
-        __slots__ = ('__dict__',)
+        __slots__ = ('__dict__', '__pydantic_extra__')
+
+        def __init__(self):
+            self.__pydantic_extra__ = None
 
     v = SchemaValidator(
         {
