@@ -8,7 +8,7 @@ from typing import Any, Dict, List, Mapping, Union
 import pytest
 from dirty_equals import FunctionCheck, HasRepr, IsStr
 
-from pydantic_core import CoreConfig, SchemaError, SchemaValidator, ValidationError, __version__, core_schema
+from pydantic_core import CoreConfig, SchemaError, SchemaValidator, ValidationError, core_schema
 
 from ..conftest import Err, PyAndJson
 
@@ -105,7 +105,7 @@ def test_with_default():
     )
 
 
-def test_missing_error():
+def test_missing_error(pydantic_version):
     v = SchemaValidator(
         {
             'type': 'model-fields',
@@ -123,7 +123,7 @@ def test_missing_error():
 1 validation error for model-fields
 field_b
   Field required [type=missing, input_value={{'field_a': b'abc'}}, input_type=dict]
-    For further information visit https://errors.pydantic.dev/{__version__}/v/missing"""
+    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/missing"""
     )
 
 
@@ -347,8 +347,8 @@ def test_validate_assignment_allow_extra():
     assert v.validate_python({'field_a': 'test'}) == ({'field_a': 'test'}, {}, {'field_a'})
 
     assert v.validate_assignment({'field_a': 'test'}, 'other_field', 456) == (
-        {'field_a': 'test', 'other_field': 456},
-        None,
+        {'field_a': 'test'},
+        {'other_field': 456},
         {'other_field'},
     )
 
@@ -364,8 +364,8 @@ def test_validate_assignment_allow_extra_validate():
     )
 
     assert v.validate_assignment({'field_a': 'test'}, 'other_field', '456') == (
-        {'field_a': 'test', 'other_field': 456},
-        None,
+        {'field_a': 'test'},
+        {'other_field': 456},
         {'other_field'},
     )
 
@@ -1445,7 +1445,7 @@ def test_alias_extra_forbid(py_and_json: PyAndJson):
             'fields': {'field_a': {'type': 'model-field', 'validation_alias': 'FieldA', 'schema': {'type': 'int'}}},
         }
     )
-    assert v.validate_test({'FieldA': 1}) == ({'field_a': 1}, {}, {'field_a'})
+    assert v.validate_test({'FieldA': 1}) == ({'field_a': 1}, None, {'field_a'})
 
 
 def test_with_default_factory():
@@ -1682,10 +1682,12 @@ def test_extra_behavior_allow(
     assert fields_set == {'f', 'extra_field'}
 
     v.validate_assignment(m, 'f', 'y')
-    assert m['f'] == 'y'
+    assert m == {'f': 'y'}
 
-    v.validate_assignment(m, 'not_f', '123')
-    assert m['not_f'] == expected_extra_value
+    new_m, new_model_extra, new_fields_set = v.validate_assignment({**m, **model_extra}, 'not_f', '123')
+    assert new_m == {'f': 'y'}
+    assert new_model_extra == {'extra_field': expected_extra_value, 'not_f': expected_extra_value}
+    assert new_fields_set == {'not_f'}
 
 
 @pytest.mark.parametrize(
