@@ -22,14 +22,14 @@ from ..conftest import Err, PyAndJson
         pytest.param((1,), Err('Input should be a valid time [type=time_type'), id='tuple'),
         pytest.param(date(2022, 6, 8), Err('Input should be a valid time [type=time_type'), id='date'),
         pytest.param(datetime(2022, 6, 8), Err('Input should be a valid time [type=time_type'), id='datetime'),
-        pytest.param(123, time(0, 2, 3), id='int'),
+        pytest.param(123, time(0, 2, 3, tzinfo=timezone.utc), id='int'),
         pytest.param(float('nan'), Err('valid time format, NaN values not permitted [type=time_parsing,'), id='nan'),
         pytest.param(float('inf'), Err('valid time format, numeric times may not exceed 86,399 seconds'), id='inf'),
         pytest.param(float('-inf'), Err('valid time format, time in seconds should be positive'), id='-inf'),
-        pytest.param(Decimal('123'), time(0, 2, 3), id='decimal'),
-        pytest.param(Decimal('123.123456'), time(0, 2, 3, 123456), id='decimal-6dig'),
-        pytest.param(Decimal('123.1234562'), time(0, 2, 3, 123456), id='decimal-7dig-up'),
-        pytest.param(Decimal('123.1234568'), time(0, 2, 3, 123457), id='decimal-7dig-down'),
+        pytest.param(Decimal('123'), time(0, 2, 3, tzinfo=timezone.utc), id='decimal'),
+        pytest.param(Decimal('123.123456'), time(0, 2, 3, 123456, tzinfo=timezone.utc), id='decimal-6dig'),
+        pytest.param(Decimal('123.1234562'), time(0, 2, 3, 123456, tzinfo=timezone.utc), id='decimal-7dig-up'),
+        pytest.param(Decimal('123.1234568'), time(0, 2, 3, 123457, tzinfo=timezone.utc), id='decimal-7dig-down'),
     ],
 )
 def test_time(input_value, expected):
@@ -49,18 +49,11 @@ def test_time(input_value, expected):
         pytest.param('12:13:14.123', time(12, 13, 14, 123_000), id='str-micro'),
         pytest.param('12:13:14.123456', time(12, 13, 14, 123_456), id='str-micro-6dig'),
         pytest.param('12:13:14.123456', time(12, 13, 14, 123_456), id='str-micro-6dig'),
-        pytest.param(
-            '12:13:14.1234561',
-            Err(
-                'Input should be in a valid time format, '
-                'second fraction value is more than 6 digits long [type=time_parsing'
-            ),
-            id='str-micro-7dig',
-        ),
-        pytest.param(123, time(0, 2, 3), id='int'),
-        pytest.param(123.4, time(0, 2, 3, 400_000), id='float'),
-        pytest.param(123.0, time(0, 2, 3), id='float.0'),
-        pytest.param(0, time(0), id='int-zero'),
+        pytest.param('12:13:14.1234561', time(12, 13, 14, 123_456), id='str-micro-7dig'),
+        pytest.param(123, time(0, 2, 3, tzinfo=timezone.utc), id='int'),
+        pytest.param(123.4, time(0, 2, 3, 400_000, tzinfo=timezone.utc), id='float'),
+        pytest.param(123.0, time(0, 2, 3, tzinfo=timezone.utc), id='float.0'),
+        pytest.param(0, time(0, tzinfo=timezone.utc), id='int-zero'),
         pytest.param(
             86400,
             Err(
@@ -87,6 +80,27 @@ def test_time_json(py_and_json: PyAndJson, input_value, expected):
         assert output == expected
 
 
+def test_time_error_microseconds_overflow(py_and_json: PyAndJson) -> None:
+    v = py_and_json(core_schema.time_schema(microseconds_precision='error'))
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_test('00:00:00.1234567')
+
+    # insert_assert(exc_info.value.errors(include_url=False))
+    assert exc_info.value.errors(include_url=False) == [
+        {
+            'type': 'time_parsing',
+            'loc': (),
+            'msg': 'Input should be in a valid time format, second fraction value is more than 6 digits long',
+            'input': '00:00:00.1234567',
+            'ctx': {'error': 'second fraction value is more than 6 digits long'},
+        }
+    ]
+
+    # insert_assert(v.validate_test('00:00:00.123456'))
+    assert v.validate_test('00:00:00.123456') == time(0, 0, 0, 123456)
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
