@@ -9,6 +9,11 @@ from pydantic_core import SchemaError, SchemaValidator, ValidationError
 
 from ..conftest import Err, PyAndJson
 
+try:
+    import pandas
+except ImportError:
+    pandas = None
+
 
 @pytest.mark.parametrize(
     'input_value,expected',
@@ -131,27 +136,35 @@ def test_timedelta_strict_json(input_value, expected):
         ({}, 'P0Y0M3D2WT1H2M3S', timedelta(days=3, weeks=2, hours=1, minutes=2, seconds=3)),
         ({'le': timedelta(days=3)}, 'P2DT1H', timedelta(days=2, hours=1)),
         ({'le': timedelta(days=3)}, 'P3DT0H', timedelta(days=3)),
-        ({'le': timedelta(days=3)}, 'P3DT1H', Err('Input should be less than or equal to P3D')),
+        ({'le': timedelta(days=3)}, 'P3DT1H', Err('Input should be less than or equal to datetime.timedelta(days=3)')),
         ({'lt': timedelta(days=3)}, 'P2DT1H', timedelta(days=2, hours=1)),
-        ({'lt': timedelta(days=3)}, 'P3DT1H', Err('Input should be less than P3D')),
+        ({'lt': timedelta(days=3)}, 'P3DT1H', Err('Input should be less than datetime.timedelta(days=3)')),
         ({'ge': timedelta(days=3)}, 'P3DT1H', timedelta(days=3, hours=1)),
         ({'ge': timedelta(days=3)}, 'P3D', timedelta(days=3)),
-        ({'ge': timedelta(days=3)}, 'P2DT1H', Err('Input should be greater than or equal to P3D')),
+        (
+            {'ge': timedelta(days=3)},
+            'P2DT1H',
+            Err('Input should be greater than or equal to datetime.timedelta(days=3)'),
+        ),
         ({'gt': timedelta(days=3)}, 'P3DT1H', timedelta(days=3, hours=1)),
-        ({'gt': 'P3D'}, 'P2DT1H', Err('Input should be greater than P3D')),
+        ({'gt': 'P3D'}, 'P2DT1H', Err('Input should be greater than datetime.timedelta(days=3)')),
         ({'le': timedelta(seconds=-86400.123)}, '-PT86400.123S', timedelta(seconds=-86400.123)),
         ({'le': timedelta(seconds=-86400.123)}, '-PT86400.124S', timedelta(seconds=-86400.124)),
         (
             {'le': timedelta(seconds=-86400.123)},
             '-PT86400.122S',
-            Err('Input should be less than or equal to -P1DT0.123S [type=less_than_equal'),
+            Err(
+                'Input should be less than or equal to datetime.timedelta(days=-2, seconds=86399, microseconds=877000) [type=less_than_equal'
+            ),
         ),
         ({'gt': timedelta(seconds=-86400.123)}, timedelta(seconds=-86400.122), timedelta(seconds=-86400.122)),
         ({'gt': timedelta(seconds=-86400.123)}, '-PT86400.122S', timedelta(seconds=-86400.122)),
         (
             {'gt': timedelta(seconds=-86400.123)},
             '-PT86400.124S',
-            Err('Input should be greater than -P1DT0.123S [type=greater_than'),
+            Err(
+                'Input should be greater than datetime.timedelta(days=-2, seconds=86399, microseconds=877000) [type=greater_than'
+            ),
         ),
     ],
     ids=repr,
@@ -251,3 +264,19 @@ def test_large_value():
     assert v.validate_python(f'{999_999_999}days, 12:34') == timedelta(days=999_999_999, hours=12, minutes=34)
     with pytest.raises(ValidationError, match='should be a valid timedelta, durations may not exceed 999,999,999 days'):
         v.validate_python(f'{999_999_999 + 1}days, 12:34')
+
+
+@pytest.mark.skipif(not pandas, reason='pandas not installed')
+def test_pandas():
+    v = SchemaValidator({'type': 'timedelta', 'ge': timedelta(hours=2)})
+    two_hours = pandas.Timestamp('2023-01-01T02:00:00Z') - pandas.Timestamp('2023-01-01T00:00:00Z')
+
+    assert v.validate_python(two_hours) == two_hours
+    assert v.validate_python(two_hours.to_pytimedelta()) == two_hours
+
+    one_55 = pandas.Timestamp('2023-01-01T01:55:00Z') - pandas.Timestamp('2023-01-01T00:00:00Z')
+    msg = r'Input should be greater than or equal to datetime.timedelta\(seconds=7200\)'
+    with pytest.raises(ValidationError, match=msg):
+        v.validate_python(one_55)
+    with pytest.raises(ValidationError, match=msg):
+        v.validate_python(one_55.to_pytimedelta())
