@@ -1,7 +1,7 @@
 import pytest
 from dirty_equals import IsFloat, IsInt
 
-from pydantic_core import SchemaError, SchemaValidator, ValidationError, __version__, core_schema
+from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema
 
 from ..conftest import plain_repr
 
@@ -232,7 +232,7 @@ def test_union_list_bool_int():
     ]
 
 
-def test_no_choices():
+def test_no_choices(pydantic_version):
     with pytest.raises(SchemaError) as exc_info:
         SchemaValidator({'type': 'union'})
 
@@ -240,7 +240,7 @@ def test_no_choices():
         'Invalid Schema:\n'
         'union.choices\n'
         "  Field required [type=missing, input_value={'type': 'union'}, input_type=dict]\n"
-        f'    For further information visit https://errors.pydantic.dev/{__version__}/v/missing'
+        f'    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/missing'
     )
     assert exc_info.value.error_count() == 1
     assert exc_info.value.errors() == [
@@ -411,3 +411,85 @@ def test_strict_reference():
 
     assert repr(v.validate_python((1, 2))) == '(1.0, 2)'
     assert repr(v.validate_python((1.0, (2.0, 3)))) == '(1.0, (2.0, 3))'
+
+
+def test_case_labels():
+    v = SchemaValidator(
+        {'type': 'union', 'choices': [{'type': 'none'}, ({'type': 'int'}, 'my_label'), {'type': 'str'}]}
+    )
+    assert v.validate_python(None) is None
+    assert v.validate_python(1) == 1
+    with pytest.raises(ValidationError, match=r'3 validation errors for union\[none,my_label,str]') as exc_info:
+        v.validate_python(1.5)
+    assert exc_info.value.errors(include_url=False) == [
+        {'input': 1.5, 'loc': ('none',), 'msg': 'Input should be None', 'type': 'none_required'},
+        {
+            'input': 1.5,
+            'loc': ('my_label',),
+            'msg': 'Input should be a valid integer, got a number with a fractional part',
+            'type': 'int_from_float',
+        },
+        {'input': 1.5, 'loc': ('str',), 'msg': 'Input should be a valid string', 'type': 'string_type'},
+    ]
+
+
+def test_left_to_right_doesnt_care_about_strict_check():
+    v = SchemaValidator(
+        core_schema.union_schema([core_schema.int_schema(), core_schema.json_schema()], mode='left_to_right')
+    )
+    assert 'strict_required' not in plain_repr(v)
+    assert 'ultra_strict_required' not in plain_repr(v)
+
+
+def test_left_to_right_union():
+    choices = [core_schema.int_schema(), core_schema.float_schema()]
+
+    # smart union prefers float
+    v = SchemaValidator(core_schema.union_schema(choices, mode='smart'))
+    out = v.validate_python(1.0)
+    assert out == 1.0
+    assert isinstance(out, float)
+
+    # left_to_right union will select int
+    v = SchemaValidator(core_schema.union_schema(choices, mode='left_to_right'))
+    out = v.validate_python(1)
+    assert out == 1
+    assert isinstance(out, int)
+
+    out = v.validate_python(1.0)
+    assert out == 1
+    assert isinstance(out, int)
+
+    # reversing them will select float
+    v = SchemaValidator(core_schema.union_schema(list(reversed(choices)), mode='left_to_right'))
+    out = v.validate_python(1.0)
+    assert out == 1.0
+    assert isinstance(out, float)
+
+    out = v.validate_python(1)
+    assert out == 1.0
+    assert isinstance(out, float)
+
+
+def test_left_to_right_union_strict():
+    choices = [core_schema.int_schema(), core_schema.float_schema()]
+
+    # left_to_right union will select not cast if int first (strict int will not accept float)
+    v = SchemaValidator(core_schema.union_schema(choices, mode='left_to_right', strict=True))
+    out = v.validate_python(1)
+    assert out == 1
+    assert isinstance(out, int)
+
+    out = v.validate_python(1.0)
+    assert out == 1.0
+    assert isinstance(out, float)
+
+    # reversing union will select float always (as strict float will accept int)
+    v = SchemaValidator(core_schema.union_schema(list(reversed(choices)), mode='left_to_right', strict=True))
+    out = v.validate_python(1.0)
+    assert out == 1.0
+    assert isinstance(out, float)
+
+    out = v.validate_python(1)
+    assert out == 1.0
+    assert isinstance(out, float)
